@@ -25,6 +25,8 @@
 #include <time.h>
 #include <cerrno>
 #include <fstream>
+#include <stdlib.h>
+#include <cstring>
 
 #include <iostream>
 
@@ -294,7 +296,7 @@ ARDOUR_UI::create_engine ()
 
 	engine->Stopped.connect (mem_fun(*this, &ARDOUR_UI::engine_stopped));
 	engine->Running.connect (mem_fun(*this, &ARDOUR_UI::engine_running));
-	engine->Halted.connect (mem_fun(*this, &ARDOUR_UI::engine_halted));
+	engine->Halted.connect (bind (mem_fun(*this, &ARDOUR_UI::engine_halted), false));
 	engine->SampleRateChanged.connect (mem_fun(*this, &ARDOUR_UI::update_sample_rate));
 
 	post_engine ();
@@ -569,7 +571,7 @@ ARDOUR_UI::update_autosave ()
 {
         ENSURE_GUI_THREAD (mem_fun (*this, &ARDOUR_UI::update_autosave));
         
-        if (session->dirty()) {
+        if (session && session->dirty()) {
                 if (_autosave_connection.connected()) {
                         _autosave_connection.disconnect();
                 }
@@ -728,9 +730,12 @@ ARDOUR_UI::finish()
 {
 	if (session) {
 
-		if (session->transport_rolling()) {
-			session->request_stop ();
-			usleep (2500000);
+		int tries = 0;
+
+		while (session->transport_rolling() && (++tries < 8)) {
+			/* stop but do not abort capture */
+			session->request_stop (false, true);
+			usleep (10000);
 		}
 
 		if (session->dirty()) {
@@ -1160,6 +1165,7 @@ ARDOUR_UI::check_audioengine ()
 			MessageDialog msg (_("Ardour is not connected to JACK\n"
 					     "You cannot open or close sessions in this condition"));
 			pop_back_splash ();
+			msg.set_position (WIN_POS_CENTER);
 			msg.run ();
 			return false;
 		}
@@ -1182,7 +1188,7 @@ ARDOUR_UI::open_session ()
 
 		/* ardour sessions are folders */
 
-		open_session_selector = new Gtk::FileChooserDialog (_("open session"), FILE_CHOOSER_ACTION_OPEN);
+		open_session_selector = new Gtk::FileChooserDialog (_("Open Session"), FILE_CHOOSER_ACTION_OPEN);
 		open_session_selector->add_button (Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
 		open_session_selector->add_button (Gtk::Stock::OPEN, Gtk::RESPONSE_ACCEPT);
 		open_session_selector->set_default_response(Gtk::RESPONSE_ACCEPT);
@@ -1396,18 +1402,14 @@ ARDOUR_UI::transport_stop ()
 		return;
 	}
 	
-	if (session->get_play_loop ()) {
-		session->request_play_loop (false);
-	}
-	
-	session->request_stop ();
+	session->request_stop (false, true);
 }
 
 void
 ARDOUR_UI::transport_stop_and_forget_capture ()
 {
 	if (session) {
-		session->request_stop (true);
+		session->request_stop (true, true);
 	}
 }
 
@@ -1451,35 +1453,107 @@ ARDOUR_UI::transport_record (bool roll)
 	//cerr << "ARDOUR_UI::transport_record () called roll = " << roll << " session->record_status() = " << session->record_status() << endl;
 }
 
-void
+void 
 ARDOUR_UI::transport_roll ()
 {
-	bool rolling;
-
 	if (!session) {
 		return;
 	}
 
-	rolling = session->transport_rolling ();
-
-	//cerr << "ARDOUR_UI::transport_roll () called session->record_status() = " << session->record_status() << endl;
-
-	if (session->get_play_loop()) {
-		session->request_play_loop (false);
-		auto_loop_button.set_visual_state (1);
-		roll_button.set_visual_state (1);
-	} else if (session->get_play_range ()) {
-		session->request_play_range (false);
-		play_selection_button.set_visual_state (0);
-	} else if (rolling) {
-		session->request_locate (session->last_transport_start(), true);
+	if (session->is_auditioning()) {
+		return;
+	}
+	
+	switch (Config->get_slave_source()) {
+	case None:
+	case JACK:
+		break;
+	default:
+		/* transport controlled by the master */
+		return;
 	}
 
-	session->request_transport_speed (1.0f);
+	bool rolling = session->transport_rolling();
+
+	if (session->get_play_loop()) {
+		/* XXX it is not possible to just leave seamless loop and keep
+		   playing at present (nov 4th 2009)
+		*/
+		if (!Config->get_seamless_loop()) {
+			/* off, and stop */
+			session->request_play_loop (false, true);
+		} else {
+			return;
+		}
+	} else if (session->get_play_range ()) {
+		session->request_play_range (0, true);
+	} 
+
+	if (!rolling) {
+		session->request_transport_speed (1.0f);
+	}
+
+	map_transport_state ();
 }
 
 void
-ARDOUR_UI::transport_loop()
+ARDOUR_UI::toggle_roll (bool with_abort, bool roll_out_of_bounded_mode)
+{
+	
+	if (!session) {
+		return;
+	}
+
+	if (session->is_auditioning()) {
+		session->cancel_audition ();
+		return;
+	}
+	
+	switch (Config->get_slave_source()) {
+	case None:
+	case JACK:
+		break;
+	default:
+		/* transport controlled by the master */
+		return;
+	}
+
+	bool rolling = session->transport_rolling();
+	bool affect_transport = true;
+
+	if (rolling && roll_out_of_bounded_mode) {
+		/* drop out of loop/range playback but leave transport rolling */
+		if (session->get_play_loop()) {
+			if (Config->get_seamless_loop()) {
+				/* the disk buffers contain copies of the loop - we can't 
+				   just keep playing, so stop the transport. the user
+				   can restart as they wish.
+				*/
+				affect_transport = true;
+			} else {
+				/* disk buffers are normal, so we can keep playing */
+				affect_transport = false;
+			}
+			session->request_play_loop (false, true);
+		} else if (session->get_play_range ()) {
+			affect_transport = false;
+			session->request_play_range (0, true);
+		} 
+	} 
+
+	if (affect_transport) {
+		if (rolling) {
+			session->request_stop (with_abort, true);
+		} else {
+			session->request_transport_speed (1.0f);
+		}
+	}
+
+	map_transport_state ();
+}
+
+void
+ARDOUR_UI::toggle_session_auto_loop ()
 {
 	if (session) {
 		if (session->get_play_loop()) {
@@ -1488,10 +1562,14 @@ ARDOUR_UI::transport_loop()
 				if (looploc) {
 					session->request_locate (looploc->start(), true);
 				}
+			} else {
+				session->request_play_loop (false);
+			}
+		} else {
+			Location * looploc = session->locations()->auto_loop_location();
+			if (looploc) {
+				session->request_play_loop (true);
 			}
-		}
-		else {
-			session->request_play_loop (true);
 		}
 	}
 }
@@ -1503,10 +1581,6 @@ ARDOUR_UI::transport_play_selection ()
 		return;
 	}
 
-	if (!session->get_play_range()) {
-		session->request_stop ();
-	}
-
 	editor->play_selection ();
 }
 
@@ -1595,17 +1669,56 @@ ARDOUR_UI::queue_transport_change ()
 void
 ARDOUR_UI::map_transport_state ()
 {
+	if (!session) {
+		auto_loop_button.set_visual_state (0);
+		play_selection_button.set_visual_state (0);
+		roll_button.set_visual_state (0);
+		stop_button.set_visual_state (1);
+		return;
+	}
+
 	float sp = session->transport_speed();
 
 	if (sp == 1.0f) {
-		transport_rolling ();
-	} else if (sp < 0.0f) {
-		transport_rewinding ();
-	} else if (sp > 0.0f) {
-		transport_forwarding ();
-	} else {
-		transport_stopped ();
+		shuttle_fract = SHUTTLE_FRACT_SPEED1;  /* speed = 1.0, believe it or not */
+		shuttle_box.queue_draw ();
+	} else if (sp == 0.0f) {
+		shuttle_fract = 0;
+		shuttle_box.queue_draw ();
+		update_disk_space ();
 	}
+
+	if (sp != 0.0) {
+
+		if (session->get_play_range()) {
+
+			play_selection_button.set_visual_state (1);
+			roll_button.set_visual_state (0);
+			auto_loop_button.set_visual_state (0);
+			
+		} else if (session->get_play_loop ()) {
+			
+			auto_loop_button.set_visual_state (1);
+			play_selection_button.set_visual_state (0);
+			roll_button.set_visual_state (0);
+
+		} else {
+			
+			roll_button.set_visual_state (1);
+			play_selection_button.set_visual_state (0);
+			auto_loop_button.set_visual_state (0);
+		}
+
+		stop_button.set_visual_state (0);
+
+	} else {
+
+		stop_button.set_visual_state (1);
+		roll_button.set_visual_state (0);
+		play_selection_button.set_visual_state (0);
+		auto_loop_button.set_visual_state (0);
+	}
+
 }
 
 void
@@ -1675,23 +1788,46 @@ ARDOUR_UI::engine_running ()
 }
 
 void
-ARDOUR_UI::engine_halted ()
+ARDOUR_UI::engine_halted (const char* reason, bool free_reason)
 {
-	ENSURE_GUI_THREAD (mem_fun(*this, &ARDOUR_UI::engine_halted));
+	if (!Gtkmm2ext::UI::instance()->caller_is_ui_thread()) {
+		/* we can't rely on the original string continuing to exist when we are called
+		   again in the GUI thread, so make a copy and note that we need to
+		   free it later.
+		*/
+		char *copy = strdup (reason);
+		Gtkmm2ext::UI::instance()->call_slot (bind (mem_fun (*this, &ARDOUR_UI::engine_halted), copy, true));
+		return;
+	} 
 
 	ActionManager::set_sensitive (ActionManager::jack_sensitive_actions, false);
 	ActionManager::set_sensitive (ActionManager::jack_opposite_sensitive_actions, true);
 
 	update_sample_rate (0);
 
-	MessageDialog msg (*editor, 
-			   _("\
+	string msgstr;
+
+	/* if the reason is a non-empty string, it means that the backend was shutdown
+	   rather than just Ardour.
+	*/
+
+	if (strlen (reason)) {
+		msgstr = string_compose (_("The audio backend (JACK) was shutdown because:\n\n%1"), reason);
+	} else {
+		msgstr = _("\
 JACK has either been shutdown or it\n\
 disconnected Ardour because Ardour\n\
 was not fast enough. Try to restart\n\
-JACK, reconnect and save the session."));
+JACK, reconnect and save the session.");
+	}
+
+	MessageDialog msg (*editor, msgstr);
 	pop_back_splash ();
 	msg.run ();
+
+	if (free_reason) {
+		free ((char *) reason);
+	}
 }
 
 int32_t
@@ -1834,22 +1970,27 @@ ARDOUR_UI::name_io_setup (AudioEngine& engine,
 /** Ask the user for the name of a new shapshot and then take it.
  */
 void
-ARDOUR_UI::snapshot_session ()
+ARDOUR_UI::snapshot_session (bool switch_to_it)
 {
 	ArdourPrompter prompter (true);
 	string snapname;
-	char timebuf[128];
-	time_t n;
-	struct tm local_time;
-
-	time (&n);
-	localtime_r (&n, &local_time);
-	strftime (timebuf, sizeof(timebuf), "%FT%T", &local_time);
 
 	prompter.set_name ("Prompter");
 	prompter.add_button (Gtk::Stock::SAVE, Gtk::RESPONSE_ACCEPT);
+	prompter.set_title (_("Take Snapshot"));
 	prompter.set_prompt (_("Name of New Snapshot"));
-	prompter.set_initial_text (timebuf);
+
+	if (!switch_to_it) {
+		char timebuf[128];
+		time_t n;
+		struct tm local_time;
+		
+		time (&n);
+		localtime_r (&n, &local_time);
+		strftime (timebuf, sizeof(timebuf), "%FT%T", &local_time);
+		
+		prompter.set_initial_text (timebuf);
+	}
 
   again:
 	switch (prompter.run()) {
@@ -1868,7 +2009,7 @@ ARDOUR_UI::snapshot_session ()
 				msg.run ();
 				goto again;
 			}
-			save_state (snapname);
+			save_state (snapname, switch_to_it);
 		}
 		break;
 
@@ -1878,13 +2019,13 @@ ARDOUR_UI::snapshot_session ()
 }
 
 void
-ARDOUR_UI::save_state (const string & name)
+ARDOUR_UI::save_state (const string & name, bool switch_to_it)
 {
-	(void) save_state_canfail (name);
+	(void) save_state_canfail (name, switch_to_it);
 }
 		
 int
-ARDOUR_UI::save_state_canfail (string name)
+ARDOUR_UI::save_state_canfail (string name, bool switch_to_it)
 {
 	if (session) {
 		int ret;
@@ -1893,7 +2034,7 @@ ARDOUR_UI::save_state_canfail (string name)
 			name = session->snap_name();
 		}
 
-		if ((ret = session->save_state (name)) != 0) {
+		if ((ret = session->save_state (name, false, switch_to_it)) != 0) {
 			return ret;
 		}
 	}
@@ -1953,7 +2094,6 @@ ARDOUR_UI::transport_rec_enable_blink (bool onoff)
 
 void
 ARDOUR_UI::save_template ()
-
 {
 	ArdourPrompter prompter (true);
 	string name;
@@ -1963,6 +2103,7 @@ ARDOUR_UI::save_template ()
 	}
 
 	prompter.set_name (X_("Prompter"));
+	prompter.set_title (_("Save Mix Template"));
 	prompter.set_prompt (_("Name for mix template:"));
 	prompter.set_initial_text(session->name() + _("-template"));
 	prompter.add_button (Gtk::Stock::SAVE, Gtk::RESPONSE_ACCEPT);
@@ -1984,6 +2125,9 @@ ARDOUR_UI::save_template ()
 void
 ARDOUR_UI::fontconfig_dialog ()
 {
+#if 0
+  /* this issue seems to have gone away with changes to font handling in GTK/Quartz
+   */
 #ifdef GTKOSX
 	/* X11 users will always have fontconfig info around, but new GTK-OSX users 
 	   may not and it can take a while to build it. Warn them.
@@ -2007,6 +2151,7 @@ ARDOUR_UI::fontconfig_dialog ()
 		msg.run ();
 	}
 #endif
+#endif
 }
 
 void
@@ -2090,6 +2235,7 @@ ARDOUR_UI::ask_about_loading_existing_session (const Glib::ustring& session_path
 	
 	
 	msg.set_name (X_("CleanupDialog"));
+	msg.set_title (_("Cleanup Unused Sources"));
 	msg.set_wmclass (X_("existing_session"), "Ardour");
 	msg.set_position (Gtk::WIN_POS_MOUSE);
 	pop_back_splash ();
@@ -2145,14 +2291,16 @@ ARDOUR_UI::build_session_from_nsd (const Glib::ustring& session_path, const Glib
 		}
 		
 		/// @todo some minor tweaks.
+
+		oconnect = AutoConnectOption (0);
 		
-		if (new_session_dialog->connect_outs_to_master()) {
-			oconnect = AutoConnectMaster;
-		} else if (new_session_dialog->connect_outs_to_physical()) {
-			oconnect = AutoConnectPhysical;
-		} else {
-			oconnect = AutoConnectOption (0);
-		} 
+		if (new_session_dialog->connect_outputs()) {
+			if (new_session_dialog->connect_outs_to_master()) {
+				oconnect = AutoConnectMaster;
+			} else if (new_session_dialog->connect_outs_to_physical()) {
+				oconnect = AutoConnectPhysical;
+			}
+		}
 		
 		nphysin = (uint32_t) new_session_dialog->input_limit_count();
 		nphysout = (uint32_t) new_session_dialog->output_limit_count();
@@ -2301,15 +2449,21 @@ ARDOUR_UI::get_session_parameters (bool backend_audio_is_running, bool should_be
 				response = Gtk::RESPONSE_REJECT;
 				goto try_again;
 			}
+			
+			/* hide the NSD while we start up the engine */
+
+			new_session_dialog->hide ();
+			flush_pending ();
 		}
-		
+
 		if (create_engine ()) {
 
 			backend_audio_error (!backend_audio_is_running, new_session_dialog);
 			flush_pending ();
 
 			new_session_dialog->set_existing_session (false);
-			new_session_dialog->set_current_page (2);
+			new_session_dialog->set_current_page (0); // new engine page
+			new_session_dialog->engine_control.unset_interface_chosen ();
 
 			response = Gtk::RESPONSE_NONE;
 			goto try_again;
@@ -2320,7 +2474,7 @@ ARDOUR_UI::get_session_parameters (bool backend_audio_is_running, bool should_be
 		if (response == Gtk::RESPONSE_OK) {
 
 			session_name = new_session_dialog->session_name();
-			
+
 			if (session_name.empty()) {
 				response = Gtk::RESPONSE_NONE;
 				goto try_again;
@@ -2338,18 +2492,26 @@ ARDOUR_UI::get_session_parameters (bool backend_audio_is_running, bool should_be
 				session_name = Glib::path_get_basename (session_name);
 				
 			} else {
-				
+
 				session_path = new_session_dialog->session_folder();
+
 			}
 
 			template_name = Glib::ustring();			
 			switch (new_session_dialog->which_page()) {
 
 			case NewSessionDialog::OpenPage: 
-			case NewSessionDialog::EnginePage:
 				goto loadit;
 				break;
 
+			case NewSessionDialog::EnginePage:
+				if (new_session_dialog->engine_control.interface_chosen() && !session_path.empty()) {
+					goto loadit;
+				} else {
+					goto try_again;
+				}
+				break;
+
 			case NewSessionDialog::NewPage: /* nominally the "new" session creator, but could be in use for an old session */
 				
 				should_be_new = true;
@@ -2377,6 +2539,8 @@ ARDOUR_UI::get_session_parameters (bool backend_audio_is_running, bool should_be
 
 				if (Glib::file_test (session_path, Glib::FileTest (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))) {
 
+					new_session_dialog->hide ();
+
 					if (ask_about_loading_existing_session (session_path)) {
 						goto loadit;
 					} else {
@@ -2466,16 +2630,6 @@ ARDOUR_UI::load_session (const Glib::ustring& path, const Glib::ustring& snap_na
 		goto out;
 	}
 
-	/* if it already exists, we must have write access */
-
-	if (Glib::file_test (path.c_str(), Glib::FILE_TEST_EXISTS) && ::access (path.c_str(), W_OK)) {
-		MessageDialog msg (*editor, _("You do not have write access to this session.\n"
-					      "This prevents the session from being loaded."));
-		pop_back_splash ();
-		msg.run ();
-		goto out;
-	}
-
 	loading_message (_("Please wait while Ardour loads your session"));
 
 	try {
@@ -2510,6 +2664,12 @@ ARDOUR_UI::load_session (const Glib::ustring& path, const Glib::ustring& snap_na
 		goto out;
 	}
 
+	/* this exception is also special */
+
+	catch (Session::SRMismatchRejected& err) {
+		goto out; /* just go back and reload something else, etc. */
+	}
+
 	catch (...) {
 
 		MessageDialog msg (string_compose(_("Session \"%1 (snapshot %2)\" did not load successfully"), path, snap_name),
@@ -2628,6 +2788,8 @@ ARDOUR_UI::show_about ()
 		about->signal_response().connect(mem_fun (*this, &ARDOUR_UI::about_signal_response) );
 	}
 
+	about->set_transient_for(*editor);
+
 	about->show_all ();
 }
 
@@ -2683,7 +2845,8 @@ ARDOUR_UI::hide_splash ()
 }
 
 void
-ARDOUR_UI::display_cleanup_results (Session::cleanup_report& rep, const gchar* list_title, const string & msg)
+ARDOUR_UI::display_cleanup_results (Session::cleanup_report& rep, const gchar* list_title, 
+				    const string& plural_msg, const string& singular_msg)
 {
 	size_t removed;
 
@@ -2738,18 +2901,32 @@ require some unused files to continue to exist."));
 
 	dimage->set_alignment(ALIGN_LEFT, ALIGN_TOP);
 
-	if (rep.space < 1048576.0f) {
-		if (removed > 1) {
-		  txt.set_text (string_compose (msg, removed, _("files were"), session->path() + "dead_sounds", (float) rep.space / 1024.0f, "kilo"));
-		} else {
-			txt.set_text (string_compose (msg, removed, _("file was"), session->path() + "dead_sounds", (float) rep.space / 1024.0f, "kilo"));
-		}
+
+	/* subst:
+	   %1 - number of files removed
+	   %2 - location of "dead_sounds"
+	   %3 - size of files affected
+	   %4 - prefix for "bytes" to produce sensible results (e.g. mega, kilo, giga)
+	*/
+
+	const char* bprefix;
+	float space_adjusted;
+
+	if (rep.space < 1000000.0f) {
+		bprefix = X_("kilo");
+		space_adjusted = truncf((float)rep.space / 1000.0f);
+	} else if (rep.space < (1000000.0f * 1000)) {
+		bprefix = X_("mega");
+		space_adjusted = truncf((float)rep.space / (1000000.0f));
 	} else {
-		if (removed > 1) {
-			txt.set_text (string_compose (msg, removed, _("files were"), session->path() + "dead_sounds", (float) rep.space / 1048576.0f, "mega"));
-		} else {
-			txt.set_text (string_compose (msg, removed, _("file was"), session->path() + "dead_sounds", (float) rep.space / 1048576.0f, "mega"));
-		}
+		bprefix = X_("giga");
+		space_adjusted = truncf((float)rep.space / (1000000.0f * 1000));
+	}
+
+	if (removed > 1) {
+		txt.set_text (string_compose (plural_msg, removed, session->path() + "dead_sounds", space_adjusted, bprefix));
+	} else {
+		txt.set_text (string_compose (singular_msg, removed, session->path() + "dead_sounds", space_adjusted, bprefix));
 	}
 
 	dhbox.pack_start (*dimage, true, false, 5);
@@ -2836,12 +3013,19 @@ After cleanup, unused audio files will be moved to a \
 	display_cleanup_results (rep, 
 				 _("cleaned files"),
 				 _("\
-The following %1 %2 not in use and \n\
+The following %1 files were not in use and \n\
 have been moved to:\n\
-%3. \n\n\
+%2. \n\n\
 Flushing the wastebasket will \n\
 release an additional\n\
-%4 %5bytes of disk space.\n"
+%3 %4bytes of disk space.\n"),
+				 _("\
+The following file was not in use and \n	\
+has been moved to:\n				\
+%2. \n\n\
+Flushing the wastebasket will \n\
+release an additional\n\
+%3 %4bytes of disk space.\n"
 					 ));
 
 }
@@ -2862,9 +3046,12 @@ ARDOUR_UI::flush_trash ()
 
 	display_cleanup_results (rep, 
 				 _("deleted file"),
-				 _("The following %1 %2 deleted from\n\
-%3,\n\
-releasing %4 %5bytes of disk space"));
+				 _("The following %1 files were deleted from\n\
+%2,\n\
+releasing %3 %4bytes of disk space"),
+				 _("The following file was deleted from\n\
+%2,\n\
+releasing %3 %4bytes of disk space"));
 }
 
 void
@@ -3077,14 +3264,17 @@ ARDOUR_UI::write_buffer_stats ()
 	std::ofstream fout;
 	struct tm tm;
 	char buf[64];
-	char* path;
+	char path[PATH_MAX+1];	int fd;
 
-	if ((path = tempnam (0, "ardourBuffering")) == 0) {
+	strcpy (path, "ardourBufferingXXXXXX");
+
+	if ((fd = mkstemp (path )) < 0) {
 		cerr << X_("cannot find temporary name for ardour buffer stats") << endl;
 		return;
 	}
-
+	
 	fout.open (path);
+	close (fd);
 
 	if (!fout) {
 		cerr << string_compose (X_("cannot open file %1 for ardour buffer stats"), path) << endl;
@@ -3102,7 +3292,6 @@ ARDOUR_UI::write_buffer_stats ()
 	fout.close ();
 
 	cerr << "Ardour buffering statistics can be found in: " << path << endl;
-	free (path);
 }
 
 void
