@@ -151,8 +151,8 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	int  unload_session (bool hide_stuff = false);
 	void close_session(); 
 
-	int  save_state_canfail (string state_name = "");
-	void save_state (const string & state_name = "");
+	int  save_state_canfail (string state_name = "", bool switch_to_it = false);
+	void save_state (const string & state_name = "", bool switch_to_it = false);
 
 	static double gain_to_slider_position (ARDOUR::gain_t g);
         static ARDOUR::gain_t slider_position_to_gain (double pos);
@@ -241,6 +241,8 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	bool ignore_dual_punch;
 	void toggle_punch_in ();
 	void toggle_punch_out ();
+	void show_loop_punch_ruler_and_disallow_hide ();
+	void reenable_hide_loop_punch_ruler_if_appropriate ();
 	void toggle_auto_return ();
 	void toggle_click ();
 
@@ -313,7 +315,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void map_transport_state ();
 	int32_t do_engine_start ();
 	
-	void engine_halted ();
+	void engine_halted (const char* reason, bool free_reason);
 	void engine_stopped ();
 	void engine_running ();
 
@@ -576,11 +578,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void transport_forward (int option);
 	void transport_rewind (int option);
 	void transport_loop ();
-
-	void transport_rolling ();
-	void transport_rewinding ();
-	void transport_forwarding ();
-	void transport_stopped ();
+	void toggle_roll (bool with_abort, bool roll_out_of_bounded_mode);
 
 	bool _session_is_new;
 	void connect_to_session (ARDOUR::Session *);
@@ -591,7 +589,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	
 	guint32  last_key_press_time;
 
-	void snapshot_session ();
+	void snapshot_session (bool switch_to_it);
 
 	Mixer_UI   *mixer;
 	int         create_mixer ();
@@ -646,7 +644,8 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	Gtk::MenuItem *cleanup_item;
 
-	void display_cleanup_results (ARDOUR::Session::cleanup_report& rep, const gchar* list_title, const string & msg);
+	void display_cleanup_results (ARDOUR::Session::cleanup_report& rep, const gchar* list_title, 
+				      const string& plural_msg, const string& singular_msg);
 	void cleanup ();
 	void flush_trash ();
 
@@ -711,6 +710,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void set_remote_model (ARDOUR::RemoteModel);
 	void set_denormal_model (ARDOUR::DenormalModel);
 
+	void toggle_seamless_loop ();
 	void toggle_sync_order_keys ();
 	void toggle_new_plugins_active();
 	void toggle_StopPluginsWithTransport();
@@ -768,6 +768,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void display_message (const char *prefix, gint prefix_len, 
 			      Glib::RefPtr<Gtk::TextBuffer::Tag> ptag, Glib::RefPtr<Gtk::TextBuffer::Tag> mtag, const char *msg);
 	Gtk::Label status_bar_label;
+	bool status_bar_button_press (GdkEventButton*);
 	Gtk::ToggleButton error_log_button;
 	
 	void loading_message (const std::string& msg);
