@@ -80,12 +80,21 @@ ARDOUR_UI::setup_windows ()
 
 #ifdef TOP_MENUBAR
 	HBox* status_bar_packer = manage (new HBox);
-	
+	EventBox* status_bar_event_box = manage (new EventBox);
+
+	status_bar_event_box->add (status_bar_label);
+	status_bar_event_box->add_events (Gdk::BUTTON_PRESS_MASK|Gdk::BUTTON_RELEASE_MASK);
 	status_bar_label.set_size_request (300, -1);
-	status_bar_packer->pack_start (status_bar_label, true, true, 6);
+	status_bar_packer->pack_start (*status_bar_event_box, true, true, 6);
 	status_bar_packer->pack_start (error_log_button, false, false);
-	
+
+	status_bar_label.show ();
+	status_bar_event_box->show ();
+	status_bar_packer->show ();
+	error_log_button.show ();
+
 	error_log_button.signal_clicked().connect (mem_fun (*this, &UI::toggle_errors));
+	status_bar_event_box->signal_button_press_event().connect (mem_fun (*this, &ARDOUR_UI::status_bar_button_press));
 
 	editor->get_status_bar_packer().pack_start (*status_bar_packer, true, true);
 	editor->get_status_bar_packer().pack_start (menu_bar_base, false, false, 6);
@@ -99,6 +108,23 @@ ARDOUR_UI::setup_windows ()
 	return 0;
 }
 
+bool
+ARDOUR_UI::status_bar_button_press (GdkEventButton* ev)
+{
+	bool handled = false;
+
+	switch (ev->button) {
+	case 1:
+		status_bar_label.set_text ("");
+		handled = true;
+		break;
+	default:
+		break;
+	}
+
+	return handled;
+}
+
 void
 ARDOUR_UI::display_message (const char *prefix, gint prefix_len, RefPtr<TextBuffer::Tag> ptag, RefPtr<TextBuffer::Tag> mtag, const char *msg)
 {
@@ -125,71 +151,12 @@ ARDOUR_UI::display_message (const char *prefix, gint prefix_len, RefPtr<TextBuff
 #endif
 }
 
-void
-ARDOUR_UI::transport_stopped ()
-{
-	stop_button.set_visual_state (1);
-	
-	roll_button.set_visual_state (0);
-	play_selection_button.set_visual_state (0);
-	auto_loop_button.set_visual_state (0);
-
-	shuttle_fract = 0;
-	shuttle_box.queue_draw ();
-
-	update_disk_space ();
-}
-
-void
-ARDOUR_UI::transport_rolling ()
-{
-	stop_button.set_visual_state (0);
-	if (session->get_play_range()) {
-		play_selection_button.set_visual_state (1);
-		roll_button.set_visual_state (0);
-		auto_loop_button.set_visual_state (0);
-
-	} else if (session->get_play_loop ()) {
-		auto_loop_button.set_visual_state (1);
-		play_selection_button.set_visual_state (0);
-		roll_button.set_visual_state (0);
-
-	} else {
-
-		roll_button.set_visual_state (1);
-		play_selection_button.set_visual_state (0);
-		auto_loop_button.set_visual_state (0);
-	}
-
-	/* reset shuttle controller */
-
-	shuttle_fract = SHUTTLE_FRACT_SPEED1;  /* speed = 1.0, believe it or not */
-	shuttle_box.queue_draw ();
-}
-
-void
-ARDOUR_UI::transport_rewinding ()
-{
-	stop_button.set_visual_state (0);
-	roll_button.set_visual_state (1);
-	play_selection_button.set_visual_state (0);
-	auto_loop_button.set_visual_state (0);
-}
-
-void
-ARDOUR_UI::transport_forwarding ()
-{
-	stop_button.set_visual_state (0);
-	roll_button.set_visual_state (1);
-	play_selection_button.set_visual_state (0);
-	auto_loop_button.set_visual_state (0);
-}
-
 void
 ARDOUR_UI::setup_transport ()
 {
 	transport_tearoff = manage (new TearOff (transport_tearoff_hbox));
 	transport_tearoff->set_name ("TransportBase");
+	transport_tearoff->tearoff_window().signal_key_press_event().connect (bind (sigc::ptr_fun (relay_key_press), &transport_tearoff->tearoff_window()));
 
 	if (Profile->get_sae()) {
 		transport_tearoff->set_can_be_torn_off (false);
@@ -670,7 +637,7 @@ ARDOUR_UI::shuttle_box_button_release (GdkEventButton* ev)
 				roll_button.set_visual_state (1);
 			} else {
 				shuttle_fract = 0;
-				session->request_transport_speed (0.0);
+				session->request_stop ();
 			}
 			shuttle_box.queue_draw ();
 		}
