@@ -229,7 +229,7 @@ AutomationLine::AutomationLine (const string & name, TimeAxisView& tv, ArdourCan
 	points_visible = false;
 	update_pending = false;
 	_vc_uses_gain_mapping = false;
-	no_draw = false;
+	auto_is_boolean = false;
 	_visible = true;
 	terminal_points_can_slide = true;
 	_height = 0;
@@ -765,7 +765,7 @@ AutomationLine::get_verbose_cursor_string (float fraction)
 		if (fraction == 0.0) {
 			snprintf (buf, sizeof (buf), "-inf dB");
 		} else {
-			snprintf (buf, sizeof (buf), "%.1fdB", coefficient_to_dB (slider_position_to_gain (fraction)));
+			snprintf (buf, sizeof (buf), "%.1fdB", accurate_coefficient_to_dB (slider_position_to_gain (fraction)));
 		}
 	} else {
 		snprintf (buf, sizeof (buf), "%.2f", fraction);
@@ -1242,10 +1242,6 @@ AutomationLine::reset ()
 {
 	update_pending = false;
 
-	if (no_draw) {
-		return;
-	}
-
 	alist.apply_to_points (*this, &AutomationLine::reset_callback);
 }
 
@@ -1274,6 +1270,10 @@ AutomationLine::change_model_range (AutomationList::iterator start, AutomationLi
 void
 AutomationLine::show_all_control_points ()
 {
+	if (auto_is_boolean) {  //show the automation line but don't allow any control points
+		return;
+	}
+
 	points_visible = true;
 
 	for (vector<ControlPoint*>::iterator i = control_points.begin(); i != control_points.end(); ++i) {
