@@ -57,6 +57,7 @@
 #include "editing.h"
 #include "enums.h"
 #include "editor_items.h"
+#include "canvas-noevent-text.h"
 #include "region_selection.h"
 #include "canvas.h"
 #include "time_axis_view.h"
@@ -108,6 +109,7 @@ class AudioStreamView;
 class ControlPoint;
 class SoundFileOmega;
 class RhythmFerret;
+class RegionLayeringOrderEditor;
 #ifdef FFT_ANALYSIS
 class AnalysisWindow;
 #endif
@@ -171,16 +173,21 @@ class Editor : public PublicEditor
 	std::vector<std::string> rb_opt_strings;
 #endif
 
-	/* option editor-access */
+	//global waveform options
 
 	void set_show_waveforms (bool yn);
 	bool show_waveforms() const { return _show_waveforms; }
 
-	void set_waveform_scale (Editing::WaveformScale);
+	void set_show_waveforms_rectified (bool yn);
+	bool show_waveforms_rectified() const { return _show_waveforms_rectified; }
 
 	void set_show_waveforms_recording (bool yn);
 	bool show_waveforms_recording() const { return _show_waveforms_recording; }
 	
+	//per-track waveform options
+	
+	void set_waveform_scale (Editing::WaveformScale);
+
 	/* things that need to be public to be used in the main menubar */
 
 	void new_region_from_selection ();
@@ -189,7 +196,6 @@ class Editor : public PublicEditor
 	void separate_region_from_punch ();
 	void separate_region_from_loop ();
 	void separate_regions_using_location (ARDOUR::Location&);
-	void toggle_playback (bool with_abort);
 	void transition_to_rolling (bool forward);
 
 	/* undo related */
@@ -318,7 +324,9 @@ class Editor : public PublicEditor
 	bool dragging_playhead () const { return _dragging_playhead; }
 
 	void toggle_waveform_visibility ();
+	void toggle_waveform_rectified ();
 	void toggle_waveforms_while_recording ();
+	
 	void toggle_measure_visibility ();
 	void toggle_logo_visibility ();
 
@@ -423,7 +431,6 @@ class Editor : public PublicEditor
 	    double              frames_per_unit;
 	    nframes64_t         leftmost_frame;
 	    Editing::ZoomFocus  zoom_focus;
-	    bool                zoomed_to_region;
 	    std::list<TAVState> track_states;
 	};
 	
@@ -572,7 +579,7 @@ class Editor : public PublicEditor
 	Gtk::Menu* build_track_selection_context_menu (nframes64_t);
 	void add_dstream_context_items (Gtk::Menu_Helpers::MenuList&);
 	void add_bus_context_items (Gtk::Menu_Helpers::MenuList&);
-	void add_region_context_items (AudioStreamView*, boost::shared_ptr<ARDOUR::Region>, Gtk::Menu_Helpers::MenuList&);
+	void add_region_context_items (AudioStreamView*, boost::shared_ptr<ARDOUR::Region>, Gtk::Menu_Helpers::MenuList&, nframes64_t position, bool multiple_regions_at_position);
 	void add_crossfade_context_items (AudioStreamView*, boost::shared_ptr<ARDOUR::Crossfade>, Gtk::Menu_Helpers::MenuList&, bool many);
 	void add_selection_context_items (Gtk::Menu_Helpers::MenuList&);
 
@@ -593,7 +600,7 @@ class Editor : public PublicEditor
 	Gdk::Cursor* which_grabber_cursor ();
 
 	ArdourCanvas::Canvas* track_canvas;
-	ArdourCanvas::Text* verbose_canvas_cursor;
+	ArdourCanvas::NoEventText* verbose_canvas_cursor;
 	bool                 verbose_cursor_visible;
 
 	void parameter_changed (const char *);
@@ -1064,6 +1071,7 @@ class Editor : public PublicEditor
 	void set_region_lock_style (ARDOUR::Region::PositionLockStyle);
 	void raise_region ();
 	void raise_region_to_top ();
+	void change_region_layering_order (nframes64_t);
 	void lower_region ();
 	void lower_region_to_bottom ();
 	void split_region ();
@@ -1143,7 +1151,6 @@ class Editor : public PublicEditor
 	void temporal_zoom_selection ();
 	void temporal_zoom_region (bool both_axes);
 	void toggle_zoom_region (bool both_axes);
-	bool zoomed_to_region;
 	void temporal_zoom_session ();
 	void temporal_zoom (gdouble scale);
 	void temporal_zoom_by_frame (nframes64_t start, nframes64_t end, const string & op);
@@ -1473,10 +1480,13 @@ public:
 	/* display control */
 	
 	bool _show_measures;
+
 	bool _show_waveforms;
+	bool _show_waveforms_rectified;
+	bool _show_waveforms_recording;
+
 	bool _stationary_playhead;
 	bool _follow_playhead;
-	bool _show_waveforms_recording;
 	
 	ARDOUR::TempoMap::BBTPointList *current_bbt_points;
 	
@@ -1779,6 +1789,7 @@ public:
 	void hide_all_routes ();
 	void show_all_audiotracks ();
 	void hide_all_audiotracks ();
+	void show_tracks_with_regions_at_playhead ();
 	void show_all_audiobus ();
 	void hide_all_audiobus ();
 
@@ -1811,6 +1822,8 @@ public:
 	void build_edit_group_list_menu ();
 	void activate_all_edit_groups ();
 	void disable_all_edit_groups ();
+	void show_all_edit_groups ();
+	void hide_all_edit_groups ();
 
 	bool in_edit_group_row_change;
 	void edit_group_row_change (const Gtk::TreeModel::Path&,const Gtk::TreeModel::iterator&);
@@ -2222,7 +2235,6 @@ public:
 
 	void remove_tracks ();
 	void toggle_tracks_active ();
-	void waveform_scale_chosen (Editing::WaveformScale);
 
 	bool _have_idled;
 	int resize_idle_id;
@@ -2230,6 +2242,10 @@ public:
 	bool idle_resize();
 	friend gboolean _idle_resize (gpointer);
 	std::vector<TimeAxisView*> pending_resizes;
+
+	RegionLayeringOrderEditor* layering_order_editor;
+
+	void update_region_layering_order_editor (nframes64_t frame);
 };
 
 #endif /* __ardour_editor_h__ */
