@@ -62,6 +62,21 @@ Editor::set_show_waveforms_recording (bool yn)
 	}
 }
 
+void
+Editor::set_show_waveforms_rectified (bool yn)
+{
+	AudioTimeAxisView* atv;
+
+	if (_show_waveforms_rectified != yn) {
+		_show_waveforms_rectified = yn;
+		for (TrackViewList::iterator i = track_views.begin(); i != track_views.end(); ++i) {
+			if ((atv = dynamic_cast<AudioTimeAxisView*>(*i)) != 0) {
+				atv->set_show_waveforms_rectified (yn);
+			}
+		}
+	}
+}
+
 gint
 Editor::start_updating ()
 {
