@@ -17,6 +17,12 @@
 
 */
 
+#ifdef __APPLE__
+#define Rect AppleHasItsOwnRect
+#include <CoreServices/../Frameworks/CarbonCore.framework/Headers/MacTypes.h>
+#undef Rect
+#endif /* __APPLE__ */
+
 #include <libgnomecanvasmm/init.h>
 #include <libgnomecanvasmm/pixbuf.h>
 #include <jack/types.h>
@@ -113,7 +119,7 @@ Editor::initialize_canvas ()
 
 	Pango::FontDescription* font = get_font_for_style (N_("VerboseCanvasCursor"));
 
-	verbose_canvas_cursor = new ArdourCanvas::Text (*track_canvas->root());
+	verbose_canvas_cursor = new ArdourCanvas::NoEventText (*track_canvas->root());
 	verbose_canvas_cursor->property_font_desc() = *font;
 	verbose_canvas_cursor->property_anchor() = ANCHOR_NW;
 
