@@ -398,24 +398,12 @@ Editor::region_list_display_button_press (GdkEventButton *ev)
 		return true;
 	}
 
-	if (region == 0) {
-		return false;
-	}
-
-	switch (ev->button) {
-	case 1:
-		break;
-
-	case 2:
-		// audition on middle click (stop audition too)
+	if (region != 0 && Keyboard::is_button2_event (ev)) {
+		// start/stop audition 
 		if (!Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier)) {
 			consider_auditioning (region);
 		}
 		return true;
-		break;
-
-	default:
-		break; 
 	}
 
 	return false;
@@ -460,7 +448,7 @@ Editor::consider_auditioning (boost::shared_ptr<Region> region)
 		if (r == last_audition_region) {
 			return;
 		}
-	}
+	} 
 
 	session->audition_region (r);
 	last_audition_region = r;
