@@ -66,12 +66,16 @@ static const char* localedir = LOCALEDIR;
 #include <sys/param.h>
 #include <fstream>
 
+extern void set_language_preference (); // cocoacarbon.mm
+
 void
 fixup_bundle_environment ()
 {
 	if (!getenv ("ARDOUR_BUNDLED")) {
 		return;
 	}
+	
+	set_language_preference ();
 
 	char execpath[MAXPATHLEN+1];
 	uint32_t pathsz = sizeof (execpath);
@@ -178,6 +182,7 @@ fixup_bundle_environment ()
 	path += "/../Resources/locale";
 	
 	localedir = strdup (path.c_str());
+	setenv ("GTK_LOCALEDIR", localedir, 1);
 
 	/* write a pango.rc file and tell pango to use it. we'd love
 	   to put this into the Ardour.app bundle and leave it there,
@@ -241,12 +246,55 @@ fixup_bundle_environment ()
 
 #endif
 
+static gboolean
+tell_about_jack_death (void* /* ignored */)
+{
+	if (AudioEngine::instance()->processed_frames() == 0) {
+		/* died during startup */
+		MessageDialog msg (_("JACK exited"), false, Gtk::MESSAGE_INFO, Gtk::BUTTONS_OK);
+		msg.set_position (Gtk::WIN_POS_CENTER);
+		msg.set_secondary_text (_(
+"JACK exited unexpectedly, and without notifying Ardour.\n\
+\n\
+This could be due to misconfiguration or to an error inside JACK.\n\
+\n\
+Click OK to exit Ardour."));
+    
+		msg.run ();
+		_exit (0);
+		
+	} else {
+
+		/* engine has already run, so this is a mid-session JACK death */
+		
+		MessageDialog* msg = manage (new MessageDialog (_("JACK exited"), false, Gtk::MESSAGE_INFO, Gtk::BUTTONS_NONE));
+		msg->set_secondary_text (_(
+"JACK exited unexpectedly, and without notifying Ardour.\n\
+\n\
+This is probably due to an error inside JACK. You should restart JACK\n\
+and reconnect Ardour to it, or exit Ardour now. You cannot save your\n\
+session at this time, because we would lose your connection information.\n"));
+		msg->present ();
+	}
+	return false; /* do not call again */
+}
+
 static void
 sigpipe_handler (int sig)
 {
-	cerr << _("SIGPIPE received - JACK has probably died") << endl;
+        static bool done_the_jack_thing = false;
+	
+	if (!done_the_jack_thing) {
+		AudioEngine::instance()->died ();
+		g_idle_add (tell_about_jack_death, 0);
+		done_the_jack_thing =  true;
+	}
 }
 
+#ifdef HAVE_LV2
+void close_external_ui_windows();
+#endif
+
 #ifdef VST_SUPPORT
 
 extern int gui_init (int* argc, char** argv[]);
@@ -269,7 +317,6 @@ int main (int argc, char* argv[])
 #endif
 
         Glib::thread_init();
-	gtk_set_locale ();
 
 #ifdef VST_SUPPORT
 	/* this does some magic that is needed to make GTK and Wine's own
@@ -284,6 +331,10 @@ int main (int argc, char* argv[])
 	   we use that when handling them.
 	*/
 	(void) bind_textdomain_codeset (PACKAGE,"UTF-8");
+
+	/* this really does nothing since we don't get gettext(),
+	   only dgettext(), but whatever ...
+	*/
 	(void) textdomain (PACKAGE);
 
 	pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, 0);
@@ -348,6 +399,9 @@ int main (int argc, char* argv[])
 
 	ARDOUR::cleanup ();
 	pthread_cancel_all ();
+#ifdef HAVE_LV2
+	close_external_ui_windows();
+#endif
 	return 0;
 }
 #ifdef VST_SUPPORT
