@@ -639,12 +639,36 @@ Mixer_UI::show_all_audiotracks()
 {
 	set_all_audio_visibility (1, true);
 }
+
 void
 Mixer_UI::hide_all_audiotracks ()
 {
 	set_all_audio_visibility (1, false);
 }
 
+void
+Mixer_UI::show_tracks_with_regions_at_playhead ()
+{
+	boost::shared_ptr<Session::RouteList> const regions = session->get_routes_with_regions_at (session->transport_frame ());
+	
+	TreeModel::Children rows = track_model->children ();
+	for (TreeModel::Children::iterator i = rows.begin(); i != rows.end(); ++i) {
+		boost::shared_ptr<Route> route = (*i)[track_columns.route];
+
+		bool found = false;
+		for (Session::RouteList::iterator x = (*regions).begin(); x != (*regions).end(); ++x) {
+			if ((*x) == route)
+				found = true;
+		}
+				
+		(*i)[track_columns.visible] = found;
+	}
+
+	strip_redisplay_does_not_sync_order_keys = true;
+	redisplay_track_list ();
+	strip_redisplay_does_not_sync_order_keys = false;
+}
+
 void
 Mixer_UI::track_list_reorder (const TreeModel::Path& path, const TreeModel::iterator& iter, int* new_order)
 {
@@ -787,16 +811,7 @@ Mixer_UI::set_auto_rebinding( bool val )
 void 
 Mixer_UI::toggle_auto_rebinding() 
 {
-	if (auto_rebinding)
-	{
-		set_auto_rebinding( FALSE );
-	}
-	
-	else
-	{
-		set_auto_rebinding( TRUE );
-	}
-
+	set_auto_rebinding (!auto_rebinding);
 	auto_rebind_midi_controls();
 }
 
@@ -953,7 +968,8 @@ Mixer_UI::build_track_menu ()
 	items.push_back (MenuElem (_("Hide All Audio Tracks"), mem_fun(*this, &Mixer_UI::hide_all_audiotracks)));
 	items.push_back (MenuElem (_("Show All Audio Busses"), mem_fun(*this, &Mixer_UI::show_all_audiobus)));
 	items.push_back (MenuElem (_("Hide All Audio Busses"), mem_fun(*this, &Mixer_UI::hide_all_audiobus)));
-
+	items.push_back (MenuElem (_("Hide All Audio Busses"), mem_fun(*this, &Mixer_UI::hide_all_audiobus)));
+	items.push_back (MenuElem (_("Show Tracks With Regions Under Playhead"), mem_fun (*this, &Mixer_UI::show_tracks_with_regions_at_playhead)));
 }
 
 void
@@ -987,6 +1003,9 @@ Mixer_UI::build_mix_group_context_menu ()
 	items.push_back (MenuElem (_("Activate All"), mem_fun(*this, &Mixer_UI::activate_all_mix_groups)));
 	items.push_back (MenuElem (_("Disable All"), mem_fun(*this, &Mixer_UI::disable_all_mix_groups)));
 	items.push_back (SeparatorElem());
+	items.push_back (MenuElem (_("Show All"), mem_fun(*this, &Mixer_UI::show_all_mix_groups)));
+	items.push_back (MenuElem (_("Hide All"), mem_fun(*this, &Mixer_UI::hide_all_mix_groups)));
+	items.push_back (SeparatorElem());
 	items.push_back (MenuElem (_("Add group"), mem_fun(*this, &Mixer_UI::new_mix_group)));
 	
 }
@@ -1068,6 +1087,8 @@ Mixer_UI::activate_all_mix_groups ()
 	}
 }
 
+
+
 void
 Mixer_UI::disable_all_mix_groups ()
 {
@@ -1077,6 +1098,24 @@ Mixer_UI::disable_all_mix_groups ()
 	}
 }
 
+void
+Mixer_UI::show_all_mix_groups ()
+{
+       Gtk::TreeModel::Children children = group_model->children();
+	for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter) {
+	        (*iter)[group_columns.visible] = true;
+	}
+}
+
+void
+Mixer_UI::hide_all_mix_groups ()
+{
+       Gtk::TreeModel::Children children = group_model->children();
+	for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter) {
+	        (*iter)[group_columns.visible] = false;
+	}
+}
+
 void
 Mixer_UI::mix_groups_changed ()
 {
@@ -1086,15 +1125,6 @@ Mixer_UI::mix_groups_changed ()
 
 	group_model->clear ();
 
-	{
-		TreeModel::Row row;
-		row = *(group_model->append());
-		row[group_columns.active] = false;
-		row[group_columns.visible] = true;
-		row[group_columns.text] = (_("-all-"));
-		row[group_columns.group] = 0;
-	}
-
 	session->foreach_mix_group (mem_fun (*this, &Mixer_UI::add_mix_group));
 }
 
@@ -1347,7 +1377,7 @@ Mixer_UI::set_state (const XMLNode& node)
 	set_window_pos_and_size ();
 
 	if ((prop = node.property ("narrow-strips"))) {
-		if (prop->value() == "yes") {
+		if (string_is_affirmative (prop->value())) {
 			set_strip_width (Narrow);
 		} else {
 			set_strip_width (Wide);
@@ -1355,7 +1385,7 @@ Mixer_UI::set_state (const XMLNode& node)
 	}
 
 	if ((prop = node.property ("show-mixer"))) {
-		if (prop->value() == "yes") {
+		if (string_is_affirmative (prop->value())) {
 		       _visible = true;
 		}
 	}
@@ -1473,8 +1503,66 @@ Mixer_UI::pane_allocation_handler (Allocation& alloc, Gtk::Paned* which)
 	}
 }
 
+void
+Mixer_UI::scroll_left () 
+{
+	Adjustment* adj = scroller.get_hscrollbar()->get_adjustment();
+	/* stupid GTK: can't rely on clamping across versions */
+	scroller.get_hscrollbar()->set_value (max (adj->get_lower(), adj->get_value() - adj->get_step_increment()));
+}
+
+void
+Mixer_UI::scroll_right ()
+{
+	Adjustment* adj = scroller.get_hscrollbar()->get_adjustment();
+	/* stupid GTK: can't rely on clamping across versions */
+	scroller.get_hscrollbar()->set_value (min (adj->get_upper(), adj->get_value() + adj->get_step_increment()));
+}
+
 bool
 Mixer_UI::on_key_press_event (GdkEventKey* ev)
 {
+	switch (ev->keyval) {
+	case GDK_Left:
+		scroll_left ();
+		return true;
+
+	case GDK_Right:
+		scroll_right ();
+		return true;
+
+	default:
+		break;
+	}
+
 	return key_press_focus_accelerator_handler (*this, ev);
 }
+
+bool
+Mixer_UI::on_scroll_event (GdkEventScroll* ev)
+{
+	switch (ev->direction) {
+	case GDK_SCROLL_LEFT:
+		scroll_left ();
+		return true;
+	case GDK_SCROLL_UP:
+		if (ev->state & Keyboard::TertiaryModifier) {
+			scroll_left ();
+			return true;
+		}
+		return false;
+
+	case GDK_SCROLL_RIGHT:
+		scroll_right ();
+		return true;
+
+	case GDK_SCROLL_DOWN:
+		if (ev->state & Keyboard::TertiaryModifier) {
+			scroll_right ();
+			return true;
+		}
+		return false;
+	}
+
+	return false;
+}
