@@ -121,9 +121,7 @@ OptionEditor::OptionEditor (ARDOUR_UI& uip, PublicEditor& ed, Mixer_UI& mixui)
 	auditioner_io_selector = 0;
 	session = 0;
 	
-	WindowTitle title(Glib::get_application_name());
-	title += _("Preferences");
-	set_title(title.get_string());
+	set_title(_("Preferences"));
 
 	set_default_size (300, 300);
 	set_wmclass (X_("ardour_preferences"), "Ardour");
@@ -259,12 +257,14 @@ OptionEditor::add_session_paths ()
 	session_raid_entry.set_text(session->raid_path());
 }
 
+#ifndef GTKOSX
 static void
 font_scale_changed (Gtk::Adjustment* adj)
 {
 	Config->set_font_scale((long)floor (adj->get_value() * 1024));
 	reset_dpi();
 }
+#endif
 
 void
 OptionEditor::setup_misc_options ()
