@@ -94,11 +94,14 @@ class PublicEditor : public Gtk::Window, public PBD::StatefulThingWithGoingAway
 	virtual void set_mouse_mode (Editing::MouseMode, bool force = false) = 0;
 	virtual void step_mouse_mode (bool next) = 0;
 	virtual Editing::MouseMode current_mouse_mode () = 0;
+
+	/* CMT */
+	virtual void connect_to_image_compositor()  = 0;
 	virtual void add_imageframe_time_axis(const std::string & track_name, void*)  = 0;
 	virtual void add_imageframe_marker_time_axis(const std::string & track_name, TimeAxisView* marked_track, void*)  = 0;
-	virtual void connect_to_image_compositor()  = 0;
 	virtual void scroll_timeaxis_to_imageframe_item(const TimeAxisViewItem* item)  = 0;
 	virtual TimeAxisView* get_named_time_axis(const std::string & name)  = 0;
+
 	virtual void consider_auditioning (boost::shared_ptr<ARDOUR::Region>) = 0;
 	virtual void set_show_waveforms (bool yn) = 0;
 	virtual bool show_waveforms() const = 0;
@@ -106,7 +109,6 @@ class PublicEditor : public Gtk::Window, public PBD::StatefulThingWithGoingAway
 	virtual bool show_waveforms_recording() const = 0;
 	virtual void new_region_from_selection () = 0;
 	virtual void separate_region_from_selection () = 0;
-	virtual void toggle_playback (bool with_abort) = 0;
 	virtual void transition_to_rolling (bool fwd) = 0;
 	virtual nframes64_t unit_to_frame (double unit) const = 0;
 	// XXX remove me when libardour goes nframes64_t
@@ -235,7 +237,7 @@ class PublicEditor : public Gtk::Window, public PBD::StatefulThingWithGoingAway
 
 	static PublicEditor* _instance;
 
-	friend class PluginUIWindow;
+	friend bool relay_key_press (GdkEventKey* ev, Gtk::Window* win);
 };
 
 #endif // __gtk_ardour_public_editor_h__
