@@ -34,7 +34,6 @@
 #include <gtkmm2ext/utils.h>
 #include <gtkmm2ext/stop_signal.h>
 #include <gtkmm2ext/doi.h>
-#include <gtkmm2ext/window_title.h>
 
 #include <ardour/ardour.h>
 #include <ardour/session.h>
@@ -152,8 +151,14 @@ RedirectBox::~RedirectBox ()
 void
 RedirectBox::set_route (boost::shared_ptr<Route> r)
 {
+
 	connections.clear ();
 
+        /* since we're dealing with a new route, any existing limit on redisplay
+	   must be irrelevant.
+	*/
+
+	no_redirect_redisplay = false;
 	_route = r;
 
 	connections.push_back (_route->redirects_changed.connect (mem_fun(*this, &RedirectBox::redisplay_redirects)));
@@ -253,7 +258,7 @@ RedirectBox::show_redirect_menu (gint arg)
 	Gtk::MenuItem* plugin_menu_item = dynamic_cast<Gtk::MenuItem*>(ActionManager::get_widget("/redirectmenu/newplugin"));
 
 	if (plugin_menu_item) {
-		plugin_menu_item->set_submenu (_plugin_selector.plugin_menu());
+		plugin_menu_item->set_submenu (*_plugin_selector.plugin_menu());
 	}
 
 	cut_action->set_sensitive (can_cut_redirects ());
@@ -738,10 +743,11 @@ RedirectBox::compute_redirect_sort_keys ()
 		Label label;
 
 		label.set_text (_("\
-You cannot reorder this set of redirects\n\
+You cannot reorder the plugins/sends/inserts\n\
 in that way because the inputs and\n\
-outputs do not work correctly."));
+outputs will not work correctly."));
 
+		dialog.get_vbox()->set_border_width (12);
 		dialog.get_vbox()->pack_start (label);
 		dialog.add_button (Stock::OK, RESPONSE_ACCEPT);
 
@@ -918,9 +924,26 @@ RedirectBox::rename_redirect (boost::shared_ptr<Redirect> redirect)
 	switch (name_prompter.run ()) {
 
 	case Gtk::RESPONSE_ACCEPT:
-        name_prompter.get_result (result);
-        if (result.length()) {
-			redirect->set_name (result, this);
+		name_prompter.get_result (result);
+		if (result.length()) {
+			int tries = 0;
+			string test = result;
+
+			while (tries < 100) {
+				if (_session.io_name_is_legal (test)) {
+					result = test;
+					break;
+				}
+				tries++;
+				test = string_compose ("%1-%2", result, tries);
+			}
+
+			if (tries < 100) {
+				redirect->set_name (result, this);
+			} else {
+				ARDOUR_UI::instance()->popup_error 
+					(string_compose (_("At least 100 IO objects exist with a name like %1 - name not changed"), result));
+			}
 		}	
 		break;
 	}
@@ -1111,9 +1134,7 @@ RedirectBox::edit_redirect (boost::shared_ptr<Redirect> redirect)
 			
 			send_ui = new SendUIWindow (send, _session);
 
-			WindowTitle title(Glib::get_application_name());
-			title += send->name();
-			send_ui->set_title (title.get_string());
+			send_ui->set_title (send->name());
 
 			send->set_gui (send_ui);
 			
@@ -1148,9 +1169,7 @@ RedirectBox::edit_redirect (boost::shared_ptr<Redirect> redirect)
 
 				plugin_ui = new PluginUIWindow (win, plugin_insert);
 				
-				WindowTitle title(Glib::get_application_name());
-				title += generate_redirect_title (plugin_insert);
-				plugin_ui->set_title (title.get_string());
+				plugin_ui->set_title (generate_redirect_title (plugin_insert));
 				
 				plugin_insert->set_gui (plugin_ui);
 
@@ -1432,19 +1451,15 @@ RedirectBox::route_name_changed (void* src)
 
 		/* rename editor windows for sends and plugins */
 
-		WindowTitle title (Glib::get_application_name());
-		
 		if ((insert = boost::dynamic_pointer_cast<Insert> (redirect)) == 0) {
 			boost::shared_ptr<Send> send = boost::dynamic_pointer_cast<Send> (redirect);
-			title += send->name();
-			static_cast<Window*>(gui)->set_title (title.get_string());
+			static_cast<Window*>(gui)->set_title (send->name());
 		} else {
 			boost::shared_ptr<PluginInsert> plugin_insert;
 			
 			if ((plugin_insert = boost::dynamic_pointer_cast<PluginInsert> (insert)) != 0) {
-				title += generate_redirect_title (plugin_insert);
+				static_cast<Window*>(gui)->set_title (generate_redirect_title (plugin_insert));
 			}
-			static_cast<Window*>(gui)->set_title (title.get_string());
 		}
 	}
 }
