@@ -170,7 +170,7 @@ RouteUI::set_route (boost::shared_ptr<Route> rp)
 	update_solo_display ();
 	update_mute_display ();
 
-	if (is_track()) {
+	if (_session.writable() && is_track()) {
 		boost::shared_ptr<Track> t = boost::dynamic_pointer_cast<Track>(_route);
 
 		connections.push_back (t->diskstream()->RecordEnableChanged.connect (mem_fun (*this, &RouteUI::route_rec_enable_changed)));
@@ -232,14 +232,13 @@ RouteUI::mute_press(GdkEventButton* ev)
 		} else {
 
 			if (Keyboard::is_button2_event (ev)) {
-				// Primary-button2 click is the midi binding click
-				// button2-click is "momentary"
 				
-				if (!Keyboard::modifier_state_equals (ev->state, Keyboard::ModifierMask (Keyboard::PrimaryModifier))) {
-					wait_for_release = true;
+				if (mute_button->on_button_press_event (ev)) {
+					return true;
 				} else {
-					return false;
-				}
+					// button2-click is "momentary"
+					wait_for_release = true;
+				} 
 			}
 
 			if (ev->button == 1 || Keyboard::is_button2_event (ev)) {
@@ -325,14 +324,12 @@ RouteUI::solo_press(GdkEventButton* ev)
 
 			if (Keyboard::is_button2_event (ev)) {
 
-				// Primary-button2 click is the midi binding click
-				// button2-click is "momentary"
-				
-				if (!Keyboard::modifier_state_equals (ev->state, Keyboard::ModifierMask (Keyboard::PrimaryModifier))) {
-					wait_for_release = true;
+				if (solo_button->on_button_press_event (ev)) {
+					return true;
 				} else {
-					return false;
-				}
+					// button2-click is "momentary"
+					wait_for_release = true;
+				} 
 			}
 
 			if (ev->button == 1 || Keyboard::is_button2_event (ev)) {
@@ -452,10 +449,9 @@ RouteUI::rec_enable_press(GdkEventButton* ev)
 
 	if (!ignore_toggle && is_track() && rec_enable_button) {
 
-		if (Keyboard::is_button2_event (ev) && Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier)) {
+		if (Keyboard::is_button2_event (ev)) {
 
-			// do nothing on midi bind event
-			return false;
+			return rec_enable_button->on_button_press_event (ev);
 
 		} else if (Keyboard::modifier_state_equals (ev->state, Keyboard::ModifierMask (Keyboard::PrimaryModifier|Keyboard::TertiaryModifier))) {
 
