@@ -410,6 +410,10 @@ SoundFileBrowser::SoundFileBrowser (Gtk::Window& parent, string title, ARDOUR::S
 		chooser.add_shortcut_folder_uri("file:///Library/Application Support/GarageBand/Instrument Library/Sampler/Sampler Files");
 	}
 	
+#ifdef GTKOSX
+	chooser.add_shortcut_folder_uri("file:///Volumes");
+#endif
+
 	//add the file chooser
 	{
 		chooser.set_border_width (12);
@@ -974,7 +978,7 @@ SoundFileOmega::reset_options ()
 				channel_strings.push_back (_("sequence files"));
 			}
 			if (same_size) {
-				channel_strings.push_back (_("all files in one region"));
+				channel_strings.push_back (_("all files in one track"));
 			}
 			
 		}
@@ -1289,6 +1293,7 @@ SoundFileOmega::SoundFileOmega (Gtk::Window& parent, string title, ARDOUR::Sessi
 	disposition_map.insert (pair<ustring,ImportDisposition>(_("one region per file"), ImportDistinctFiles));
 	disposition_map.insert (pair<ustring,ImportDisposition>(_("one region per channel"), ImportDistinctChannels));
 	disposition_map.insert (pair<ustring,ImportDisposition>(_("all files in one region"), ImportMergeFiles));
+	disposition_map.insert (pair<ustring,ImportDisposition>(_("all files in one track"), ImportSerializeFiles));
 
 	chooser.signal_selection_changed().connect (mem_fun (*this, &SoundFileOmega::file_selection_changed));
 
