@@ -54,6 +54,10 @@ sigc::signal<void>  DPIReset;
 int
 pixel_width (const ustring& str, Pango::FontDescription& font)
 {
+	if (str.empty()) {
+		return 0;
+	}
+
 	Label foo;
 	Glib::RefPtr<Pango::Layout> layout = foo.create_pango_layout ("");
 
@@ -396,9 +400,80 @@ set_color (Gdk::Color& c, int rgb)
 	c.set_rgb((rgb >> 16)*256, ((rgb & 0xff00) >> 8)*256, (rgb & 0xff)*256);
 }
 
+bool
+relay_key_press (GdkEventKey* ev, Gtk::Window* win)
+{
+	if (!key_press_focus_accelerator_handler (*win, ev)) {
+		return PublicEditor::instance().on_key_press_event(ev);
+	} else {
+		return true;
+	}
+}
+
 #ifdef GTKOSX
-extern "C" {
-	gboolean gdk_quartz_possibly_forward (GdkEvent*);
+static guint
+osx_keyval_without_alt (guint accent_keyval)
+{
+	switch (accent_keyval) {
+	case GDK_oe:
+		return GDK_q;
+	case GDK_registered:
+		return GDK_r;
+	case GDK_dagger:
+		return GDK_t;
+	case GDK_yen:
+		return GDK_y;
+	case GDK_diaeresis:
+		return GDK_u;
+	case GDK_oslash:
+		return GDK_o;
+	case GDK_Greek_pi:
+		return GDK_p;
+	case GDK_leftdoublequotemark:
+		return GDK_bracketleft;
+	case GDK_leftsinglequotemark:
+		return GDK_bracketright;
+	case GDK_guillemotleft:
+		return GDK_backslash;
+	case GDK_aring:
+		return GDK_a;
+	case GDK_ssharp:
+		return GDK_s;
+	case GDK_partialderivative:
+		return GDK_d;
+	case GDK_function:
+		return GDK_f;
+	case GDK_copyright:
+		return GDK_g;
+	case GDK_abovedot:
+		return GDK_h;
+	case GDK_notsign:
+		return GDK_l;
+	case GDK_ellipsis:
+		return GDK_semicolon;
+	case GDK_ae:
+		return GDK_apostrophe;
+	case GDK_Greek_OMEGA:
+		return GDK_z;
+	case GDK_ccedilla:
+		return GDK_c;
+	case GDK_radical:
+		return GDK_v;
+	case GDK_integral:
+		return GDK_b;
+	case GDK_mu:
+		return GDK_m;
+	case GDK_lessthanequal:
+		return GDK_comma;
+	case GDK_greaterthanequal:
+		return GDK_period;
+	case GDK_division:
+		return GDK_slash;
+	default:
+		break;
+	}
+
+	return GDK_VoidSymbol;
 }
 #endif
 
@@ -419,6 +494,9 @@ key_press_focus_accelerator_handler (Gtk::Window& window, GdkEventKey* ev)
 		if (GTK_IS_ENTRY(focus) || Keyboard::some_magic_widget_has_focus()) {
 			special_handling_of_unmodified_accelerators = true;
 		} 
+#ifdef DEBUG_ACCELERATOR_HANDLING
+		cerr << "Focus widget name " << gtk_widget_get_name(focus) << endl;
+#endif
 	} 
 
 #ifdef GTKOSX
@@ -430,7 +508,13 @@ key_press_focus_accelerator_handler (Gtk::Window& window, GdkEventKey* ev)
 
 #ifdef DEBUG_ACCELERATOR_HANDLING
 	if (debug) {
-		cerr << "Win = " << win << " Key event: code = " << ev->keyval << " state = " << hex << ev->state << dec << " special handling ? " 
+		cerr << "Win = " << win << " Key event: code = " << ev->keyval << " name = " << gdk_keyval_name (ev->keyval) << " state = " << hex << ev->state << dec 
+		     << " ctrl " << ((ev->state & GDK_CONTROL_MASK) ? 1 : 0)
+		     << " alt " << ((ev->state & GDK_MOD1_MASK) ? 1 : 0)
+		     << " shift " << ((ev->state & GDK_SHIFT_MASK) ? 1 : 0)
+		     << " cmd/meta " << ((ev->state & GDK_META_MASK) ? 1 : 0)
+		     << " lock " << ((ev->state & GDK_LOCK_MASK) ? 1 : 0)
+		     << " special handling ? " 
 		     << special_handling_of_unmodified_accelerators
 		     << " magic widget focus ? "
 		     << Keyboard::some_magic_widget_has_focus()
@@ -471,6 +555,24 @@ key_press_focus_accelerator_handler (Gtk::Window& window, GdkEventKey* ev)
 	   all "normal text" accelerators.
 	*/
 
+#ifdef GTKOSX
+	if (!special_handling_of_unmodified_accelerators) {
+		if (ev->state & GDK_MOD1_MASK) {
+			/* we're not in a text entry or "magic focus" widget so we don't want OS X "special-character" 
+			   text-style handling of alt-<key>. change the keyval back to what it would be without
+			   the alt key. this way, we see <alt>-v rather than <alt>-radical and so on.
+			*/
+			guint keyval_without_alt = osx_keyval_without_alt (ev->keyval);
+
+			if (keyval_without_alt != GDK_VoidSymbol) {
+#ifdef DEBUG_ACCELERATOR_HANDLING
+				cerr << "Remapped " << gdk_keyval_name (ev->keyval) << " to " << gdk_keyval_name (keyval_without_alt) << endl;
+
+#endif				ev->keyval = keyval_without_alt;
+			}
+		}
+	}
+#endif
 
 	if (!special_handling_of_unmodified_accelerators) {
 
@@ -482,20 +584,19 @@ key_press_focus_accelerator_handler (Gtk::Window& window, GdkEventKey* ev)
 		uint32_t fakekey = ev->keyval;
 
 		if (possibly_translate_keyval_to_make_legal_accelerator (fakekey)) {
-			if (allow_activating && gtk_accel_groups_activate(G_OBJECT(win), fakekey, GdkModifierType(ev->state))) {
-				return true;
-			}
-
-#ifdef GTKOSX
-			if (allow_activating) {
-				int oldval = ev->keyval;
-				ev->keyval = fakekey;
-				if (gdk_quartz_possibly_forward ((GdkEvent*) ev)) {
-					return true;
-				}
-				ev->keyval = oldval;
+#ifdef DEBUG_ACCELERATOR_HANDLING
+			if (debug) {
+				cerr << "\tactivate (was " << ev->keyval << " now " << fakekey << ") without special handling of unmodified accels\n";
 			}
 #endif
+			if (allow_activating && gtk_accel_groups_activate(G_OBJECT(win), fakekey, GdkModifierType(ev->state))) {
+#ifdef DEBUG_ACCELERATOR_HANDLING
+				if (debug) {
+					cerr << "\tactivation handled key\n";
+				}
+#endif
+				return true;
+			}
 		}
 	}
 
@@ -514,12 +615,13 @@ key_press_focus_accelerator_handler (Gtk::Window& window, GdkEventKey* ev)
 #endif
 
 		if (allow_activating) {
-#ifdef GTKOSX
-			if (gdk_quartz_possibly_forward ((GdkEvent*) ev)) {
-				return true;
-			}
-#endif
 			if (gtk_window_activate_key (win, ev)) {
+#ifdef DEBUG_ACCELERATOR_HANDLING
+				if (debug) {
+					cerr << "\tactivation handled key " << ev->keyval << endl;
+				}
+#endif
+				
 				return true;
 			}
 		}
@@ -547,12 +649,6 @@ key_press_focus_accelerator_handler (Gtk::Window& window, GdkEventKey* ev)
 #endif
 
 		if (allow_activating) {
-			
-#ifdef GTKOSX
-			if (gdk_quartz_possibly_forward ((GdkEvent*) ev)) {
-				return true;
-			}
-#endif
 			return gtk_window_activate_key (win, ev);
 		} 
 			
