@@ -135,8 +135,7 @@ class Session : public PBD::StatefulDestructible
 		    SetSlaveSource,
 		    Audition,
 		    InputConfigurationChange,
-		    SetAudioRange,
-		    SetPlayRange,
+		    SetPlayAudioRange,
 		    
 		    /* only one of each of these events
 		       can be queued at any one time
@@ -155,29 +154,34 @@ class Session : public PBD::StatefulDestructible
 
 	    Type           type;
 	    Action         action;
-	    nframes_t action_frame;
-	    nframes_t target_frame;
+	    nframes_t      action_frame;
+	    nframes_t      target_frame;
 	    float          speed;
 
 	    union {
-			void*                ptr;
-			bool                 yes_or_no;
-		        nframes_t            target2_frame;
-			SlaveSource slave;
+		void*                ptr;
+		bool                 yes_or_no;
+		nframes_t            target2_frame;
+		SlaveSource slave;
 	    };
 
+	    union {
+		bool second_yes_or_no;
+	    };
+	    
 	    boost::shared_ptr<Region>     region;
 
 	    list<AudioRange>     audio_range;
 	    list<MusicRange>     music_range;
 
-	    Event(Type t, Action a, nframes_t when, nframes_t where, float spd, bool yn = false)
-		    : type (t), 
-		      action (a),
-		      action_frame (when),
-		      target_frame (where),
-		      speed (spd),
-		      yes_or_no (yn) {}
+	Event(Type t, Action a, nframes_t when, nframes_t where, float spd, bool yn = false, bool yn2 = false)
+		    : type (t)
+		    , action (a)
+		    , action_frame (when)
+		    , target_frame (where)
+		    , speed (spd)
+		    , yes_or_no (yn)
+		    , second_yes_or_no (yn2) {}
 
 	    void set_ptr (void* p) { 
 		    ptr = p;
@@ -241,7 +245,8 @@ class Session : public PBD::StatefulDestructible
 	string export_dir () const;
 
 	void set_snap_name ();
-
+	
+	bool writable() const { return _writable; }
 	void set_dirty ();
 	void set_clean ();
 	bool dirty() const { return _state_of_the_state & Dirty; }
@@ -307,6 +312,8 @@ class Session : public PBD::StatefulDestructible
 		return routes.reader ();
 	}
 
+	boost::shared_ptr<RouteList> get_routes_with_regions_at (nframes64_t const) const;
+
 	uint32_t nroutes() const { return routes.reader()->size(); }
 	uint32_t ntracks () const;
 	uint32_t nbusses () const;
@@ -321,7 +328,9 @@ class Session : public PBD::StatefulDestructible
 	template<class T> void foreach_route (T *obj, void (T::*func)(boost::shared_ptr<Route>));
 	template<class T, class A> void foreach_route (T *obj, void (T::*func)(Route&, A), A arg);
 
-	boost::shared_ptr<Route> route_by_name (string);
+	bool io_name_is_legal (const std::string&);
+
+	boost::shared_ptr<Route> route_by_name (const std::string&);
 	boost::shared_ptr<Route> route_by_id (PBD::ID);
 	boost::shared_ptr<Route> route_by_remote_id (uint32_t id);
 
@@ -370,10 +379,10 @@ class Session : public PBD::StatefulDestructible
 
 	void request_roll_at_and_return (nframes_t start, nframes_t return_to);
 	void request_bounded_roll (nframes_t start, nframes_t end);
-	void request_stop (bool abort = false);
+	void request_stop (bool abort = false, bool clear_state = true);
 	void request_locate (nframes_t frame, bool with_roll = false);
-
-	void request_play_loop (bool yn);
+	void request_transport_speed (float speed);
+	void request_play_loop (bool yn, bool leave_rolling = false);
 	bool get_play_loop () const { return play_loop; }
 
 	nframes_t  last_transport_start() const { return _last_roll_location; }
@@ -383,7 +392,6 @@ class Session : public PBD::StatefulDestructible
 	void set_session_end (nframes_t end) { end_location->set_start(end); _end_location_is_free = false; }
 	void use_rf_shuttle_speed ();
 	void allow_auto_play (bool yn);
-	void request_transport_speed (float speed);
 	void request_overwrite_buffer (Diskstream*);
 	void request_diskstream_speed (Diskstream&, float speed);
 	void request_input_change_handling ();
@@ -437,7 +445,7 @@ class Session : public PBD::StatefulDestructible
 	nframes_t worst_input_latency () const { return _worst_input_latency; }
 	nframes_t worst_track_latency () const { return _worst_track_latency; }
 
-	int save_state (string snapshot_name, bool pending = false);
+	int save_state (string snapshot_name, bool pending = false, bool switch_to_snapshot = false);
 	int restore_state (string snapshot_name);
 	int save_template (string template_name);
         int save_history (string snapshot_name = "");
@@ -605,6 +613,7 @@ class Session : public PBD::StatefulDestructible
 	    
 	    /* control info */
 	    uint32_t total;
+	    uint32_t count;
 	    SrcQuality quality;
 	    volatile bool freeze;
 	    std::vector<Glib::ustring> paths;
@@ -656,6 +665,12 @@ class Session : public PBD::StatefulDestructible
 
 	static sigc::signal<int,nframes_t, nframes_t> AskAboutSampleRateMismatch;
 
+	class SRMismatchRejected : public std::exception {
+	  public:
+		SRMismatchRejected () {}
+		const char* what() const throw() { return "Sample rate mismatch rejected"; }
+	};
+
 	/* handlers should return !0 for use pending state, 0 for
 	   ignore it.
 	*/
@@ -915,17 +930,13 @@ class Session : public PBD::StatefulDestructible
 
 	/* ranges */
 
-	void set_audio_range (list<AudioRange>&);
-	void set_music_range (list<MusicRange>&);
-
-	void request_play_range (bool yn);
+	void request_play_range (list<AudioRange>*, bool leave_rolling = false);
 	bool get_play_range () const { return _play_range; }
 
 	/* favorite dirs */
 	typedef vector<string> FavoriteDirs;
 
 	static int read_favorite_dirs (FavoriteDirs&);
-
 	static int write_favorite_dirs (FavoriteDirs&);
 	
 	/* file suffixes */
@@ -997,6 +1008,8 @@ class Session : public PBD::StatefulDestructible
 
 	nframes_t compute_initial_length ();
 
+	bool   _writable;
+
 	static const char* _template_suffix;
 	static const char* _statefile_suffix;
 	static const char* _pending_suffix;
@@ -1050,6 +1063,7 @@ class Session : public PBD::StatefulDestructible
 	float                   _meter_hold;
 	float                   _meter_falloff;
 	bool                    _end_location_is_free;
+	bool                    _was_seamless;
 
 	void set_worst_io_latencies ();
 	void set_worst_io_latencies_x (IOChange asifwecare, void *ignored) {
@@ -1124,13 +1138,7 @@ class Session : public PBD::StatefulDestructible
 		}
 	}
 
-	bool maybe_stop (nframes_t limit) {
-		if ((_transport_speed > 0.0f && _transport_frame >= limit) || (_transport_speed < 0.0f && _transport_frame == 0)) {
-			stop_transport ();
-			return true;
-		}
-		return false;
-	}
+	bool maybe_stop (nframes_t limit);
 
 	bool maybe_sync_start (nframes_t&);
 
@@ -1174,10 +1182,9 @@ class Session : public PBD::StatefulDestructible
 
 	bool              pending_locate_flush;
 	bool              pending_abort;
+	bool              pending_clear_substate;
 	bool              pending_auto_loop;
 	
-	Sample*           butler_mixdown_buffer;
-	float*            butler_gain_buffer;
 	pthread_t         butler_thread;
 	Glib::Mutex       butler_request_lock;
         Glib::Cond        butler_paused;
@@ -1211,7 +1218,8 @@ class Session : public PBD::StatefulDestructible
 		PostTransportScrub              = 0x8000,
 		PostTransportReverse            = 0x10000,
 		PostTransportInputChange        = 0x20000,
-		PostTransportCurveRealloc       = 0x40000
+		PostTransportCurveRealloc       = 0x40000,
+		PostTransportClearSubstate      = 0x80000
 	};
 	
 	static const PostTransportWork ProcessCannotProceedMask = 
@@ -1222,7 +1230,8 @@ class Session : public PBD::StatefulDestructible
 				   PostTransportScrub|
 				   PostTransportAudition|
 				   PostTransportLocate|
-				   PostTransportStop);
+				   PostTransportStop|
+				   PostTransportClearSubstate);
 	
 	PostTransportWork post_transport_work;
 
@@ -1367,6 +1376,8 @@ class Session : public PBD::StatefulDestructible
 	nframes_t  last_smpte_when;
 	SMPTE::Time last_smpte;
 
+	bool _send_smpte_update; ///< Flag to send a full frame (SMPTE) MTC message this cycle
+	
 	int send_full_time_code ();
 	int send_midi_time_code ();
 
@@ -1444,24 +1455,25 @@ class Session : public PBD::StatefulDestructible
 	int           use_config_midi_ports ();
 
 	void set_play_loop (bool yn);
+	void unset_play_loop ();
 	void overwrite_some_buffers (Diskstream*);
 	void flush_all_redirects ();
 	int  micro_locate (nframes_t distance);
-	void locate (nframes_t, bool with_roll, bool with_flush, bool with_loop=false);
-	void start_locate (nframes_t, bool with_roll, bool with_flush, bool with_loop=false);
+	void locate (nframes_t, bool with_roll, bool with_flush, bool with_loop=false, bool force=false);
+	void start_locate (nframes_t, bool with_roll, bool with_flush, bool with_loop=false, bool force=false);
 	void force_locate (nframes_t frame, bool with_roll = false);
 	void set_diskstream_speed (Diskstream*, float speed);
-	void set_transport_speed (float speed, bool abort = false);
-	void stop_transport (bool abort = false);
+	void set_transport_speed (float speed, bool abort = false, bool clear_state = false);
+	void stop_transport (bool abort = false, bool clear_state = false);
 	void start_transport ();
-	void realtime_stop (bool abort);
+	void realtime_stop (bool abort, bool clear_state);
 	void non_realtime_start_scrub ();
 	void non_realtime_set_speed ();
 	void non_realtime_stop (bool abort, int entry_request_count, bool& finished);
 	void non_realtime_overwrite (int entry_request_count, bool& finished);
 	void butler_transport_work ();
 	void post_transport ();
-	void engine_halted ();
+	void engine_halted (const char*);
 	void xrun_recovery ();
 
 	TempoMap    *_tempo_map;
@@ -1725,8 +1737,8 @@ class Session : public PBD::StatefulDestructible
 
 	list<AudioRange> current_audio_range;
 	bool _play_range;
-	void set_play_range (bool yn);
-	void setup_auto_play ();
+	void set_play_range (std::list<AudioRange>&, bool leave_rolling);
+	void unset_play_range ();
 
 	/* main outs */
 	uint32_t main_outs;
