@@ -103,7 +103,7 @@ class VSTPlugin : public ARDOUR::Plugin
 class VSTPluginInfo : public PluginInfo
 {
   public:
-	VSTPluginInfo () {}
+	VSTPluginInfo ();
 	~VSTPluginInfo () {}
 	
 	PluginPtr load (Session& session);
