@@ -417,10 +417,10 @@ AudioDiskstream::check_record_status (nframes_t transport_frame, nframes_t nfram
 
 			if (_alignment_style == ExistingMaterial) {
 				//cerr << "A FRF += " << _session.worst_output_latency () << endl;
-				// first_recordable_frame += _session.worst_output_latency();
+				first_recordable_frame += _session.worst_output_latency();
 			} else {
 				// cerr << "B FRF += " << _roll_delay<< endl;
-				// first_recordable_frame += _roll_delay;
+				first_recordable_frame += _roll_delay;
   			}
 
 		} else {
@@ -429,49 +429,25 @@ AudioDiskstream::check_record_status (nframes_t transport_frame, nframes_t nfram
 
 			if (_alignment_style == ExistingMaterial) {
 
-				if (!Config->get_punch_in()) {
+				/* manual punch in happens at the correct transport frame
+				    because the user hit a button. but to get alignment correct 
+				    we have to back up the position of the new region to the 
+				    appropriate spot given the roll delay.
+				*/
 
-					/* manual punch in happens at the correct transport frame
-					   because the user hit a button. but to get alignment correct 
-					   we have to back up the position of the new region to the 
-					   appropriate spot given the roll delay.
-					*/
 
-					capture_start_frame -= _roll_delay;
-
-					/* XXX paul notes (august 2005): i don't know why
-					   this is needed.
-					*/
-
-					// cerr << "1 FRF += " << _capture_offset << endl;
-					first_recordable_frame += _capture_offset;
-
-				} else {
-
-					/* autopunch toggles recording at the precise
-					   transport frame, and then the DS waits
-					   to start recording for a time that depends
-					   on the output latency.
-					*/
-
-					// cerr << "2 FRF += " << _session.worst_output_latency() << endl;
-					first_recordable_frame += _session.worst_output_latency();
-				}
+				/* autopunch toggles recording at the precise
+				   transport frame, and then the DS waits
+				   to start recording for a time that depends
+				   on the output latency.
+				*/
 
+				first_recordable_frame += _session.worst_output_latency();
 			} else {
-
-				if (Config->get_punch_in()) {
-					// cerr << "3 FRF += " << _roll_delay << endl;
-					first_recordable_frame += _roll_delay;
-				} else {
-					capture_start_frame -= _roll_delay;
-				}
+				capture_start_frame -= _roll_delay;
 			}
-			
 		}
 
-		// cerr << _name << " FRF = " << first_recordable_frame << " CSF = " << capture_start_frame << endl;
-
 		if (recordable() && destructive()) {
 			boost::shared_ptr<ChannelList> c = channels.reader();
 			for (ChannelList::iterator chan = c->begin(); chan != c->end(); ++chan) {
@@ -983,7 +959,7 @@ AudioDiskstream::seek (nframes_t frame, bool complete_refill)
 		(*chan)->playback_buf->reset ();
 		(*chan)->capture_buf->reset ();
 	}
-	
+
 	/* can't rec-enable in destructive mode if transport is before start */
 	
 	if (destructive() && record_enabled() && frame < _session.current_start_frame()) {
@@ -992,7 +968,7 @@ AudioDiskstream::seek (nframes_t frame, bool complete_refill)
 	
 	playback_sample = frame;
 	file_frame = frame;
-	
+
 	if (complete_refill) {
 		while ((ret = do_refill_with_alloc ()) > 0) ;
 	} else {
@@ -2095,7 +2071,7 @@ AudioDiskstream::reset_write_sources (bool mark_write_complete, bool force)
 	boost::shared_ptr<ChannelList> c = channels.reader();
 	uint32_t n;
 
-	if (!recordable()) {
+	if (!_session.writable() || !recordable()) {
 		return;
 	}
 	
