@@ -1,6 +1,7 @@
 /*
-    Copyright (C) 2006 Paul Davis 
-	
+    Copyright (C) 2006-2009 Paul Davis 
+    Some portions Copyright (C) Sophia Poirier.
+
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
@@ -20,6 +21,7 @@
 #include <sstream>
 #include <errno.h>
 #include <string.h>
+#include <math.h>
 
 #include <pbd/transmitter.h>
 #include <pbd/xml++.h>
@@ -35,6 +37,7 @@
 #include <ardour/io.h>
 #include <ardour/audio_unit.h>
 #include <ardour/session.h>
+#include <ardour/tempo.h>
 #include <ardour/utils.h>
 
 #include <appleutility/CAAudioUnit.h>
@@ -43,6 +46,7 @@
 #include <CoreFoundation/CoreFoundation.h>
 #include <CoreServices/CoreServices.h>
 #include <AudioUnit/AudioUnit.h>
+#include <AudioToolbox/AudioUnitUtilities.h>
 
 #include "i18n.h"
 
@@ -62,6 +66,7 @@ AUPluginInfo::CachedInfoMap AUPluginInfo::cached_info;
 static string preset_search_path = "/Library/Audio/Presets:/Network/Library/Audio/Presets";
 static string preset_suffix = ".aupreset";
 static bool preset_search_path_initialized = false;
+static bool debug_io_config = true;
 
 static OSStatus 
 _render_callback(void *userData,
@@ -71,9 +76,58 @@ _render_callback(void *userData,
 		 UInt32       inNumberFrames,
 		 AudioBufferList*       ioData)
 {
-	return ((AUPlugin*)userData)->render_callback (ioActionFlags, inTimeStamp, inBusNumber, inNumberFrames, ioData);
+	if (userData) {
+		return ((AUPlugin*)userData)->render_callback (ioActionFlags, inTimeStamp, inBusNumber, inNumberFrames, ioData);
+	}
+	return paramErr;
 }
 
+static OSStatus 
+_get_beat_and_tempo_callback (void*                userData,
+			      Float64*             outCurrentBeat, 
+			      Float64*             outCurrentTempo)
+{
+	if (userData) {
+		return ((AUPlugin*)userData)->get_beat_and_tempo_callback (outCurrentBeat, outCurrentTempo);
+	}
+    
+	return paramErr;
+}
+
+static OSStatus 
+_get_musical_time_location_callback (void *     userData,
+				     UInt32 *   outDeltaSampleOffsetToNextBeat,
+				     Float32 *  outTimeSig_Numerator,
+				     UInt32 *   outTimeSig_Denominator,
+				     Float64 *  outCurrentMeasureDownBeat)
+{
+	if (userData) {
+		return ((AUPlugin*)userData)->get_musical_time_location_callback (outDeltaSampleOffsetToNextBeat,
+										  outTimeSig_Numerator,
+										  outTimeSig_Denominator,
+										  outCurrentMeasureDownBeat);
+	}
+	return paramErr;
+}
+
+static OSStatus 
+_get_transport_state_callback (void*     userData,
+			       Boolean*  outIsPlaying,
+			       Boolean*  outTransportStateChanged,
+			       Float64*  outCurrentSampleInTimeLine,
+			       Boolean*  outIsCycling,
+			       Float64*  outCycleStartBeat,
+			       Float64*  outCycleEndBeat)
+{
+	if (userData) {
+		return ((AUPlugin*)userData)->get_transport_state_callback (outIsPlaying, outTransportStateChanged,
+									    outCurrentSampleInTimeLine, outIsCycling,
+									    outCycleStartBeat, outCycleEndBeat);
+	}
+	return paramErr;
+}
+
+
 static int 
 save_property_list (CFPropertyListRef propertyList, Glib::ustring path)
 
@@ -111,7 +165,7 @@ save_property_list (CFPropertyListRef propertyList, Glib::ustring path)
 
 	size_t cnt = CFDataGetLength (xmlData);
 
-	if (write (fd, CFDataGetBytePtr (xmlData), cnt) != cnt) {
+	if (write (fd, CFDataGetBytePtr (xmlData), cnt) != (ssize_t) cnt) {
 		CFRelease (xmlData);
 		close (fd);
 		return -1;
@@ -281,15 +335,17 @@ Boolean ComponentAndDescriptionMatch_Loosely(Component inComponent, const Compon
 
 
 AUPlugin::AUPlugin (AudioEngine& engine, Session& session, boost::shared_ptr<CAComponent> _comp)
-	: Plugin (engine, session),
-	  comp (_comp),
-	  unit (new CAAudioUnit),
-	  initialized (false),
-	  buffers (0),
-	  current_maxbuf (0),
-	  current_offset (0),
-	  current_buffers (0),
-	frames_processed (0)
+	: Plugin (engine, session)
+	, comp (_comp)
+	, unit (new CAAudioUnit)
+	, initialized (false)
+	, buffers (0)
+	, current_maxbuf (0)
+	, current_offset (0)
+	, current_buffers (0)
+	, frames_processed (0)
+	, last_transport_rolling (false)
+	, last_transport_speed (0.0)
 {			
 	if (!preset_search_path_initialized) {
 		Glib::ustring p = Glib::get_home_dir();
@@ -328,6 +384,33 @@ AUPlugin::~AUPlugin ()
 	}
 }
 
+void
+AUPlugin::discover_factory_presets ()
+{
+	CFArrayRef presets;
+	UInt32 dataSize = sizeof (presets);
+	OSStatus err;
+	
+	if ((err = unit->GetProperty (kAudioUnitProperty_FactoryPresets, kAudioUnitScope_Global, 0, (void*) &presets, &dataSize)) != 0) {
+		cerr << "cannot get factory preset info: " << err << endl;
+		return;
+	}
+
+	if (!presets) {
+		return;
+	}
+
+	CFIndex cnt = CFArrayGetCount (presets);
+
+	for (CFIndex i = 0; i < cnt; ++i) {
+		AUPreset* preset = (AUPreset*) CFArrayGetValueAtIndex (presets, i);
+
+		string name = CFStringRefToStdString (preset->presetName);
+		factory_preset_map[name] = preset->presetNumber;
+	}
+	
+	CFRelease (presets);
+}
 
 void
 AUPlugin::init ()
@@ -357,6 +440,22 @@ AUPlugin::init ()
 		throw failed_constructor();
 	}
 
+	/* tell the plugin about tempo/meter/transport callbacks in case it wants them */
+
+	HostCallbackInfo info;
+	memset (&info, 0, sizeof (HostCallbackInfo));
+	info.hostUserData = this;
+	info.beatAndTempoProc = _get_beat_and_tempo_callback;
+	info.musicalTimeLocationProc = _get_musical_time_location_callback;
+	info.transportStateProc = _get_transport_state_callback;
+	
+        //ignore result of this - don't care if the property isn't supported
+	unit->SetProperty (kAudioUnitProperty_HostCallbacks, 
+			   kAudioUnitScope_Global, 
+			   0, //elementID 
+			   &info,
+			   sizeof (HostCallbackInfo));
+
 	unit->GetElementCount (kAudioUnitScope_Global, global_elements);
 	unit->GetElementCount (kAudioUnitScope_Input, input_elements);
 	unit->GetElementCount (kAudioUnitScope_Output, output_elements);
@@ -374,6 +473,7 @@ AUPlugin::init ()
 	}
 
 	discover_parameters ();
+	discover_factory_presets ();
 
 	Plugin::setup_controls ();
 }
@@ -532,6 +632,18 @@ AUPlugin::set_parameter (uint32_t which, float val)
 	if (which < descriptors.size()) {
 		const AUParameterDescriptor& d (descriptors[which]);
 		unit->SetParameter (d.id, d.scope, d.element, val);
+
+		/* tell the world what we did */
+
+		AudioUnitEvent theEvent;
+		
+		theEvent.mEventType = kAudioUnitEvent_ParameterValueChange;
+		theEvent.mArgument.mParameter.mAudioUnit = unit->AU();
+		theEvent.mArgument.mParameter.mParameterID = d.id;
+		theEvent.mArgument.mParameter.mScope = d.scope;
+		theEvent.mArgument.mParameter.mElement = d.element;
+		
+		AUEventListenerNotify (NULL, NULL, &theEvent);
 	}
 }
 
@@ -585,6 +697,7 @@ void
 AUPlugin::deactivate ()
 {
 	unit->GlobalReset ();
+	initialized = false;
 }
 
 void
@@ -622,6 +735,12 @@ int32_t
 AUPlugin::configure_io (int32_t in, int32_t out)
 {
 	AudioStreamBasicDescription streamFormat;
+	bool was_initialized = initialized;
+
+	if (initialized) {
+		unit->Uninitialize ();
+		initialized = false;
+	}
 
 	streamFormat.mSampleRate = _session.frame_rate();
 	streamFormat.mFormatID = kAudioFormatLinearPCM;
@@ -654,7 +773,15 @@ AUPlugin::configure_io (int32_t in, int32_t out)
 		return -1;
 	}
 
-	return Plugin::configure_io (in, out);
+	int ret = Plugin::configure_io (in, out);
+
+	if (ret == 0) {
+		if (was_initialized) {
+			activate ();
+		}
+	}
+
+	return ret;
 }
 
 int32_t
@@ -670,11 +797,19 @@ AUPlugin::can_do (int32_t in, int32_t& out)
 
 	vector<pair<int,int> >& io_configs = pinfo->cache.io_configs;
 
+	if (debug_io_config) {
+		cerr << name() << " has " << io_configs.size() << " IO Configurations\n";
+	}
+
 	for (vector<pair<int,int> >::iterator i = io_configs.begin(); i != io_configs.end(); ++i) {
 
 		int32_t possible_in = i->first;
 		int32_t possible_out = i->second;
 
+		if (debug_io_config) {
+			cerr << "\tin " << possible_in << " out " << possible_out << endl;
+		}
+
 		if (possible_out == 0) {
 			warning << string_compose (_("AU %1 has zero outputs - configuration ignored"), name()) << endmsg;
 			continue;
@@ -805,6 +940,14 @@ AUPlugin::can_do (int32_t in, int32_t& out)
 
 	}
 
+	if (debug_io_config) {
+		if (plugcnt > 0) {
+			cerr << "\tCHOSEN: in " << in << " out " << out << " plugcnt will be " << plugcnt << endl;
+		} else {
+			cerr << "\tFAIL: no configs match requested in " << in << endl;
+		}
+	}
+
 	return plugcnt;
 }
 
@@ -891,11 +1034,12 @@ AUPlugin::render_callback(AudioUnitRenderActionFlags *ioActionFlags,
 		error << _("AUPlugin: render callback called illegally!") << endmsg;
 		return kAudioUnitErr_CannotDoInCurrentContext;
 	}
-
-	for (uint32_t i = 0; i < current_maxbuf; ++i) {
+	uint32_t limit = min ((uint32_t) ioData->mNumberBuffers, current_maxbuf);
+	for (uint32_t i = 0; i < limit; ++i) {
 		ioData->mBuffers[i].mNumberChannels = 1;
 		ioData->mBuffers[i].mDataByteSize = sizeof (Sample) * inNumberFrames;
 		ioData->mBuffers[i].mData = (*current_buffers)[i] + cb_offset + current_offset;
+		// cerr << "chn " << i << " rendering from " << ioData->mBuffers[i].mData << endl;
 	}
 
 	cb_offset += inNumberFrames;
@@ -914,9 +1058,10 @@ AUPlugin::connect_and_run (vector<Sample*>& bufs, uint32_t maxbuf, int32_t& in,
 	current_offset = offset;
 	cb_offset = 0;
 
-	buffers->mNumberBuffers = maxbuf;
+	buffers->mNumberBuffers = min ((uint32_t) output_channels, maxbuf);
+	// cerr << "Will render " << buffers->mNumberBuffers << " channels\n";
 
-	for (uint32_t i = 0; i < maxbuf; ++i) {
+	for (uint32_t i = 0; i < buffers->mNumberBuffers; ++i) {
 		buffers->mBuffers[i].mNumberChannels = 1;
 		buffers->mBuffers[i].mDataByteSize = nframes * sizeof (Sample);
 		buffers->mBuffers[i].mData = 0;
@@ -929,9 +1074,12 @@ AUPlugin::connect_and_run (vector<Sample*>& bufs, uint32_t maxbuf, int32_t& in,
 
 		current_maxbuf = 0;
 		frames_processed += nframes;
-		
-		for (uint32_t i = 0; i < maxbuf; ++i) {
+
+		uint32_t limit = min ((uint32_t) buffers->mNumberBuffers, maxbuf);
+
+		for (uint32_t i = 0; i < limit; ++i) {
 			if (bufs[i] + offset != buffers->mBuffers[i].mData) {
+				// cerr << "chn " << i << " rendered into " << bufs[i]+offset << endl;
 				memcpy (bufs[i]+offset, buffers->mBuffers[i].mData, nframes * sizeof (Sample));
 			}
 		}
@@ -941,6 +1089,183 @@ AUPlugin::connect_and_run (vector<Sample*>& bufs, uint32_t maxbuf, int32_t& in,
 	return -1;
 }
 
+OSStatus 
+AUPlugin::get_beat_and_tempo_callback (Float64* outCurrentBeat, 
+				       Float64* outCurrentTempo)
+{
+	TempoMap& tmap (_session.tempo_map());
+
+	/* more than 1 meter or more than 1 tempo means that a simplistic computation 
+	   (and interpretation) of a beat position will be incorrect. So refuse to 
+	   offer the value.
+	*/
+
+	if (tmap.n_tempos() > 1 || tmap.n_meters() > 1) {
+		return kAudioUnitErr_CannotDoInCurrentContext;
+	}
+
+	BBT_Time bbt;
+	TempoMap::Metric metric = tmap.metric_at (_session.transport_frame() + current_offset);
+	tmap.bbt_time_with_metric (_session.transport_frame() + current_offset, bbt, metric);
+
+	if (outCurrentBeat) {
+		float beat;
+		beat = metric.meter().beats_per_bar() * bbt.bars;
+		beat += bbt.beats;
+		beat += bbt.ticks / Meter::ticks_per_beat;
+		*outCurrentBeat = beat;
+	}
+
+	if (outCurrentTempo) {
+		*outCurrentTempo = floor (metric.tempo().beats_per_minute());
+	}
+
+	return noErr;
+
+}
+
+OSStatus 
+AUPlugin::get_musical_time_location_callback (UInt32*   outDeltaSampleOffsetToNextBeat,
+					      Float32*  outTimeSig_Numerator,
+					      UInt32*   outTimeSig_Denominator,
+					      Float64*  outCurrentMeasureDownBeat)
+{
+	TempoMap& tmap (_session.tempo_map());
+
+	/* more than 1 meter or more than 1 tempo means that a simplistic computation 
+	   (and interpretation) of a beat position will be incorrect. So refuse to 
+	   offer the value.
+	*/
+
+	if (tmap.n_tempos() > 1 || tmap.n_meters() > 1) {
+		return kAudioUnitErr_CannotDoInCurrentContext;
+	}
+
+	BBT_Time bbt;
+	TempoMap::Metric metric = tmap.metric_at (_session.transport_frame() + current_offset);
+	tmap.bbt_time_with_metric (_session.transport_frame() + current_offset, bbt, metric);
+
+	if (outDeltaSampleOffsetToNextBeat) {
+		if (bbt.ticks == 0) {
+			/* on the beat */
+			*outDeltaSampleOffsetToNextBeat = 0;
+		} else {
+			*outDeltaSampleOffsetToNextBeat = (UInt32) floor (((Meter::ticks_per_beat - bbt.ticks)/Meter::ticks_per_beat) * // fraction of a beat to next beat
+									  metric.tempo().frames_per_beat(_session.frame_rate(), metric.meter())); // frames per beat
+		}
+	}
+	
+	if (outTimeSig_Numerator) {
+		*outTimeSig_Numerator = (UInt32) lrintf (metric.meter().beats_per_bar());
+	}
+	if (outTimeSig_Denominator) {
+		*outTimeSig_Denominator = (UInt32) lrintf (metric.meter().note_divisor());
+	}
+
+	if (outCurrentMeasureDownBeat) {
+
+		/* beat for the start of the bar. 
+		   1|1|0 -> 1
+		   2|1|0 -> 1 + beats_per_bar
+		   3|1|0 -> 1 + (2 * beats_per_bar)
+		   etc. 
+		*/
+
+		*outCurrentMeasureDownBeat = 1 + metric.meter().beats_per_bar() * (bbt.bars - 1);
+	}
+
+	return noErr;
+}
+
+OSStatus 
+AUPlugin::get_transport_state_callback (Boolean*  outIsPlaying,
+					Boolean*  outTransportStateChanged,
+					Float64*  outCurrentSampleInTimeLine,
+					Boolean*  outIsCycling,
+					Float64*  outCycleStartBeat,
+					Float64*  outCycleEndBeat)
+{
+	bool rolling;
+	float speed;
+
+	rolling = _session.transport_rolling();
+	speed = _session.transport_speed ();
+
+	if (outIsPlaying) {
+		*outIsPlaying = _session.transport_rolling();
+	}
+
+	if (outTransportStateChanged) {
+		if (rolling != last_transport_rolling) {
+			*outTransportStateChanged = true;
+		} else if (speed != last_transport_speed) {
+			*outTransportStateChanged = true;
+		} else {
+			*outTransportStateChanged = false;
+		}
+	}
+
+	if (outCurrentSampleInTimeLine) {
+		/* this assumes that the AU can only call this host callback from render context,
+		   where current_offset is valid.
+		*/
+		*outCurrentSampleInTimeLine = _session.transport_frame() + current_offset;
+	}
+
+	if (outIsCycling) {
+		Location* loc = _session.locations()->auto_loop_location();
+
+		*outIsCycling = (loc && _session.transport_rolling() && _session.get_play_loop());
+
+		if (*outIsCycling) {
+
+			if (outCycleStartBeat || outCycleEndBeat) {
+
+				TempoMap& tmap (_session.tempo_map());
+
+				/* more than 1 meter means that a simplistic computation (and interpretation) of 
+				   a beat position will be incorrect. so refuse to offer the value.
+				*/
+
+				if (tmap.n_meters() > 1) {
+					return kAudioUnitErr_CannotDoInCurrentContext;
+				}
+				
+				BBT_Time bbt;
+
+				if (outCycleStartBeat) {
+					TempoMap::Metric metric = tmap.metric_at (loc->start() + current_offset);
+					_session.tempo_map().bbt_time_with_metric (loc->start(), bbt, metric);
+
+					float beat;
+					beat = metric.meter().beats_per_bar() * bbt.bars;
+					beat += bbt.beats;
+					beat += bbt.ticks / Meter::ticks_per_beat;
+					
+					*outCycleStartBeat = beat;
+				}
+
+				if (outCycleEndBeat) {
+					TempoMap::Metric metric = tmap.metric_at (loc->end() + current_offset);
+					_session.tempo_map().bbt_time_with_metric (loc->end(), bbt, metric);
+					
+					float beat;
+					beat = metric.meter().beats_per_bar() * bbt.bars;
+					beat += bbt.beats;
+					beat += bbt.ticks / Meter::ticks_per_beat;
+					
+					*outCycleEndBeat = beat;
+				}
+			}
+		}
+	}
+
+	last_transport_rolling = rolling;
+	last_transport_speed = speed;
+
+	return noErr;
+}
+
 set<uint32_t>
 AUPlugin::automatable() const
 {
@@ -1076,6 +1401,13 @@ AUPlugin::set_state(const XMLNode& node)
 	if (propertyList) {
 		if (unit->SetAUPreset (propertyList) == noErr) {
 			ret = 0;
+			
+			/* tell the world */
+
+			AudioUnitParameter changedUnit;
+			changedUnit.mAudioUnit = unit->AU();
+			changedUnit.mParameterID = kAUParameterListener_AnyParameter;
+			AUParameterListenerNotify (NULL, NULL, &changedUnit);
 		} 
 		CFRelease (propertyList);
 	}
@@ -1097,19 +1429,48 @@ AUPlugin::load_preset (const string preset_label)
 	bool ret = false;
 	CFPropertyListRef propertyList;
 	Glib::ustring path;
-	PresetMap::iterator x = preset_map.find (preset_label);
+	UserPresetMap::iterator ux;
+	FactoryPresetMap::iterator fx;
 
-	if (x == preset_map.end()) {
-		return false;
-	}
+	/* look first in "user" presets */
+
+	if ((ux = user_preset_map.find (preset_label)) != user_preset_map.end()) {
 	
-	if ((propertyList = load_property_list (x->second)) != 0) {
-		if (unit->SetAUPreset (propertyList) == noErr) {
+		if ((propertyList = load_property_list (ux->second)) != 0) {
+			if (unit->SetAUPreset (propertyList) == noErr) {
+				ret = true;
+
+				/* tell the world */
+				
+				AudioUnitParameter changedUnit;
+				changedUnit.mAudioUnit = unit->AU();
+				changedUnit.mParameterID = kAUParameterListener_AnyParameter;
+				AUParameterListenerNotify (NULL, NULL, &changedUnit);
+			}
+			CFRelease(propertyList);
+		}
+
+	} else if ((fx = factory_preset_map.find (preset_label)) != factory_preset_map.end()) {
+		
+		AUPreset preset;
+		
+		preset.presetNumber = fx->second;
+		preset.presetName = CFStringCreateWithCString (kCFAllocatorDefault, fx->first.c_str(), kCFStringEncodingUTF8);
+		
+		cerr << "Setting factory preset " << fx->second << endl;
+
+		if (unit->SetPresentPreset (preset) == 0) {
 			ret = true;
+
+			/* tell the world */
+
+			AudioUnitParameter changedUnit;
+			changedUnit.mAudioUnit = unit->AU();
+			changedUnit.mParameterID = kAUParameterListener_AnyParameter;
+			AUParameterListenerNotify (NULL, NULL, &changedUnit);
 		}
-		CFRelease(propertyList);
 	}
-	
+		
 	return ret;
 #else
 	if (!seen_loading_message) {
@@ -1360,10 +1721,14 @@ AUPlugin::current_preset() const
 vector<string>
 AUPlugin::get_presets ()
 {
-	vector<string*>* preset_files;
 	vector<string> presets;
+
+#ifdef AU_STATE_SUPPORT
+	vector<string*>* preset_files;
 	PathScanner scanner;
 
+	user_preset_map.clear ();
+
 	preset_files = scanner (preset_search_path, au_preset_filter, this, true, true, -1, true);
 	
 	if (!preset_files) {
@@ -1386,14 +1751,27 @@ AUPlugin::get_presets ()
 		*/
 
 		if (check_and_get_preset_name (get_comp()->Comp(), path, preset_name)) {
-			presets.push_back (preset_name);
-			preset_map[preset_name] = path;
+			user_preset_map[preset_name] = path;
 		} 
 
 		delete *x;
 	}
 
 	delete preset_files;
+
+        /* now fill the vector<string> with the names we have */
+
+	for (UserPresetMap::iterator i = user_preset_map.begin(); i != user_preset_map.end(); ++i) {
+		presets.push_back (i->first);
+	}
+
+        /* add factory presets */
+
+	for (FactoryPresetMap::iterator i = factory_preset_map.begin(); i != factory_preset_map.end(); ++i) {
+		presets.push_back (i->first);
+	}
+
+#endif
 	
 	return presets;
 }
@@ -1409,11 +1787,12 @@ AUPlugin::has_editor () const
 AUPluginInfo::AUPluginInfo (boost::shared_ptr<CAComponentDescription> d)
 	: descriptor (d)
 {
-
+	type = ARDOUR::AudioUnit;
 }
 
 AUPluginInfo::~AUPluginInfo ()
 {
+	type = ARDOUR::AudioUnit;
 }
 
 PluginPtr
@@ -1662,9 +2041,7 @@ AUPluginInfo::discover_by_description (PluginInfoList& plugs, CAComponentDescrip
 			info->n_inputs = info->cache.io_configs.front().first;
 			info->n_outputs = info->cache.io_configs.front().second;
 
-			if (info->cache.io_configs.size() > 1) {
-				cerr << "ODD: variable IO config for " << info->unique_id << endl;
-			}
+			cerr << "detected AU: " << info->name.c_str() << "  (" << info->cache.io_configs.size() << " i/o configurations) - " << info->unique_id << endl;
 
 			plugs.push_back (info);
 
@@ -1691,7 +2068,7 @@ AUPluginInfo::cached_io_configuration (const std::string& unique_id,
 	   follow Apple "guidelines".
 	 */
 
-	snprintf (buf, sizeof (buf), "%u", version);
+	snprintf (buf, sizeof (buf), "%u", (uint32_t) version);
 	id = unique_id;
 	id += '/';
 	id += buf;
@@ -1757,12 +2134,15 @@ AUPluginInfo::add_cached_info (const std::string& id, AUPluginCachedInfo& cinfo)
 	cached_info[id] = cinfo;
 }
 
+#define AU_CACHE_VERSION "2.0"
+
 void
 AUPluginInfo::save_cached_info ()
 {
 	XMLNode* node;
 
 	node = new XMLNode (X_("AudioUnitPluginCache"));
+	node->add_property( "version", AU_CACHE_VERSION );
 	
 	for (map<string,AUPluginCachedInfo>::iterator i = cached_info.begin(); i != cached_info.end(); ++i) {
 		XMLNode* parent = new XMLNode (X_("plugin"));
@@ -1804,13 +2184,24 @@ AUPluginInfo::load_cached_info ()
 		return 0;
 	}
 
-	tree.read (path);
+	if ( !tree.read (path) ) {
+		error << "au_cache is not a valid XML file.  AU plugins will be re-scanned" << endmsg;
+		return -1;
+	}
+	
 	const XMLNode* root (tree.root());
 
 	if (root->name() != X_("AudioUnitPluginCache")) {
 		return -1;
 	}
-
+	
+	//initial version has incorrectly stored i/o info, and/or garbage chars.
+	const XMLProperty* version = root->property(X_("version"));
+	if (! ((version != NULL) && (version->value() == X_(AU_CACHE_VERSION)))) {
+		error << "au_cache is not correct version.  AU plugins will be re-scanned" << endmsg;
+		return -1;
+	}
+	
 	cached_info.clear ();
 
 	const XMLNodeList children = root->children();
@@ -1846,7 +2237,7 @@ AUPluginInfo::load_cached_info ()
 					if (((iprop = gchild->property (X_("in"))) != 0) &&
 					    ((oprop = gchild->property (X_("out"))) != 0)) {
 						in = atoi (iprop->value());
-						out = atoi (iprop->value());
+						out = atoi (oprop->value());
 						
 						cinfo.io_configs.push_back (pair<int,int> (in, out));
 					}
@@ -1907,11 +2298,12 @@ AUPluginInfo::get_names (CAComponentDescription& comp_desc, std::string& name, G
 	if (colon) {
 		name = str.substr (colon+1);
 		maker = str.substr (0, colon);
-		// strip_whitespace_edges (maker);
-		// strip_whitespace_edges (name);
+		strip_whitespace_edges (maker);
+		strip_whitespace_edges (name);
 	} else {
 		name = str;
 		maker = "unknown";
+		strip_whitespace_edges (name);
 	}
 }
 
