@@ -43,6 +43,23 @@ using namespace ARDOUR;
 using namespace PBD;
 
 gint AudioEngine::m_meter_exit;
+AudioEngine* AudioEngine::_instance = 0;
+
+#define GET_PRIVATE_JACK_POINTER(j)  jack_client_t* _priv_jack = (jack_client_t*) (j); if (!_priv_jack) { return; }
+#define GET_PRIVATE_JACK_POINTER_RET(j,r) jack_client_t* _priv_jack = (jack_client_t*) (j); if (!_priv_jack) { return r; }
+
+typedef void (*_JackInfoShutdownCallback)(jack_status_t code, const char* reason, void *arg);
+
+static void (*on_info_shutdown)(jack_client_t*, _JackInfoShutdownCallback, void *);
+extern void jack_on_info_shutdown (jack_client_t*, _JackInfoShutdownCallback, void *) __attribute__((weak));
+
+static void check_jack_symbols () __attribute__((constructor));
+
+void check_jack_symbols ()
+{
+	/* use weak linking to see if we really have various late-model JACK function */
+	on_info_shutdown = jack_on_info_shutdown;
+}
 
 static void 
 ardour_jack_error (const char* msg) 
@@ -54,6 +71,8 @@ ardour_jack_error (const char* msg)
 AudioEngine::AudioEngine (string client_name) 
 	: ports (new Ports)
 {
+	_instance = this; /* singleton */
+
 	session = 0;
 	session_remove_pending = false;
 	_running = false;
@@ -104,14 +123,11 @@ _thread_init_callback (void *arg)
 int
 AudioEngine::start ()
 {
-	if (!_jack) {
-		error << _("AudioEngine::start() called while disconnected from JACK") << endmsg;
-		return -1;
-	}
+        GET_PRIVATE_JACK_POINTER_RET (_jack, -1);
 
 	if (!_running) {
 
-		nframes_t blocksize = jack_get_buffer_size (_jack);
+		nframes_t blocksize = jack_get_buffer_size (_priv_jack);
 		Port::set_buffer_size (blocksize);
 
 		if (session) {
@@ -119,7 +135,7 @@ AudioEngine::start ()
 			BootMessage (_("Connect session to engine"));
 
 			session->set_block_size (blocksize);
-			session->set_frame_rate (jack_get_sample_rate (_jack));
+			session->set_frame_rate (jack_get_sample_rate (_priv_jack));
 
 			/* page in as much of the session process code as we
 			   can before we really start running.
@@ -138,21 +154,26 @@ AudioEngine::start ()
 		_processed_frames = 0;
 		last_monitor_check = 0;
 
-		jack_on_shutdown (_jack, halted, this);
-		jack_set_graph_order_callback (_jack, _graph_order_callback, this);
-		jack_set_thread_init_callback (_jack, _thread_init_callback, this);
-		jack_set_process_callback (_jack, _process_callback, this);
-		jack_set_sample_rate_callback (_jack, _sample_rate_callback, this);
-		jack_set_buffer_size_callback (_jack, _bufsize_callback, this);
-		jack_set_xrun_callback (_jack, _xrun_callback, this);
-		jack_set_sync_callback (_jack, _jack_sync_callback, this);
-		jack_set_freewheel_callback (_jack, _freewheel_callback, this);
-
-		if (Config->get_jack_time_master()) {
-			jack_set_timebase_callback (_jack, 0, _jack_timebase_callback, this);
+		if (on_info_shutdown) {
+			on_info_shutdown (_priv_jack, halted_info, this);
+		} else {
+			jack_on_shutdown (_priv_jack, halted, this);
 		}
 
-		if (jack_activate (_jack) == 0) {
+		jack_set_graph_order_callback (_priv_jack, _graph_order_callback, this);
+		jack_set_thread_init_callback (_priv_jack, _thread_init_callback, this);
+		jack_set_process_callback (_priv_jack, _process_callback, this);
+		jack_set_sample_rate_callback (_priv_jack, _sample_rate_callback, this);
+		jack_set_buffer_size_callback (_priv_jack, _bufsize_callback, this);
+		jack_set_xrun_callback (_priv_jack, _xrun_callback, this);
+		jack_set_sync_callback (_priv_jack, _jack_sync_callback, this);
+		jack_set_freewheel_callback (_priv_jack, _freewheel_callback, this);
+
+		if (Config->get_jack_time_master()) {
+			jack_set_timebase_callback (_priv_jack, 0, _jack_timebase_callback, this);
+		}
+
+		if (jack_activate (_priv_jack) == 0) {
 			_running = true;
 			_has_run = true;
 			Running(); /* EMIT SIGNAL */
@@ -169,11 +190,13 @@ AudioEngine::start ()
 int
 AudioEngine::stop (bool forever)
 {
-	if (_jack) {
+        GET_PRIVATE_JACK_POINTER_RET (_jack, -1);
+
+	if (_priv_jack) {
 		if (forever) {
 			disconnect_from_jack ();
 		} else {
-			jack_deactivate (_jack);
+			jack_deactivate (_priv_jack);
 			Stopped(); /* EMIT SIGNAL */
 		}
 	}
@@ -188,10 +211,12 @@ AudioEngine::get_sync_offset (nframes_t& offset) const
 
 #ifdef HAVE_JACK_VIDEO_SUPPORT
 
+        GET_PRIVATE_JACK_POINTER_RET (_jack, false);
+
 	jack_position_t pos;
 	
-	if (_jack) {
-		(void) jack_transport_query (_jack, &pos);
+	if (_priv_jack) {
+		(void) jack_transport_query (_priv_jack, &pos);
 		
 		if (pos.valid & JackVideoFrameOffset) {
 			offset = pos.video_offset;
@@ -272,6 +297,7 @@ int
 AudioEngine::process_callback (nframes_t nframes)
 {
 	// CycleTimer ct ("AudioEngine::process");
+        GET_PRIVATE_JACK_POINTER_RET (_jack, -1);
 	Glib::Mutex::Lock tm (_process_lock, Glib::TRY_LOCK);
 	nframes_t next_processed_frames;
 	
@@ -306,7 +332,7 @@ AudioEngine::process_callback (nframes_t nframes)
 
 	if (_freewheeling) {
 		if (Freewheel (nframes)) {
-			jack_set_freewheel (_jack, false);
+			jack_set_freewheel (_priv_jack, false);
 		}
 		return 0;
 	}
@@ -402,7 +428,6 @@ AudioEngine::jack_bufsize_callback (nframes_t nframes)
 	_usecs_per_cycle = (int) floor ((((double) nframes / frame_rate())) * 1000000.0);
 	last_monitor_check = 0;
 
-	cerr << "bufsize: Set Port buffer size to " << nframes << endl;
 	Port::set_buffer_size (nframes);
 
 	boost::shared_ptr<Ports> p = ports.reader();
@@ -498,12 +523,13 @@ AudioEngine::remove_session ()
 void
 AudioEngine::port_registration_failure (const std::string& portname)
 {
+        GET_PRIVATE_JACK_POINTER (_jack);
 	string full_portname = jack_client_name;
 	full_portname += ':';
 	full_portname += portname;
 	
 	
-	jack_port_t* p = jack_port_by_name (_jack, full_portname.c_str());
+	jack_port_t* p = jack_port_by_name (_priv_jack, full_portname.c_str());
 	string reason;
 	
 	if (p) {
@@ -518,6 +544,7 @@ AudioEngine::port_registration_failure (const std::string& portname)
 Port *
 AudioEngine::register_input_port (DataType type, const string& portname)
 {
+        GET_PRIVATE_JACK_POINTER_RET (_jack,   0);
 	if (!_running) {
 		if (!_has_run) {
 			fatal << _("register input port called before engine was started") << endmsg;
@@ -527,7 +554,7 @@ AudioEngine::register_input_port (DataType type, const string& portname)
 		}
 	}
 
-	jack_port_t *p = jack_port_register (_jack, portname.c_str(), type.to_jack_type(), JackPortIsInput, 0);
+	jack_port_t *p = jack_port_register (_priv_jack, portname.c_str(), type.to_jack_type(), JackPortIsInput, 0);
 
 	if (p) {
 
@@ -552,6 +579,8 @@ AudioEngine::register_input_port (DataType type, const string& portname)
 Port *
 AudioEngine::register_output_port (DataType type, const string& portname)
 {
+        GET_PRIVATE_JACK_POINTER_RET (_jack, 0);
+
 	if (!_running) {
 		if (!_has_run) {
 			fatal << _("register output port called before engine was started") << endmsg;
@@ -563,7 +592,7 @@ AudioEngine::register_output_port (DataType type, const string& portname)
 
 	jack_port_t *p;
 
-	if ((p = jack_port_register (_jack, portname.c_str(),
+	if ((p = jack_port_register (_priv_jack, portname.c_str(),
 		type.to_jack_type(), JackPortIsOutput, 0)) != 0) {
 
 		Port *newport = 0;
@@ -591,6 +620,8 @@ AudioEngine::register_output_port (DataType type, const string& portname)
 int          
 AudioEngine::unregister_port (Port *port)
 {
+        GET_PRIVATE_JACK_POINTER_RET (_jack,-1);
+
 	if (!_running) { 
 		/* probably happening when the engine has been halted by JACK,
 		   in which case, there is nothing we can do here.
@@ -600,7 +631,7 @@ AudioEngine::unregister_port (Port *port)
 
 	if (port) {
 
-		int ret = jack_port_unregister (_jack, port->_port);
+		int ret = jack_port_unregister (_priv_jack, port->_port);
 		
 		if (ret == 0) {
 			
@@ -632,19 +663,12 @@ AudioEngine::unregister_port (Port *port)
 int 
 AudioEngine::connect (const string& source, const string& destination)
 {
-	if (!_running) {
-		if (!_has_run) {
-			fatal << _("connect called before engine was started") << endmsg;
-			/*NOTREACHED*/
-		} else {
-			return -1;
-		}
-	}
+        GET_PRIVATE_JACK_POINTER_RET (_jack, -1);
 	
 	string s = make_port_name_non_relative (source);
 	string d = make_port_name_non_relative (destination);
 
-	int ret = jack_connect (_jack, s.c_str(), d.c_str());
+	int ret = jack_connect (_priv_jack, s.c_str(), d.c_str());
 
 	if (ret == 0) {
 		pair<string,string> c (s, d);
@@ -665,6 +689,8 @@ AudioEngine::connect (const string& source, const string& destination)
 int 
 AudioEngine::disconnect (const string& source, const string& destination)
 {
+        GET_PRIVATE_JACK_POINTER_RET (_jack,-1);
+     
 	if (!_running) {
 		if (!_has_run) {
 			fatal << _("disconnect called before engine was started") << endmsg;
@@ -677,7 +703,7 @@ AudioEngine::disconnect (const string& source, const string& destination)
 	string s = make_port_name_non_relative (source);
 	string d = make_port_name_non_relative (destination);
 
-	int ret = jack_disconnect (_jack, s.c_str(), d.c_str());
+	int ret = jack_disconnect (_priv_jack, s.c_str(), d.c_str());
 
 	if (ret == 0) {
 		pair<string,string> c (s, d);
@@ -694,6 +720,8 @@ AudioEngine::disconnect (const string& source, const string& destination)
 int
 AudioEngine::disconnect (Port *port)
 {
+        GET_PRIVATE_JACK_POINTER_RET (_jack,-1);
+
 	if (!_running) {
 		if (!_has_run) {
 			fatal << _("disconnect called before engine was started") << endmsg;
@@ -703,7 +731,7 @@ AudioEngine::disconnect (Port *port)
 		}
 	}
 
-	int ret = jack_port_disconnect (_jack, port->_port);
+	int ret = jack_port_disconnect (_priv_jack, port->_port);
 
 	if (ret == 0) {
 		remove_connections_for (port);
@@ -716,34 +744,22 @@ AudioEngine::disconnect (Port *port)
 nframes_t
 AudioEngine::frame_rate ()
 {
-	if (_jack) {
-		if (_frame_rate == 0) {
-			return (_frame_rate = jack_get_sample_rate (_jack));
-		} else {
-			return _frame_rate;
-		}
+        GET_PRIVATE_JACK_POINTER_RET (_jack,0);
+	if (_frame_rate == 0) {
+	  return (_frame_rate = jack_get_sample_rate (_priv_jack));
 	} else {
-		fatal << X_("programming error: AudioEngine::frame_rate() called while disconnected from JACK")
-		      << endmsg;
-		/*NOTREACHED*/
-		return 0;
+	  return _frame_rate;
 	}
 }
 
 nframes_t
 AudioEngine::frames_per_cycle ()
 {
-	if (_jack) {
-		if (_buffer_size == 0) {
-			return (_buffer_size = jack_get_buffer_size (_jack));
-		} else {
-			return _buffer_size;
-		}
+        GET_PRIVATE_JACK_POINTER_RET (_jack,0);
+	if (_buffer_size == 0) {
+	  return (_buffer_size = jack_get_buffer_size (_jack));
 	} else {
-		fatal << X_("programming error: AudioEngine::frame_rate() called while disconnected from JACK")
-		      << endmsg;
-		/*NOTREACHED*/
-		return 0;
+	  return _buffer_size;
 	}
 }
 
@@ -751,6 +767,7 @@ Port *
 AudioEngine::get_port_by_name (const string& portname, bool keep)
 {
 	Glib::Mutex::Lock lm (_process_lock);
+        GET_PRIVATE_JACK_POINTER_RET (_jack,0);
 
 	if (!_running) {
 		if (!_has_run) {
@@ -773,7 +790,7 @@ AudioEngine::get_port_by_name (const string& portname, bool keep)
 
 	jack_port_t *p;
 
-	if ((p = jack_port_by_name (_jack, portname.c_str())) != 0) {
+	if ((p = jack_port_by_name (_priv_jack, portname.c_str())) != 0) {
 		Port *newport = new Port (p);
 
 		{
@@ -796,6 +813,7 @@ AudioEngine::get_port_by_name (const string& portname, bool keep)
 const char **
 AudioEngine::get_ports (const string& port_name_pattern, const string& type_name_pattern, uint32_t flags)
 {
+        GET_PRIVATE_JACK_POINTER_RET (_jack,0);
 	if (!_running) {
 		if (!_has_run) {
 			fatal << _("get_ports called before engine was started") << endmsg;
@@ -804,12 +822,14 @@ AudioEngine::get_ports (const string& port_name_pattern, const string& type_name
 			return 0;
 		}
 	}
-	return jack_get_ports (_jack, port_name_pattern.c_str(), type_name_pattern.c_str(), flags);
+	return jack_get_ports (_priv_jack, port_name_pattern.c_str(), type_name_pattern.c_str(), flags);
 }
 
 void
 AudioEngine::halted (void *arg)
 {
+        /* called from jack shutdown handler  */
+
 	AudioEngine* ae = static_cast<AudioEngine *> (arg);
 	bool was_running = ae->_running;
 
@@ -821,20 +841,60 @@ AudioEngine::halted (void *arg)
 	ae->_jack = 0;
 
 	if (was_running) {
-		ae->Halted(); /* EMIT SIGNAL */
+		ae->Halted(""); /* EMIT SIGNAL */
 	}
 }
 
+void
+AudioEngine::halted_info (jack_status_t code, const char* reason, void *arg)
+{
+        /* called from jack shutdown handler  */
+
+	AudioEngine* ae = static_cast<AudioEngine *> (arg);
+	bool was_running = ae->_running;
+
+	ae->stop_metering_thread ();
+
+	ae->_running = false;
+	ae->_buffer_size = 0;
+	ae->_frame_rate = 0;
+	ae->_jack = 0;
+
+	if (was_running) {
+#ifdef HAVE_JACK_ON_INFO_SHUTDOWN
+		switch (code) {
+		case JackBackendError:
+			ae->Halted(reason); /* EMIT SIGNAL */
+			break;
+		default:
+			ae->Halted(""); /* EMIT SIGNAL */
+		}
+#else
+		ae->Halted(""); /* EMIT SIGNAL */
+#endif
+	}
+}
+
+void
+AudioEngine::died ()
+{
+        /* called from a signal handler for SIGPIPE */
+
+	stop_metering_thread ();
+
+        _running = false;
+	_buffer_size = 0;
+	_frame_rate = 0;
+	_jack = 0;
+}
+
 bool
 AudioEngine::can_request_hardware_monitoring () 
 {
+        GET_PRIVATE_JACK_POINTER_RET (_jack,false);
 	const char ** ports;
 
-	if (!_jack) {
-		return 0;
-	}
-
-	if ((ports = jack_get_ports (_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortCanMonitor)) == 0) {
+	if ((ports = jack_get_ports (_priv_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortCanMonitor)) == 0) {
 		return false;
 	}
 
@@ -847,14 +907,11 @@ AudioEngine::can_request_hardware_monitoring ()
 uint32_t
 AudioEngine::n_physical_audio_outputs () const
 {
+        GET_PRIVATE_JACK_POINTER_RET (_jack,0);
 	const char ** ports;
 	uint32_t i = 0;
 
-	if (!_jack) {
-		return 0;
-	}
-
-	if ((ports = jack_get_ports (_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsInput)) == 0) {
+	if ((ports = jack_get_ports (_priv_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsInput)) == 0) {
 		return 0;
 	}
 
@@ -867,84 +924,66 @@ AudioEngine::n_physical_audio_outputs () const
 uint32_t
 AudioEngine::n_physical_audio_inputs () const
 {
+        GET_PRIVATE_JACK_POINTER_RET (_jack,0);
 	const char ** ports;
 	uint32_t i = 0;
 	
-	if (!_jack) {
-		return 0;
-	}
-	
-	if ((ports = jack_get_ports (_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsOutput)) == 0) {
+	if ((ports = jack_get_ports (_priv_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsOutput)) == 0) {
 		return 0;
 	}
 
-	if (ports) {
-		for (i = 0; ports[i]; ++i);
-		free (ports);
-	}
+	for (i = 0; ports[i]; ++i);
+	free (ports);
+
 	return i;
 }
 
 void
 AudioEngine::get_physical_audio_inputs (vector<string>& ins)
 {
+        GET_PRIVATE_JACK_POINTER (_jack);
 	const char ** ports;
 	uint32_t i = 0;
 	
-	if (!_jack) {
-		return;
-	}
-	
-	if ((ports = jack_get_ports (_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsOutput)) == 0) {
+	if ((ports = jack_get_ports (_priv_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsOutput)) == 0) {
 		return;
 	}
 
-	if (ports) {
-		for (i = 0; ports[i]; ++i) {
-			ins.push_back (ports[i]);
-		}
-		free (ports);
+	for (i = 0; ports[i]; ++i) {
+		ins.push_back (ports[i]);
 	}
+
+	free (ports);
 }
 
 void
 AudioEngine::get_physical_audio_outputs (vector<string>& outs)
 {
+        GET_PRIVATE_JACK_POINTER (_jack);
 	const char ** ports;
 	uint32_t i = 0;
 	
-	if (!_jack) {
-		return;
-	}
-	
-	if ((ports = jack_get_ports (_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsInput)) == 0) {
+	if ((ports = jack_get_ports (_priv_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|JackPortIsInput)) == 0) {
 		return;
 	}
 
-	if (ports) {
-		for (i = 0; ports[i]; ++i) {
-			outs.push_back (ports[i]);
-		}
-		free (ports);
+	for (i = 0; ports[i]; ++i) {
+		outs.push_back (ports[i]);
 	}
+
+	free (ports);
 }
 
 string
 AudioEngine::get_nth_physical_audio (uint32_t n, int flag)
 {
+        GET_PRIVATE_JACK_POINTER_RET (_jack,"");
 	const char ** ports;
 	uint32_t i;
 	string ret;
 
-	if (!_jack) {
-		fatal << _("get_nth_physical called before engine was connected") << endmsg;
-		/*NOTREACHED*/
-	}
-
-	ports = jack_get_ports (_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|flag);
-	
-	if (ports == 0) {
-		return "";
+	if ((ports = jack_get_ports (_priv_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortIsPhysical|flag)) == 0) {
+		return ret;
 	}
 
 	for (i = 0; i < n && ports[i]; ++i);
@@ -953,7 +992,7 @@ AudioEngine::get_nth_physical_audio (uint32_t n, int flag)
 		ret = ports[i];
 	}
 
-	free ((char *) ports);
+	free (ports);
 
 	return ret;
 }
@@ -961,106 +1000,95 @@ AudioEngine::get_nth_physical_audio (uint32_t n, int flag)
 nframes_t
 AudioEngine::get_port_total_latency (const Port& port)
 {
-	if (!_jack) {
-		return 0;
-	}
+        GET_PRIVATE_JACK_POINTER_RET (_jack,0);
 
-	return jack_port_get_total_latency (_jack, port._port);
+	return jack_port_get_total_latency (_priv_jack, port._port);
 }
 
 void
 AudioEngine::transport_stop ()
 {
-	if (_jack) {
-		jack_transport_stop (_jack);
-	}
+        GET_PRIVATE_JACK_POINTER (_jack);
+	// cerr << "tell jack to stop\n";
+	jack_transport_stop (_priv_jack);
 }
 
 void
 AudioEngine::transport_start ()
 {
-	if (_jack) {
-		jack_transport_start (_jack);
-	}
+        GET_PRIVATE_JACK_POINTER (_jack);
+	// cerr << "tell jack to roll\n";
+	jack_transport_start (_priv_jack);
 }
 
 void
 AudioEngine::transport_locate (nframes_t where)
 {
+        GET_PRIVATE_JACK_POINTER (_jack);
 	// cerr << "tell JACK to locate to " << where << endl;
-	if (_jack) {
-		jack_transport_locate (_jack, where);
-	}
+	jack_transport_locate (_priv_jack, where);
 }
 
 AudioEngine::TransportState
 AudioEngine::transport_state ()
 {
-	if (_jack) {
-		jack_position_t pos;
-		return (TransportState) jack_transport_query (_jack, &pos);
-	} else {
-		return (TransportState) JackTransportStopped;
-	}
+        GET_PRIVATE_JACK_POINTER_RET (_jack, ((TransportState) JackTransportStopped));
+	jack_position_t pos;
+	return (TransportState) jack_transport_query (_priv_jack, &pos);
 }
 
 int
 AudioEngine::reset_timebase ()
 {
-	if (_jack) {
-		if (Config->get_jack_time_master()) {
-			return jack_set_timebase_callback (_jack, 0, _jack_timebase_callback, this);
-		} else {
-			return jack_release_timebase (_jack);
-		}
+        GET_PRIVATE_JACK_POINTER_RET (_jack, -1);
+	if (Config->get_jack_time_master()) {
+	  return jack_set_timebase_callback (_priv_jack, 0, _jack_timebase_callback, this);
 	} else {
-		return -1;
+	  return jack_release_timebase (_jack);
 	}
 }
 
 int
 AudioEngine::freewheel (bool onoff)
 {
-	if (_jack) {
-
-		if (onoff != _freewheeling) {
-
-			if (onoff) {
-				_freewheel_thread_registered = false;
-			}
-
-			return jack_set_freewheel (_jack, onoff);
-
-		} else {
-			/* already doing what has been asked for */
-			return 0;
-		}
+        GET_PRIVATE_JACK_POINTER_RET (_jack, -1);
 
+	if (onoff != _freewheeling) {
+	  
+	  if (onoff) {
+	    _freewheel_thread_registered = false;
+	  }
+	  
+	  return jack_set_freewheel (_priv_jack, onoff);
+	  
 	} else {
-		return -1;
+	  /* already doing what has been asked for */
+	  return 0;
 	}
 }
 
 void
 AudioEngine::remove_all_ports ()
 {
+       jack_client_t* _priv_jack = _jack;
+
 	/* process lock MUST be held */
 
-	if (_jack) {
-		boost::shared_ptr<Ports> p = ports.reader();
+       if (_priv_jack) {
 
-		for (Ports::iterator i = p->begin(); i != p->end(); ++i) {
-			jack_port_unregister (_jack, (*i)->_port);
-		}
-	}
-
-	{
-		RCUWriter<Ports> writer (ports);
-		boost::shared_ptr<Ports> ps = writer.get_copy ();
-		ps->clear ();
-	}
-
-	port_connections.clear ();
+	 boost::shared_ptr<Ports> p = ports.reader();
+	 
+	 for (Ports::iterator i = p->begin(); i != p->end(); ++i) {
+	   jack_port_unregister (_priv_jack, (*i)->_port);
+	 }
+       }
+       {
+	 RCUWriter<Ports> writer (ports);
+	 boost::shared_ptr<Ports> ps = writer.get_copy ();
+	 ps->clear ();
+       }
+       
+       port_connections.clear ();
 }
 
 void
@@ -1097,8 +1125,10 @@ AudioEngine::connect_to_jack (string client_name)
 		return -1;
 	}
 
+        GET_PRIVATE_JACK_POINTER_RET (_jack, -1);
+
 	if (status & JackNameNotUnique) {
-		jack_client_name = jack_get_client_name (_jack);
+		jack_client_name = jack_get_client_name (_priv_jack);
 	}
 
 	jack_set_error_function (ardour_jack_error);
@@ -1125,10 +1155,7 @@ AudioEngine::connect_to_jack (string client_name)
 int 
 AudioEngine::disconnect_from_jack ()
 {
-	if (!_jack) {
-		return 0;
-	}
-
+        GET_PRIVATE_JACK_POINTER_RET (_jack, 0);
 
 	if (_running) {
 		stop_metering_thread ();
@@ -1136,7 +1163,7 @@ AudioEngine::disconnect_from_jack ()
 
 	{ 
 		Glib::Mutex::Lock lm (_process_lock);
-		jack_client_close (_jack);
+		jack_client_close (_priv_jack);
 		_jack = 0;
 	}
 
@@ -1165,8 +1192,13 @@ AudioEngine::reconnect_to_jack ()
 		return -1;
 	}
 
-	Ports::iterator i;
+	/* Now that we're connected again, we can get a private
+	   pointer to the jack client.
+	*/
 
+        GET_PRIVATE_JACK_POINTER_RET (_jack,-1);
+
+	Ports::iterator i;
 	boost::shared_ptr<Ports> p = ports.reader ();
 
 	for (i = p->begin(); i != p->end(); ++i) {
@@ -1178,7 +1210,7 @@ AudioEngine::reconnect_to_jack ()
 		
 		short_name = long_name.substr (long_name.find_last_of (':') + 1);
 
-		if (((*i)->_port = jack_port_register (_jack, short_name.c_str(), (*i)->type(), (*i)->flags(), 0)) == 0) {
+		if (((*i)->_port = jack_port_register (_priv_jack, short_name.c_str(), (*i)->type(), (*i)->flags(), 0)) == 0) {
 			error << string_compose (_("could not reregister %1"), (*i)->name()) << endmsg;
 			break;
 		} else {
@@ -1187,44 +1219,44 @@ AudioEngine::reconnect_to_jack ()
 		(*i)->reset ();
 
 		if ((*i)->flags() & JackPortIsOutput) {
-			(*i)->silence (jack_get_buffer_size (_jack));
+			(*i)->silence (jack_get_buffer_size (_priv_jack));
 		}
 	}
 
 	if (i != p->end()) {
 		/* failed */
 		for (Ports::iterator i = p->begin(); i != p->end(); ++i) {
-			jack_port_unregister (_jack, (*i)->_port);
+			jack_port_unregister (_priv_jack, (*i)->_port);
 		}
 		return -1;
 	} 
 
 
 	if (session) {
-		session->reset_jack_connection (_jack);
-		nframes_t blocksize = jack_get_buffer_size (_jack);
+		session->reset_jack_connection (_priv_jack);
+		nframes_t blocksize = jack_get_buffer_size (_priv_jack);
 		Port::set_buffer_size (blocksize);
 		session->set_block_size (blocksize);
-		session->set_frame_rate (jack_get_sample_rate (_jack));
+		session->set_frame_rate (jack_get_sample_rate (_priv_jack));
 	}
 
 	last_monitor_check = 0;
 	
-	jack_on_shutdown (_jack, halted, this);
-	jack_set_graph_order_callback (_jack, _graph_order_callback, this);
-	jack_set_thread_init_callback (_jack, _thread_init_callback, this);
-	jack_set_process_callback (_jack, _process_callback, this);
-	jack_set_sample_rate_callback (_jack, _sample_rate_callback, this);
-	jack_set_buffer_size_callback (_jack, _bufsize_callback, this);
-	jack_set_xrun_callback (_jack, _xrun_callback, this);
-	jack_set_sync_callback (_jack, _jack_sync_callback, this);
-	jack_set_freewheel_callback (_jack, _freewheel_callback, this);
+	jack_on_shutdown (_priv_jack, halted, this);
+	jack_set_graph_order_callback (_priv_jack, _graph_order_callback, this);
+	jack_set_thread_init_callback (_priv_jack, _thread_init_callback, this);
+	jack_set_process_callback (_priv_jack, _process_callback, this);
+	jack_set_sample_rate_callback (_priv_jack, _sample_rate_callback, this);
+	jack_set_buffer_size_callback (_priv_jack, _bufsize_callback, this);
+	jack_set_xrun_callback (_priv_jack, _xrun_callback, this);
+	jack_set_sync_callback (_priv_jack, _jack_sync_callback, this);
+	jack_set_freewheel_callback (_priv_jack, _freewheel_callback, this);
 	
 	if (Config->get_jack_time_master()) {
-		jack_set_timebase_callback (_jack, 0, _jack_timebase_callback, this);
+		jack_set_timebase_callback (_priv_jack, 0, _jack_timebase_callback, this);
 	} 
 	
-	if (jack_activate (_jack) == 0) {
+	if (jack_activate (_priv_jack) == 0) {
 		_running = true;
 		_has_run = true;
 	} else {
@@ -1236,16 +1268,13 @@ AudioEngine::reconnect_to_jack ()
 	for (PortConnections::iterator i = port_connections.begin(); i != port_connections.end(); ++i) {
 		
 		int err;
-		jack_client_t* j = _jack;
 
-		/* JACK could have zombified us. */
-
-		if (!j) {
+		if (!_jack) {
 			error << _("Disconnected from JACK while reconnecting. You should quit Ardour now.") << endmsg;
 			return -1;
 		}
 		
-		if ((err = jack_connect (j, (*i).first.c_str(), (*i).second.c_str())) != 0) {
+		if ((err = jack_connect (_priv_jack, (*i).first.c_str(), (*i).second.c_str())) != 0) {
 			if (err != EEXIST) {
 				error << string_compose (_("could not reconnect %1 and %2 (err = %3)"),
 						  (*i).first, (*i).second, err)
@@ -1264,26 +1293,21 @@ AudioEngine::reconnect_to_jack ()
 int
 AudioEngine::request_buffer_size (nframes_t nframes)
 {
-	if (_jack) {
+        GET_PRIVATE_JACK_POINTER_RET (_jack, -1);
 
-		if (nframes == jack_get_buffer_size (_jack)) {
-			return 0;
-		}
-
-		return jack_set_buffer_size (_jack, nframes);
-
-	} else {
-		return -1;
+	if (nframes == jack_get_buffer_size (_priv_jack)) {
+	  return 0;
 	}
+	
+	return jack_set_buffer_size (_priv_jack, nframes);
 }
 
 void
 AudioEngine::update_total_latencies ()
 {
 #ifdef HAVE_JACK_RECOMPUTE_LATENCIES
-	if (_jack) {
-		jack_recompute_total_latencies (_jack);
-	}
+        GET_PRIVATE_JACK_POINTER (_jack);
+	jack_recompute_total_latencies (_priv_jack);
 #endif
 }
 		
@@ -1327,9 +1351,6 @@ AudioEngine::make_port_name_non_relative (string portname)
 bool
 AudioEngine::is_realtime () const
 {
-	if (_jack) {
-		return jack_is_realtime (_jack);
-	} else {
-		return false;
-	}
+        GET_PRIVATE_JACK_POINTER_RET (_jack,false);
+	return jack_is_realtime (_priv_jack);
 }
