@@ -181,7 +181,6 @@ AudioRegion::AudioRegion (boost::shared_ptr<const AudioRegion> other, nframes_t
 	_fade_in_disabled = 0;
 	_fade_out_disabled = 0;
 
-
 	if (_flags & LeftOfSplit) {
 		if (_fade_in.back()->when >= _length) {
 			set_default_fade_in ();
@@ -847,7 +846,7 @@ AudioRegion::set_live_state (const XMLNode& node, Change& what_changed, bool sen
 			}
 
 			if ((prop = child->property ("active")) != 0) {
-				if (prop->value() == "yes") {
+				if (string_is_affirmative (prop->value())) {
 					set_fade_in_active (true);
 				} else {
 					set_fade_in_active (true);
@@ -868,7 +867,7 @@ AudioRegion::set_live_state (const XMLNode& node, Change& what_changed, bool sen
 			}
 
 			if ((prop = child->property ("active")) != 0) {
-				if (prop->value() == "yes") {
+				if (string_is_affirmative (prop->value())) {
 					set_fade_out_active (true);
 				} else {
 					set_fade_out_active (false);
@@ -1141,15 +1140,18 @@ AudioRegion::recompute_at_end ()
 	_envelope.set_max_xval (_length);
 	_envelope.thaw ();
 
+	if (_flags & LeftOfSplit) {
+		set_default_fade_out ();
+		_flags = Flag (_flags & ~Region::LeftOfSplit);
+	} else if (_fade_out.back()->when > _length) {
+		_fade_out.extend_to (_length);
+		send_change (FadeOutChanged);
+	}
+
 	if (_fade_in.back()->when > _length) {
 		_fade_in.extend_to (_length);
 		send_change (FadeInChanged);
 	}
-
-	if (_fade_out.back()->when > _length) {
-		_fade_out.extend_to (_length);
-		send_change (FadeOutChanged);
-	}
 }	
 
 void
@@ -1159,7 +1161,10 @@ AudioRegion::recompute_at_start ()
 
 	_envelope.truncate_start (_length);
 
-	if (_fade_in.back()->when > _length) {
+	if (_flags & RightOfSplit) {
+		set_default_fade_in ();
+		_flags = Flag (_flags & ~Region::RightOfSplit);
+	} else if (_fade_in.back()->when > _length) {
 		_fade_in.extend_to (_length);
 		send_change (FadeInChanged);
 	}
@@ -1636,14 +1641,21 @@ AudioRegion::get_transients (AnalysisFeatureList& results, bool force_new)
 
 	/* no existing/complete transient info */
 
+	static bool analyse_dialog_shown = false;
 	if (!Config->get_auto_analyse_audio()) {
-		pl->session().Dialog (_("\
+		if ( !analyse_dialog_shown ) {
+			pl->session().Dialog (_("\
 You have requested an operation that requires audio analysis.\n\n\
 You currently have \"auto-analyse-audio\" disabled, which means\n\
 that transient data must be generated every time it is required.\n\n\
 If you are doing work that will require transient data on a\n\
 regular basis, you should probably enable \"auto-analyse-audio\"\n\
-then quit ardour and restart."));
+then quit ardour and restart.\n\n\
+This dialog will not display again.  But you may notice a slight delay\n\
+in this and future transient-detection operations.\n\
+"));
+			analyse_dialog_shown = true;  //only show this dialog once 
+			}
 	}
 
 	TransientDetector t (pl->session().frame_rate());
