@@ -163,12 +163,6 @@ Crossfade::Crossfade (const Playlist& playlist, XMLNode& node)
 	if (set_state (node)) {
 		throw failed_constructor();
 	}
-
-	/* we should not need to do this here, since all values were set via XML. however
-	   some older sessions may have bugs and this allows us to fix them.
-	*/
-	
-	update ();
 }
 
 Crossfade::Crossfade (const Crossfade &orig, boost::shared_ptr<AudioRegion> newin, boost::shared_ptr<AudioRegion> newout)
@@ -728,7 +722,7 @@ Crossfade::set_state (const XMLNode& node)
 	}
 
 	if ((prop = node.property ("active")) != 0) {
-		bool x = (prop->value() == "yes");
+		bool x = string_is_affirmative (prop->value());
 		if (x != _active) {
 			_active = x;
 			what_changed = Change (what_changed | ActiveChanged);
@@ -738,13 +732,13 @@ Crossfade::set_state (const XMLNode& node)
 	}
 
 	if ((prop = node.property ("follow-overlap")) != 0) {
-		_follow_overlap = (prop->value() == "yes");
+		_follow_overlap = string_is_affirmative (prop->value());
 	} else {
 		_follow_overlap = false;
 	}
 
 	if ((prop = node.property ("fixed")) != 0) {
-		_fixed = (prop->value() == "yes");
+		_fixed = string_is_affirmative (prop->value());
 	} else {
 		_fixed = false;
 	}
