@@ -330,15 +330,15 @@ ARDOUR::init (bool use_vst, bool try_optimization)
 #endif
 
 #ifdef VST_SUPPORT
-	if (Config->get_use_vst() && fst_init ()) {
-		return -1;
+	if (Config->get_use_vst() && fst_init (0)) {
+	        return -1;
 	}
 #endif
 
 #ifdef HAVE_AUDIOUNITS
 	AUPluginInfo::load_cached_info ();
 #endif
-	
+
 	/* Make VAMP look in our library ahead of anything else */
 
 	char *p = getenv ("VAMP_PATH");
@@ -378,6 +378,9 @@ ARDOUR::cleanup ()
 	delete Library;
 	lrdf_cleanup ();
 	delete &ControlProtocolManager::instance();
+#ifdef VST_SUPPORT
+	fst_exit ();
+#endif
 	return 0;
 }
 
@@ -412,7 +415,7 @@ ARDOUR::new_change ()
 string
 ARDOUR::get_ardour_revision ()
 {
-	return "$Rev: 4214 $";
+	return "$Rev: 4889 $";
 }
 
 string
