@@ -26,6 +26,7 @@
 #include <pbd/stacktrace.h>
 
 #include <ardour/insert.h>
+#include <ardour/mtdm.h>
 #include <ardour/plugin.h>
 #include <ardour/port.h>
 #include <ardour/route.h>
@@ -165,7 +166,10 @@ PluginInsert::auto_state_changed (uint32_t which)
 {
 	AutomationList& alist (automation_list (which));
 
-	if (alist.automation_state() != Off) {
+	/* don't reset automation if we're moving to Off or Write mode;
+	   if we're moving to Write, the user may have manually set up automation
+	   that they don't want to lose */		
+	if (alist.automation_state() != Off && alist.automation_state() != Write) {
 		_plugins[0]->set_parameter (which, alist.eval (_session.transport_frame()));
 	}
 }
@@ -399,9 +403,15 @@ PluginInsert::set_parameter (uint32_t port, float val)
 {
 	/* the others will be set from the event triggered by this */
 
+	float last_val = _plugins[0]->get_parameter (port);
+	Plugin::ParameterDescriptor desc;
+	_plugins[0]->get_parameter_descriptor(port, desc);
+	
 	_plugins[0]->set_parameter (port, val);
 	
 	if (automation_list (port).automation_write()) {
+		if ( desc.toggled )  //store the previous value just before this so any interpolation works right 
+			automation_list (port).add (_session.audible_frame()-1, last_val);
 		automation_list (port).add (_session.audible_frame(), val);
 	}
 
@@ -843,38 +853,7 @@ PluginInsert::latency()
 ARDOUR::PluginType
 PluginInsert::type ()
 {
-	boost::shared_ptr<LadspaPlugin> lp;
-#ifdef VST_SUPPORT
-	boost::shared_ptr<VSTPlugin> vp;
-#endif
-#ifdef HAVE_AUDIOUNITS
-	boost::shared_ptr<AUPlugin> ap;
-#endif
-#ifdef HAVE_LV2
-	boost::shared_ptr<LV2Plugin> lv2p;
-#endif
-	
-	PluginPtr other = plugin ();
-
-	if ((lp = boost::dynamic_pointer_cast<LadspaPlugin> (other)) != 0) {
-		return ARDOUR::LADSPA;
-#ifdef VST_SUPPORT
-	} else if ((vp = boost::dynamic_pointer_cast<VSTPlugin> (other)) != 0) {
-		return ARDOUR::VST;
-#endif
-#ifdef HAVE_AUDIOUNITS
-	} else if ((ap = boost::dynamic_pointer_cast<AUPlugin> (other)) != 0) {
-		return ARDOUR::AudioUnit;
-#endif
-#ifdef HAVE_LV2
-	} else if ((lv2p = boost::dynamic_pointer_cast<LV2Plugin> (other)) != 0) {
-		return ARDOUR::LV2;
-#endif
-	} else {
-		error << "Unknown plugin type" << endmsg;
-		/* NOT REACHED */
-		return (ARDOUR::PluginType) 0;
-	}
+	return plugin()->get_info()->type;
 }
 
 /***************************************************************
@@ -899,12 +878,19 @@ PortInsert::PortInsert (const PortInsert& other)
 void
 PortInsert::init ()
 {
+	_mtdm = 0;
+	_latency_detect = false;
+	_latency_flush_frames = false;
+	_measured_latency = 0;
 }
 
 PortInsert::PortInsert (Session& s, const XMLNode& node)
 	: Insert (s, "will change", PreFader)
 {
+	init ();
+
 	bitslot = 0xffffffff;
+
 	if (set_state (node)) {
 		throw failed_constructor();
 	}
@@ -914,9 +900,36 @@ PortInsert::PortInsert (Session& s, const XMLNode& node)
 
 PortInsert::~PortInsert ()
 {
+	delete _mtdm;
 	GoingAway ();
 }
 
+void
+PortInsert::start_latency_detection ()
+{
+	if (_mtdm != 0) {
+		delete _mtdm;
+	}
+
+	_mtdm = new MTDM;
+	_latency_flush_frames = false;
+	_latency_detect = true;
+	_measured_latency = 0;
+}
+
+void
+PortInsert::stop_latency_detection ()
+{
+	_latency_flush_frames = latency() + _session.engine().frames_per_cycle();
+	_latency_detect = false;
+}
+
+void
+PortInsert::set_measured_latency (nframes_t n)
+{
+	_measured_latency = n;
+}
+
 void
 PortInsert::run (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes)
 {
@@ -924,23 +937,57 @@ PortInsert::run (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes)
 		return;
 	}
 
+	vector<Port*>::iterator o;
+
+	if (_latency_detect) {
+
+		if (n_inputs() != 0) {
+			Sample* in = get_input_buffer (0, nframes);
+			Sample* out = get_output_buffer (0, nframes);
+
+			_mtdm->process (nframes, in, out);
+			
+			for (o = _outputs.begin(); o != _outputs.end(); ++o) {
+				(*o)->mark_silence (false);
+			}
+		}
+
+		return;
+
+	} else if (_latency_flush_frames) {
+
+		/* wait for the entire input buffer to drain before picking up input again so that we can't
+		   hear the remnants of whatever MTDM pumped into the pipeline.
+		*/
+
+		silence (nframes);
+
+		if (_latency_flush_frames > nframes) {
+			_latency_flush_frames -= nframes;
+		} else {
+			_latency_flush_frames = 0;
+		}
+
+		return;
+	}
+
 	if (!active()) {
 		/* deliver silence */
 		silence (nframes);
 		return;
 	}
 
-	uint32_t n;
-	vector<Port*>::iterator o;
-	vector<Port*>::iterator i;
-
 	/* deliver output */
 
+	uint32_t n;
+
 	for (o = _outputs.begin(), n = 0; o != _outputs.end(); ++o, ++n) {
 		memcpy (get_output_buffer (n, nframes), bufs[min(nbufs,n)], sizeof (Sample) * nframes);
 		(*o)->mark_silence (false);
 	}
 	
+	vector<Port*>::iterator i;
+
 	/* collect input */
 	
 	for (i = _inputs.begin(), n = 0; i != _inputs.end(); ++i, ++n) {
@@ -1021,7 +1068,11 @@ PortInsert::latency()
 	   need to take that into account too.
 	*/
 
-	return _session.engine().frames_per_cycle() + input_latency();
+	if (_measured_latency == 0) {
+		return _session.engine().frames_per_cycle() + input_latency();
+	} else {
+		return _measured_latency;
+	}
 }
 
 int32_t
