@@ -97,32 +97,34 @@ static double direct_gain_to_control (gain_t gain) {
 
 static bool sort_ports_by_name (Port* a, Port* b) {
 
-	unsigned int	last_digit_position_a = a->name().size();
-	std::string::reverse_iterator	r_iterator = a->name().rbegin();
+	std::string const a_name = a->name();
+	unsigned int	last_digit_position_a = a_name.size();
+	std::string::const_reverse_iterator	r_iterator = a_name.rbegin();
 
-	while (r_iterator!=a->name().rend() and Glib::Unicode::isdigit(*r_iterator)) {
+	while (r_iterator != a_name.rend() and Glib::Unicode::isdigit(*r_iterator)) {
 		r_iterator++; last_digit_position_a--;
 	}
 
-	unsigned int	last_digit_position_b = b->name().size();
-	r_iterator = b->name().rbegin();
+	std::string const b_name = b->name();
+	unsigned int	last_digit_position_b = b_name.size();
+	r_iterator = b_name.rbegin();
 
-	while (r_iterator!=b->name().rend() and Glib::Unicode::isdigit(*r_iterator)) {
+	while (r_iterator != b_name.rend() and Glib::Unicode::isdigit(*r_iterator)) {
 		r_iterator++; last_digit_position_b--;
 	}
 
 	// if some of the names don't have a number as posfix, compare as strings
-	if (last_digit_position_a == a->name().size() or last_digit_position_b == b->name().size()) {
-		return a->name() < b->name();
+	if (last_digit_position_a == a_name.size() or last_digit_position_b == b_name.size()) {
+		return a_name < b_name;
 	}
 
-	const std::string 	prefix_a = a->name().substr(0, last_digit_position_a - 1);
-	const unsigned int 	posfix_a = std::atoi(a->name().substr(last_digit_position_a, a->name().size() - last_digit_position_a).c_str());
-	const std::string 	prefix_b = b->name().substr(0, last_digit_position_b - 1);
-	const unsigned int 	posfix_b = std::atoi(b->name().substr(last_digit_position_b, b->name().size() - last_digit_position_b).c_str());
+	const std::string 	prefix_a = a_name.substr(0, last_digit_position_a - 1);
+	const unsigned int 	posfix_a = std::atoi(a_name.substr(last_digit_position_a, a_name.size() - last_digit_position_a).c_str());
+	const std::string 	prefix_b = b_name.substr(0, last_digit_position_b - 1);
+	const unsigned int 	posfix_b = std::atoi(b_name.substr(last_digit_position_b, b_name.size() - last_digit_position_b).c_str());
 
 	if (prefix_a != prefix_b) {
-		return a->name() < b->name();
+		return a_name < b_name;
 	} else {
 		return posfix_a < posfix_b;
 	}
@@ -1611,7 +1613,7 @@ IO::set_state (const XMLNode& node)
 	}
 
 	if ((prop = node.property (X_("active"))) != 0) {
-		set_active (prop->value() == "yes");
+		set_active (string_is_affirmative (prop->value()));
 	}
 
 	for (iter = node.children().begin(); iter != node.children().end(); ++iter) {
@@ -2545,7 +2547,7 @@ IO::meter ()
 		/* compute new visible value using falloff */
 
 		if (new_peak > 0.0f) {
-			new_peak = coefficient_to_dB (new_peak);
+			new_peak = fast_coefficient_to_dB (new_peak);
 		} else {
 			new_peak = -INFINITY;
 		}
@@ -2718,14 +2720,28 @@ IO::build_legal_port_name (bool in)
 {
 	const int name_size = jack_port_name_size();
 	int limit;
-	char* suffix;
+	const char* suffix;
 	int maxports;
 
+	/* note that if "in" or "out" are translated it will break a session
+	   across locale switches because a port's connection list will
+	   show (old) translated names, but the current port name will
+	   use the (new) translated name.
+	*/
+
 	if (in) {
-		suffix = _("in");
+		if (getenv ("ARDOUR_RETAIN_PORT_NAME_SUFFIX_TRANSLATION") != 0) {
+			suffix = _("in");
+		} else {
+			suffix = X_("in");
+		}
 		maxports = _input_maximum;
 	} else {
-		suffix = _("out");
+		if (getenv ("ARDOUR_RETAIN_PORT_NAME_SUFFIX_TRANSLATION") != 0) {
+			suffix = _("out");
+		} else {
+			suffix = X_("out");
+		}
 		maxports = _output_maximum;
 	}
 	
