@@ -182,34 +182,44 @@ LadspaPlugin::default_value (uint32_t port)
 			ret = prh[port].LowerBound;
 			bounds_given = true;
 			sr_scaling = true;
-			earlier_hint = true;
 		}
 		
 		/* FIXME: add support for logarithmic defaults */
 		
 		else if (LADSPA_IS_HINT_DEFAULT_LOW(prh[port].HintDescriptor)) {
-			ret = prh[port].LowerBound * 0.75f + prh[port].UpperBound * 0.25f;
+			if (LADSPA_IS_HINT_LOGARITHMIC(prh[port].HintDescriptor)) {
+				ret = exp(log(prh[port].LowerBound) * 0.75f + log(prh[port].UpperBound) * 0.25f);
+			}
+			else {
+				ret = prh[port].LowerBound * 0.75f + prh[port].UpperBound * 0.25f;
+			}
 			bounds_given = true;
 			sr_scaling = true;
-			earlier_hint = true;
 		}
 		else if (LADSPA_IS_HINT_DEFAULT_MIDDLE(prh[port].HintDescriptor)) {
-			ret = prh[port].LowerBound * 0.50f + prh[port].UpperBound * 0.50f;
+			if (LADSPA_IS_HINT_LOGARITHMIC(prh[port].HintDescriptor)) {
+				ret = exp(log(prh[port].LowerBound) * 0.5f + log(prh[port].UpperBound) * 0.5f);
+			}
+			else {
+				ret = prh[port].LowerBound * 0.5f + prh[port].UpperBound * 0.5f;
+			}
 			bounds_given = true;
 			sr_scaling = true;
-			earlier_hint = true;
 		}
 		else if (LADSPA_IS_HINT_DEFAULT_HIGH(prh[port].HintDescriptor)) {
-			ret = prh[port].LowerBound * 0.25f + prh[port].UpperBound * 0.75f;
+			if (LADSPA_IS_HINT_LOGARITHMIC(prh[port].HintDescriptor)) {
+				ret = exp(log(prh[port].LowerBound) * 0.25f + log(prh[port].UpperBound) * 0.75f);
+			}
+			else {
+				ret = prh[port].LowerBound * 0.25f + prh[port].UpperBound * 0.75f;
+			}
 			bounds_given = true;
 			sr_scaling = true;
-			earlier_hint = true;
 		}
 		else if (LADSPA_IS_HINT_DEFAULT_MAXIMUM(prh[port].HintDescriptor)) {
 			ret = prh[port].UpperBound;
 			bounds_given = true;
 			sr_scaling = true;
-			earlier_hint = true;
 		}
 		else if (LADSPA_IS_HINT_DEFAULT_0(prh[port].HintDescriptor)) {
 			ret = 0.0f;
@@ -523,16 +533,16 @@ LadspaPlugin::connect_and_run (vector<Sample*>& bufs, uint32_t nbufs, int32_t& i
 	while (port_index < parameter_count()) {
 		if (LADSPA_IS_PORT_AUDIO (port_descriptor(port_index))) {
 			if (LADSPA_IS_PORT_INPUT (port_descriptor(port_index))) {
-				connect_port (port_index, bufs[min((uint32_t) in_index,nbufs - 1)]);
-				//cerr << this << ' ' << name() << " @ " << offset << " inport " << in_index << " = buf " 
-				//     << min((uint32_t)in_index,nbufs) << " = " << &bufs[min((uint32_t)in_index,nbufs)][offset] << endl;
+				connect_port (port_index, bufs[min((uint32_t) in_index,nbufs - 1)] + offset);
+				// cerr << this << ' ' << name() << " @ " << offset << " inport " << in_index << " = buf " 
+				// << min((uint32_t)in_index,nbufs) << " = " << &bufs[min((uint32_t)in_index,nbufs)][offset] << endl;
 				in_index++;
 
 
 			} else if (LADSPA_IS_PORT_OUTPUT (port_descriptor (port_index))) {
-				connect_port (port_index, bufs[min((uint32_t) out_index,nbufs - 1)]);
+				connect_port (port_index, bufs[min((uint32_t) out_index,nbufs - 1)] + offset);
 				// cerr << this << ' ' << name() << " @ " << offset << " outport " << out_index << " = buf " 
-				//     << min((uint32_t)out_index,nbufs) << " = " << &bufs[min((uint32_t)out_index,nbufs)][offset] << endl;
+				//   << min((uint32_t)out_index,nbufs) << " = " << &bufs[min((uint32_t)out_index,nbufs)][offset] << endl;
 				out_index++;
 			}
 		}
@@ -647,6 +657,7 @@ LadspaPluginInfo::load (Session& session)
 		if ((module = dlopen (path.c_str(), RTLD_NOW)) == 0) {
 			error << string_compose(_("LADSPA: cannot load module from \"%1\""), path) << endmsg;
 			error << dlerror() << endmsg;
+			return PluginPtr ((Plugin*) 0);
 		} else {
 			plugin.reset (new LadspaPlugin (module, session.engine(), session, index, session.frame_rate()));
 		}
@@ -659,3 +670,9 @@ LadspaPluginInfo::load (Session& session)
 		return PluginPtr ((Plugin*) 0);
 	}	
 }
+
+
+LadspaPluginInfo::LadspaPluginInfo()
+{
+	type = ARDOUR::LADSPA;
+}
