@@ -149,6 +149,17 @@ LV2Plugin::init (LV2World& world, SLV2Plugin plugin, nframes_t rate)
 				break;
 			}
 		}
+
+		// if gtk gui is not available, try to find external gui
+		if (!_ui) {
+			for (unsigned i=0; i < slv2_uis_size(uis); ++i) {
+				SLV2UI ui = slv2_uis_get_at(uis, i);
+				if (slv2_ui_is_a(ui, _world.external_gui)) {
+					_ui = ui;
+					break;
+				}
+			}
+		}
 	}
 
 	Plugin::setup_controls ();
@@ -176,6 +187,12 @@ LV2Plugin::~LV2Plugin ()
 	}
 }
 
+bool
+LV2Plugin::is_external_ui() const
+{
+	return slv2_ui_is_a(_ui, _world.external_gui);
+}
+
 string
 LV2Plugin::unique_id() const
 {
@@ -346,7 +363,7 @@ LV2Plugin::get_parameter_descriptor (uint32_t which, ParameterDescriptor& desc)
 	
     desc.integer_step = slv2_port_has_property(_plugin, port, _world.integer);
     desc.toggled = slv2_port_has_property(_plugin, port, _world.toggled);
-    desc.logarithmic = false; // TODO (LV2 extension)
+    desc.logarithmic = slv2_port_has_property(_plugin, port, _world.logarithmic);
     desc.sr_dependent = slv2_port_has_property(_plugin, port, _world.srate);
     desc.label = slv2_value_as_string(slv2_port_get_name(_plugin, port));
     desc.lower = min ? slv2_value_as_float(min) : 0.0f;
@@ -552,6 +569,8 @@ LV2World::LV2World()
 	toggled = slv2_value_new_uri(world, SLV2_NAMESPACE_LV2 "toggled");
 	srate = slv2_value_new_uri(world, SLV2_NAMESPACE_LV2 "sampleRate");
 	gtk_gui = slv2_value_new_uri(world, "http://lv2plug.in/ns/extensions/ui#GtkUI");
+	external_gui = slv2_value_new_uri(world, "http://lv2plug.in/ns/extensions/ui#external");
+ 	logarithmic = slv2_value_new_uri(world, "http://lv2plug.in/ns/dev/extportinfo#logarithmic");
 }
 
 LV2World::~LV2World()
@@ -567,6 +586,7 @@ LV2PluginInfo::LV2PluginInfo (void* lv2_world, void* slv2_plugin)
 	: _lv2_world(lv2_world)
 	, _slv2_plugin(slv2_plugin)
 {
+	type = ARDOUR::LV2;
 }
 
 LV2PluginInfo::~LV2PluginInfo()
@@ -606,6 +626,12 @@ LV2PluginInfo::discover (void* lv2_world)
 		LV2PluginInfoPtr info (new LV2PluginInfo(lv2_world, p));
 
 		SLV2Value name = slv2_plugin_get_name(p);
+
+		if (!name) {
+			cerr << "LV2: invalid plugin\n";
+			continue;
+		}
+		
 		info->name = string(slv2_value_as_string(name));
 		slv2_value_free(name);
 
