@@ -171,7 +171,7 @@ StreamPanner::set_state (const XMLNode& node)
 	XMLNodeConstIterator iter;
 
 	if ((prop = node.property (X_("muted")))) {
-		set_muted (prop->value() == "yes");
+		set_muted (string_is_affirmative (prop->value()));
 	}
 
 	return 0;
@@ -224,8 +224,11 @@ BaseStereoPanner::set_automation_state (AutoState state)
 	if (state != _automation.automation_state()) {
 
 		_automation.set_automation_state (state);
-		
-		if (state != Off) {
+
+		/* don't reset pan if we're moving to Off or Write mode;
+		   if we're moving to Write, the user may have manually set up pans
+		   that they don't want to lose */		
+		if (state != Off && state != Write) {
 			set_position (_automation.eval (parent.session().transport_frame()));
 		}
 	}
@@ -1162,12 +1165,12 @@ Panner::set_state (const XMLNode& node)
 	outputs.clear ();
 
 	if ((prop = node.property (X_("linked"))) != 0) {
-		set_linked (prop->value() == "yes");
+		set_linked (string_is_affirmative (prop->value()));
 	}
 
 
 	if ((prop = node.property (X_("bypassed"))) != 0) {
-		set_bypassed (prop->value() == "yes");
+		set_bypassed (string_is_affirmative (prop->value()));
 	}
 
 	if ((prop = node.property (X_("link_direction"))) != 0) {
