@@ -31,6 +31,7 @@
 #include <pbd/stl_delete.h>
 #include <pbd/xml++.h>
 #include <pbd/stacktrace.h>
+#include <pbd/memento_command.h>
 
 #include <ardour/playlist.h>
 #include <ardour/session.h>
@@ -791,11 +792,11 @@ Playlist::partition_internal (nframes_t start, nframes_t end, bool cutting, Regi
 				new_regions.push_back (region);
 				
 				/* "front" ***** */
-				
+			
 				current->freeze ();
 				thawlist.push_back (current);
-				current->trim_end (pos2, this);
-				
+				current->cut_end (pos2 - 1, this);
+
 			} else if (overlap == OverlapEnd) {
 				
 				/*
@@ -823,7 +824,7 @@ Playlist::partition_internal (nframes_t start, nframes_t end, bool cutting, Regi
 				
 				current->freeze ();
 				thawlist.push_back (current);
-				current->trim_end (pos2, this);
+				current->cut_end (pos2 - 1, this);
 				
 			} else if (overlap == OverlapStart) {
 				
@@ -892,6 +893,19 @@ Playlist::partition_internal (nframes_t start, nframes_t end, bool cutting, Regi
 	}
 }
 
+bool
+Playlist::has_region_at (nframes64_t const p) const
+{
+	RegionLock (const_cast<Playlist *> (this));
+	
+	RegionList::const_iterator i = regions.begin ();
+	while (i != regions.end() && !(*i)->covers (p)) {
+		++i;
+	}
+
+	return (i != regions.end());
+}
+
 boost::shared_ptr<Playlist>
 Playlist::cut_copy (boost::shared_ptr<Playlist> (Playlist::*pmf)(nframes_t, nframes_t,bool), list<AudioRange>& ranges, bool result_is_hidden)
 {
@@ -1371,6 +1385,21 @@ Playlist::regions_at (nframes_t frame)
 	return find_regions_at (frame);
 }	
 
+uint32_t
+Playlist::count_regions_at (nframes_t frame)
+{
+	RegionLock rlock (this);
+	uint32_t cnt = 0;
+
+	for (RegionList::iterator i = regions.begin(); i != regions.end(); ++i) {
+		if ((*i)->covers (frame)) {
+			cnt++;
+		}
+	}
+
+	return cnt;
+}	
+
 boost::shared_ptr<Region>
 Playlist::top_region_at (nframes_t frame)
 
@@ -1768,7 +1797,7 @@ Playlist::set_state (const XMLNode& node)
 		} else if (prop->name() == X_("orig_diskstream_id")) {
 			_orig_diskstream_id = prop->value ();
 		} else if (prop->name() == X_("frozen")) {
-			_frozen = (prop->value() == X_("yes"));
+			_frozen = string_is_affirmative (prop->value());
 		}
 	}
 
@@ -1989,10 +2018,10 @@ Playlist::relayer ()
 void
 Playlist::raise_region (boost::shared_ptr<Region> region)
 {
-	uint32_t rsz = regions.size();
+	layer_t top = regions.size() - 1;
 	layer_t target = region->layer() + 1U;
 
-	if (target >= rsz) {
+	if (target >= top) {
 		/* its already at the effective top */
 		return;
 	}
@@ -2017,22 +2046,48 @@ void
 Playlist::raise_region_to_top (boost::shared_ptr<Region> region)
 {
 	/* does nothing useful if layering mode is later=higher */
-	if ((Config->get_layer_model() == MoveAddHigher) ||
-	    (Config->get_layer_model() == AddHigher)) {
-		timestamp_layer_op (region);
-		relayer ();
+	switch (Config->get_layer_model()) {
+	case LaterHigher:
+		return;
+	default:
+		break;
 	}
+
+	layer_t top = regions.size() - 1;
+
+	if (region->layer() >= top) {
+		/* already on the top */
+		return;
+	}
+
+	move_region_to_layer (top, region, 1);
+	/* mark the region's last_layer_op as now, so that it remains on top when
+	   doing future relayers (until something else takes over)
+	 */
+	timestamp_layer_op (region);
 }
 
 void
 Playlist::lower_region_to_bottom (boost::shared_ptr<Region> region)
 {
 	/* does nothing useful if layering mode is later=higher */
-	if ((Config->get_layer_model() == MoveAddHigher) ||
-	    (Config->get_layer_model() == AddHigher)) {
-		region->set_last_layer_op (0);
-		relayer ();
+	switch (Config->get_layer_model()) {
+	case LaterHigher:
+		return;
+	default:
+		break;
 	}
+
+	if (region->layer() == 0) {
+		/* already on the bottom */
+		return;
+	}
+
+	move_region_to_layer (0, region, -1);
+	/* force region's last layer op to zero so that it stays at the bottom
+	   when doing future relayers
+	*/
+	region->set_last_layer_op (0);
 }
 
 int
@@ -2043,6 +2098,10 @@ Playlist::move_region_to_layer (layer_t target_layer, boost::shared_ptr<Region>
 	list<LayerInfo> layerinfo;
 	layer_t dest;
 
+	_session.begin_reversible_command (_("change region layer"));
+
+	XMLNode& before (get_state());
+
 	{
 		RegionLock rlock (const_cast<Playlist *> (this));
 		
@@ -2104,7 +2163,13 @@ Playlist::move_region_to_layer (layer_t target_layer, boost::shared_ptr<Region>
 	
 	check_dependents (region, false);
 #endif
-	
+
+	XMLNode& after (get_state());
+
+	_session.add_command (new MementoCommand<Playlist>(*this, &before, &after));
+
+	_session.commit_reversible_command ();
+
 	return 0;
 }
 
@@ -2213,8 +2278,6 @@ Playlist::set_frozen (bool yn)
 void
 Playlist::timestamp_layer_op (boost::shared_ptr<Region> region)
 {
-//	struct timeval tv;
-//	gettimeofday (&tv, 0);
 	region->set_last_layer_op (++layer_op_counter);
 }
 
