@@ -203,16 +203,14 @@ string
 Route::ensure_track_or_route_name(string name, Session &session)
 {
 	string newname = name;
-
-	while (session.route_by_name (newname)!=NULL)
-	{
+	
+	while (!session.io_name_is_legal (newname)) {
 		newname = bump_name_once (newname);
 	}
 
 	return newname;
 }
 
-
 void
 Route::inc_gain (gain_t fraction, void *src)
 {
@@ -1696,15 +1694,15 @@ Route::_set_state (const XMLNode& node, bool call_base)
 	}
 
 	if ((prop = node.property (X_("phase-invert"))) != 0) {
-		set_phase_invert (prop->value()=="yes"?true:false, this);
+		set_phase_invert (string_is_affirmative (prop->value()), this);
 	}
 
 	if ((prop = node.property (X_("denormal-protection"))) != 0) {
-		set_denormal_protection (prop->value()=="yes"?true:false, this);
+		set_denormal_protection (string_is_affirmative (prop->value()), this);
 	}
 
 	if ((prop = node.property (X_("muted"))) != 0) {
-		bool yn = prop->value()=="yes"?true:false; 
+		bool yn = string_is_affirmative (prop->value());
 
 		/* force reset of mute status */
 
@@ -1714,7 +1712,7 @@ Route::_set_state (const XMLNode& node, bool call_base)
 	}
 
 	if ((prop = node.property (X_("soloed"))) != 0) {
-		bool yn = prop->value()=="yes"?true:false; 
+		bool yn = string_is_affirmative (prop->value());
 
 		/* force reset of solo status */
 
@@ -1724,19 +1722,19 @@ Route::_set_state (const XMLNode& node, bool call_base)
 	}
 
 	if ((prop = node.property (X_("mute-affects-pre-fader"))) != 0) {
-		_mute_affects_pre_fader = (prop->value()=="yes")?true:false;
+		_mute_affects_pre_fader = string_is_affirmative (prop->value());
 	}
 
 	if ((prop = node.property (X_("mute-affects-post-fader"))) != 0) {
-		_mute_affects_post_fader = (prop->value()=="yes")?true:false;
+		_mute_affects_post_fader = string_is_affirmative (prop->value());
 	}
 
 	if ((prop = node.property (X_("mute-affects-control-outs"))) != 0) {
-		_mute_affects_control_outs = (prop->value()=="yes")?true:false;
+		_mute_affects_control_outs = string_is_affirmative (prop->value());
 	}
 
 	if ((prop = node.property (X_("mute-affects-main-outs"))) != 0) {
-		_mute_affects_main_outs = (prop->value()=="yes")?true:false;
+		_mute_affects_main_outs = string_is_affirmative (prop->value());
 	}
 
 	if ((prop = node.property (X_("meter-point"))) != 0) {
@@ -2268,7 +2266,7 @@ Route::handle_transport_stopped (bool abort_ignored, bool did_locate, bool can_f
 			if (Config->get_plugins_stop_with_transport() && can_flush_redirects) {
 				(*i)->deactivate ();
 				(*i)->activate ();
-			}
+			} 
 			
 			(*i)->transport_stopped (now);
 		}
@@ -2317,11 +2315,26 @@ Route::no_roll (nframes_t nframes, nframes_t start_frame, nframes_t end_frame,
 		return 0;
 	}
 
-	if (session_state_changing || !_active)  {
+	if (!_active)  {
 		silence (nframes);
 		return 0;
 	}
 
+	if (session_state_changing) {
+		if (_session.transport_speed() != 0.0f) {
+			/* we're rolling but some state is changing (e.g. our diskstream contents)
+			   so we cannot use them. Be silent till this is over.
+			   
+			   XXX note the absurdity of ::no_roll() being called when we ARE rolling!
+			*/
+			silence (nframes);
+			return 0;
+		}
+		/* we're really not rolling, so we're either delivery silence or actually
+		   monitoring, both of which are safe to do while session_state_changing is true.
+		*/
+	}
+
 	apply_gain_automation = false;
 	
 	if (n_inputs()) {
@@ -2712,3 +2725,21 @@ Route::set_name (string str, void* src)
 	}
 	return ret;
 }
+
+bool
+Route::has_io_redirect_named (const string& name)
+{
+	Glib::RWLock::ReaderLock lm (redirect_lock);
+	RedirectList::iterator i;
+
+	for (i = _redirects.begin(); i != _redirects.end(); ++i) {
+		if (boost::dynamic_pointer_cast<Send> (*i) ||
+		    boost::dynamic_pointer_cast<PortInsert> (*i)) {
+			if ((*i)->name() == name) {
+				return true;
+			}
+		}
+	}
+
+	return false;
+}
