@@ -155,7 +155,7 @@ Session::wait_till_butler_finished ()
 void *
 Session::_butler_thread_work (void* arg)
 {
-	PBD::ThreadCreated (pthread_self(), X_("Butler"));
+	PBD::notify_gui_about_thread_creation (pthread_self(), X_("Butler"));
 	return ((Session *) arg)->butler_thread_work ();
 	return 0;
 }
