@@ -23,6 +23,7 @@
 #include <ardour/timestamps.h>
 
 #include <pbd/error.h>
+#include <pbd/enumwriter.h>
 #include <glibmm/thread.h>
 
 #include <ardour/ardour.h>
@@ -36,28 +37,6 @@ using namespace PBD;
 
 MultiAllocSingleReleasePool Session::Event::pool ("event", sizeof (Session::Event), 512);
 
-static const char* event_names[] = {
-	"SetTransportSpeed",
-	"SetDiskstreamSpeed",
-	"Locate",
-	"LocateRoll",
-	"LocateRollLocate",
-	"SetLoop",
-	"PunchIn",
-	"PunchOut",
-	"RangeStop",
-	"RangeLocate",
-	"Overwrite",
-	"SetSlaveSource",
-	"Audition",
-	"InputConfigurationChange",
-	"SetAudioRange",
-	"SetMusicRange",
-	"SetPlayRange",
-	"StopOnce",
-	"AutoLoop"
-};
-
 void
 Session::add_event (nframes_t frame, Event::Type type, nframes_t target_frame)
 {
@@ -158,8 +137,8 @@ Session::merge_event (Event* ev)
 	default:
 		for (Events::iterator i = events.begin(); i != events.end(); ++i) {
 			if ((*i)->type == ev->type && (*i)->action_frame == ev->action_frame) {
-			  error << string_compose(_("Session: cannot have two events of type %1 at the same frame (%2)."), 
-						 event_names[ev->type], ev->action_frame) << endmsg;
+				error << string_compose(_("Session: cannot have two events of type %1 at the same frame (%2)."), 
+							enum_2_string (ev->type), ev->action_frame) << endmsg;
 				return;
 			}
 		}
@@ -336,6 +315,7 @@ Session::process_event (Event* ev)
 			// cerr << "soft locate to " << ev->target_frame << endl;
 			start_locate (ev->target_frame, false, true, false);
 		}
+		_send_smpte_update = true;
 		break;
 
 	case Event::LocateRoll:
@@ -346,18 +326,18 @@ Session::process_event (Event* ev)
 			// cerr << "soft locate to+roll " << ev->target_frame << endl;
 			start_locate (ev->target_frame, true, true, false);
 		}
+		_send_smpte_update = true;
 		break;
 
 	case Event::LocateRollLocate:
 		// locate is handled by ::request_roll_at_and_return()
 		_requested_return_frame = ev->target_frame;
-		cerr << "Set RRF " << ev->target_frame << endl;
 		request_locate (ev->target2_frame, true);
 		break;
 
 
 	case Event::SetTransportSpeed:
-		set_transport_speed (ev->speed, ev->yes_or_no);
+		set_transport_speed (ev->speed, ev->yes_or_no, ev->second_yes_or_no);
 		break;
 		
 	case Event::PunchIn:
@@ -380,7 +360,7 @@ Session::process_event (Event* ev)
 
 	case Event::StopOnce:
 		if (!non_realtime_work_pending()) {
-			stop_transport (ev->yes_or_no);
+			set_transport_speed (0.0, ev->yes_or_no, ev->second_yes_or_no);
 			_clear_event_type (Event::StopOnce);
 		}
 		remove = false;
@@ -424,13 +404,8 @@ Session::process_event (Event* ev)
 		schedule_butler_transport_work ();
 		break;
 
-	case Event::SetAudioRange:
-		current_audio_range = ev->audio_range;
-		setup_auto_play ();
-		break;
-
-	case Event::SetPlayRange:
-		set_play_range (ev->yes_or_no);
+	case Event::SetPlayAudioRange:
+		set_play_range (ev->audio_range, (ev->speed == 1.0f));
 		break;
 
 	default:
