@@ -258,7 +258,7 @@ Session::process_with_events (nframes_t nframes)
 	bool session_needs_butler = false;
 	nframes_t stop_limit;
 	long           frames_moved;
-	
+
 	/* make sure the auditioner is silent */
 
 	if (auditioner) {
@@ -282,6 +282,14 @@ Session::process_with_events (nframes_t nframes)
 		process_event (ev);
 	}
 
+	/* Events caused a transport change, send an MTC Full Frame (SMPTE) message.
+	 * This is sent whether rolling or not, to give slaves an idea of ardour time
+	 * on locates (and allow slow slaves to position and prepare for rolling)
+	 */
+	if (_send_smpte_update) {
+		send_full_time_code ();
+	}
+
 	if (!process_can_proceed()) {
 		_silent = true;
 		return;
@@ -542,7 +550,8 @@ Session::follow_slave (nframes_t nframes)
 				Location* al = _locations.auto_loop_location();
 
 				if (al && play_loop && (slave_transport_frame < al->start() || slave_transport_frame > al->end())) {
-					// cancel looping
+					// master has moved outside the loop: stop looping
+					cerr << "Stop looping - master out of range\n";
 					request_play_loop(false);
 				}
 
