@@ -62,13 +62,21 @@ void
 Session::request_slave_source (SlaveSource src)
 {
 	Event* ev = new Event (Event::SetSlaveSource, Event::Add, Event::Immediate, 0, 0.0);
+	bool seamless;
+
+	seamless = Config->get_seamless_loop ();
 
 	if (src == JACK) {
-		/* could set_seamless_loop() be disposed of entirely?*/
+		/* JACK cannot support seamless looping at present */
 		Config->set_seamless_loop (false);
 	} else {
-		Config->set_seamless_loop (true);
+		/* reset to whatever the value was before we last switched slaves */
+		Config->set_seamless_loop (_was_seamless);
 	}
+
+	/* save value of seamless from before the switch */
+	_was_seamless = seamless;
+
 	ev->slave = src;
 	queue_event (ev);
 }
@@ -89,9 +97,9 @@ Session::request_diskstream_speed (Diskstream& ds, float speed)
 }
 
 void
-Session::request_stop (bool abort)
+Session::request_stop (bool abort, bool clear_state)
 {
-	Event* ev = new Event (Event::SetTransportSpeed, Event::Add, Event::Immediate, 0, 0.0, abort);
+	Event* ev = new Event (Event::SetTransportSpeed, Event::Add, Event::Immediate, 0, 0.0, abort, clear_state);
 	queue_event (ev);
 }
 
@@ -110,7 +118,7 @@ Session::force_locate (nframes_t target_frame, bool with_roll)
 }
 
 void
-Session::request_play_loop (bool yn)
+Session::request_play_loop (bool yn, bool leave_rolling)
 {
 	Event* ev;	
 	Location *location = _locations.auto_loop_location();
@@ -121,25 +129,39 @@ Session::request_play_loop (bool yn)
 		return;
 	}
 
-	ev = new Event (Event::SetLoop, Event::Add, Event::Immediate, 0, 0.0, yn);
+	ev = new Event (Event::SetLoop, Event::Add, Event::Immediate, 0, (leave_rolling ? 1.0 : 0.0), yn);
 	queue_event (ev);
 
-	if (!yn && Config->get_seamless_loop() && transport_rolling()) {
+	if (!leave_rolling && !yn && Config->get_seamless_loop() && transport_rolling()) {
 		// request an immediate locate to refresh the diskstreams
 		// after disabling looping
 		request_locate (_transport_frame-1, false);
-	}
+	} 
 }
 
 void
-Session::realtime_stop (bool abort)
+Session::request_play_range (list<AudioRange>* range, bool leave_rolling)
 {
+	Event* ev = new Event (Event::SetPlayAudioRange, Event::Add, Event::Immediate, 0, (leave_rolling ? 1.0 : 0.0));
+	if (range) {
+		ev->audio_range = *range;
+	} else {
+		ev->audio_range.clear ();
+	}
+	queue_event (ev);
+}
+
+void
+Session::realtime_stop (bool abort, bool clear_state)
+{
+	PostTransportWork todo = PostTransportWork (0);
+	
 	/* assume that when we start, we'll be moving forwards */
 
 	if (_transport_speed < 0.0f) {
-		post_transport_work = PostTransportWork (post_transport_work | PostTransportStop | PostTransportReverse);
+		todo = PostTransportWork (todo | PostTransportStop | PostTransportReverse);
 	} else {
-		post_transport_work = PostTransportWork (post_transport_work | PostTransportStop);
+		todo = PostTransportWork (todo | PostTransportStop);
 	}
 
 	if (actively_recording()) {
@@ -155,18 +177,28 @@ Session::realtime_stop (bool abort)
 
 		/* the duration change is not guaranteed to have happened, but is likely */
 
-		post_transport_work = PostTransportWork (post_transport_work | PostTransportDuration);
+		todo = PostTransportWork (todo | PostTransportDuration);
 	}
 
 	if (abort) {
-		post_transport_work = PostTransportWork (post_transport_work | PostTransportAbort);
+		todo = PostTransportWork (todo | PostTransportAbort);
 	}
 
+	if (clear_state) {
+		todo = PostTransportWork (todo | PostTransportClearSubstate);
+	}
+	
+	if (todo) {
+		post_transport_work = PostTransportWork (post_transport_work | todo);
+	}
+						  
 	_clear_event_type (Event::StopOnce);
 	_clear_event_type (Event::RangeStop);
 	_clear_event_type (Event::RangeLocate);
 
-	disable_record (true);
+	/* if we're going to clear loop state, then force disabling record BUT only if we're not doing latched rec-enable */
+
+	disable_record (true, (!Config->get_latched_record_enable() && clear_state));
 
 	reset_slave_state ();
 		
@@ -385,55 +417,88 @@ Session::non_realtime_stop (bool abort, int on_entry, bool& finished)
 		     _requested_return_frame >= 0) &&
 		    !(post_transport_work & PostTransportLocate)) {
 
+			/* no explicit locate queued */
+
 			bool do_locate = false;
 
 			if (_requested_return_frame >= 0) {
+
+				/* explicit return request pre-queued in event list. overrides everything else */
+				
 				_transport_frame = _requested_return_frame;
-				_requested_return_frame = -1;
 				do_locate = true;
+
 			} else {
-				_transport_frame = last_stop_frame;
-				_requested_return_frame = -1;
+				if (Config->get_auto_return()) {
+
+					if (play_loop) {
+
+						/* don't try to handle loop play when synced to JACK */
+
+						if (!synced_to_jack()) {
+
+							Location *location = _locations.auto_loop_location();
+							
+							if (location != 0) {
+								_transport_frame = location->start();
+							} else {
+								_transport_frame = _last_roll_location;
+							}
+							do_locate = true;
+						}
+
+					} else if (_play_range) {
+
+						/* return to start of range */
+
+						if (!current_audio_range.empty()) {
+							_transport_frame = current_audio_range.front().start;
+							do_locate = true;
+						}
+
+					} else {
+						
+						/* regular auto-return */
+						
+						_transport_frame = _last_roll_location;
+						do_locate = true;
+					}
+				} 
 			}
 
-			if (synced_to_jack() && !play_loop) {
-				do_locate = true;
-			}
-			
+			_requested_return_frame = -1;			
+
 			if (do_locate) {
-				// cerr << "non-realtimestop: transport locate to " << _transport_frame << endl;
 				_engine.transport_locate (_transport_frame);
 			}
 		} 
 
-#ifndef LEAVE_TRANSPORT_UNADJUSTED
 	}
-#endif
 
-		for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
-			if (!(*i)->hidden()) {
-				if ((*i)->speed() != 1.0f || (*i)->speed() != -1.0f) {
-					(*i)->seek ((nframes_t) (_transport_frame * (double) (*i)->speed()));
-				}
-				else {
-					(*i)->seek (_transport_frame);
-				}
+	/* do this before seeking, because otherwise the Diskstreams will do the wrong thing in seamless loop mode.
+	*/
+
+	if (post_transport_work & PostTransportClearSubstate) {
+		_play_range = false;
+		unset_play_loop ();
+	}
+
+	/* this for() block can be put inside the previous if() and has the effect of ... ??? what */
+
+	for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
+		if (!(*i)->hidden()) {
+			if ((*i)->speed() != 1.0f || (*i)->speed() != -1.0f) {
+				(*i)->seek ((nframes_t) (_transport_frame * (double) (*i)->speed()));
 			}
-			if (on_entry != g_atomic_int_get (&butler_should_do_transport_work)) {
-				finished = false;
-				/* we will be back */
-				return;
+			else {
+				(*i)->seek (_transport_frame);
 			}
 		}
-#ifdef LEAVE_TRANSPORT_UNADJUSTED
-	}
-#endif
-
-        if (_requested_return_frame < 0) {
-		last_stop_frame = _transport_frame;
-	} else {
-		last_stop_frame = _requested_return_frame;
-		_requested_return_frame = -1;
+		if (on_entry != g_atomic_int_get (&butler_should_do_transport_work)) {
+			finished = false;
+			/* we will be back */
+			return;
+		}
 	}
 
         have_looped = false; 
@@ -442,23 +507,6 @@ Session::non_realtime_stop (bool abort, int on_entry, bool& finished)
 	deliver_mmc (MIDI::MachineControl::cmdStop, 0);
 	deliver_mmc (MIDI::MachineControl::cmdLocate, _transport_frame);
 
-	if (did_record) {
-
-		/* XXX its a little odd that we're doing this here
-		   when realtime_stop(), which has already executed,
-		   will have done this.
-		   JLC - so let's not because it seems unnecessary and breaks loop record
-		*/
-#if 0
-		if (!Config->get_latched_record_enable()) {
-			g_atomic_int_set (&_record_status, Disabled);
-		} else {
-			g_atomic_int_set (&_record_status, Enabled);
-		}
-		RecordStateChanged (); /* emit signal */
-#endif
-	}
-	
 	if ((post_transport_work & PostTransportLocate) && get_record_enabled()) {
 		/* capture start has been changed, so save pending state */
 		save_state ("", true);
@@ -479,13 +527,6 @@ Session::non_realtime_stop (bool abort, int on_entry, bool& finished)
 		DurationChanged (); /* EMIT SIGNAL */
 	}
 
-	if (post_transport_work & PostTransportStop) { 
-		_play_range = false;
-
-		/* do not turn off autoloop on stop */
-		
-	}
-
         nframes_t tf = _transport_frame;
 
         PositionChanged (tf); /* EMIT SIGNAL */
@@ -517,37 +558,55 @@ Session::check_declick_out ()
 			start_locate (pending_locate_frame, pending_locate_roll, pending_locate_flush);
 			transport_sub_state &= ~(PendingDeclickOut|PendingLocate);
 		} else {
-			stop_transport (pending_abort);
+			stop_transport (pending_abort, pending_clear_substate);
 			transport_sub_state &= ~(PendingDeclickOut|PendingLocate);
 		}
 	}
 }
 
+void
+Session::unset_play_loop ()
+{
+	play_loop = false;
+	clear_events (Event::AutoLoop);
+	
+	// set all diskstreams to NOT use internal looping
+	boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
+	for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
+		if (!(*i)->hidden()) {
+			(*i)->set_loop (0);
+		}
+	}
+}
+
 void
 Session::set_play_loop (bool yn)
 {
 	/* Called from event-handling context */
-	
-	if ((actively_recording() && yn) || _locations.auto_loop_location() == 0) {
+
+	Location *loc;
+
+	if (yn == play_loop || (actively_recording() && yn) || (loc = _locations.auto_loop_location()) == 0) {
+		/* nothing to do, or can't change loop status while recording */
 		return;
 	}
 	
 	set_dirty();
-
+	
 	if (yn && Config->get_seamless_loop() && synced_to_jack()) {
 		warning << _("Seamless looping cannot be supported while Ardour is using JACK transport.\n"
 			     "Recommend changing the configured options")
 			<< endmsg;
 		return;
 	}
-
 	
-	if ((play_loop = yn)) {
+	if (yn) {
 
-		Location *loc;
+		play_loop = true;
 
-		
-		if ((loc = _locations.auto_loop_location()) != 0) {
+		if (loc) {
+
+			unset_play_range ();
 
 			if (Config->get_seamless_loop()) {
 				// set all diskstreams to use internal looping
@@ -568,37 +627,24 @@ Session::set_play_loop (bool yn)
 				}
 			}
 			
-			/* stick in the loop event */
+			/* put the loop event into the event list */
 			
 			Event* event = new Event (Event::AutoLoop, Event::Replace, loc->end(), loc->start(), 0.0f);
 			merge_event (event);
 
-			/* locate to start of loop and roll if current pos is outside of the loop range */
-			if (_transport_frame < loc->start() || _transport_frame > loc->end()) {
-				event = new Event (Event::LocateRoll, Event::Add, Event::Immediate, loc->start(), 0, !synced_to_jack());
-				merge_event (event);
-			}
-			else {
-				// locate to current position (+ 1 to force reload)
-				event = new Event (Event::LocateRoll, Event::Add, Event::Immediate, _transport_frame + 1, 0, !synced_to_jack());
-				merge_event (event);
-			}
+			/* locate to start of loop and roll. If doing seamless loop, force a 
+			   locate+buffer refill even if we are positioned there already.
+			*/
+
+			start_locate (loc->start(), true, true, false, Config->get_seamless_loop());
 		}
 
-
-
 	} else {
-		clear_events (Event::AutoLoop);
 
-		// set all diskstreams to NOT use internal looping
-		boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
-		for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
-			if (!(*i)->hidden()) {
-				(*i)->set_loop (0);
-			}
-		}
-		
+		unset_play_loop ();
 	}
+
+	TransportStateChange ();
 }
 
 void
@@ -612,7 +658,7 @@ Session::flush_all_redirects ()
 }
 
 void
-Session::start_locate (nframes_t target_frame, bool with_roll, bool with_flush, bool with_loop)
+Session::start_locate (nframes_t target_frame, bool with_roll, bool with_flush, bool with_loop, bool force)
 {
 	if (synced_to_jack()) {
 
@@ -628,16 +674,15 @@ Session::start_locate (nframes_t target_frame, bool with_roll, bool with_flush,
 			*/
 
 			_engine.transport_locate (target_frame);
+		}
 
-			if (sp != 1.0f && with_roll) {
-				_engine.transport_start ();
-			}
-
+		if (sp != 1.0f && with_roll) {
+			_engine.transport_start ();
 		}
 
 	} else {
 
-		locate (target_frame, with_roll, with_flush, with_loop);
+		locate (target_frame, with_roll, with_flush, with_loop, force);
 	}
 }
 
@@ -661,13 +706,13 @@ Session::micro_locate (nframes_t distance)
 }
 
 void
-Session::locate (nframes_t target_frame, bool with_roll, bool with_flush, bool with_loop)
+Session::locate (nframes_t target_frame, bool with_roll, bool with_flush, bool with_loop, bool force)
 {
 	if (actively_recording() && !with_loop) {
 		return;
 	}
 
-	if (_transport_frame == target_frame && !loop_changing && !with_loop) {
+	if (!force && _transport_frame == target_frame && !loop_changing && !with_loop) {
 		if (with_roll) {
 			set_transport_speed (1.0, false);
 		}
@@ -689,11 +734,15 @@ Session::locate (nframes_t target_frame, bool with_roll, bool with_flush, bool w
 		} 
 	}
 
+	/* stop if we are rolling and we're not doing autoplay and we don't plan to roll when done and we not looping while synced to
+	   jack
+	*/
+
 	if (transport_rolling() && (!auto_play_legal || !Config->get_auto_play()) && !with_roll && !(synced_to_jack() && play_loop)) {
-		realtime_stop (false);
+		realtime_stop (false, true); // XXX paul - check if the 2nd arg is really correct
 	} 
 
-	if ( !with_loop || loop_changing) {
+	if (force || !with_loop || loop_changing) {
 
 		post_transport_work = PostTransportWork (post_transport_work | PostTransportLocate);
 		
@@ -773,10 +822,12 @@ Session::locate (nframes_t target_frame, bool with_roll, bool with_flush, bool w
 	}
 	
 	loop_changing = false;
+
+	_send_smpte_update = true;
 }
 
 void
-Session::set_transport_speed (float speed, bool abort)
+Session::set_transport_speed (float speed, bool abort, bool clear_state)
 {
 	if (_transport_speed == speed) {
 		return;
@@ -803,9 +854,16 @@ Session::set_transport_speed (float speed, bool abort)
 		}
 
 		if (synced_to_jack ()) {
+			if (clear_state) {
+				/* do this here because our response to the slave won't 
+				   take care of it.
+				*/
+				_play_range = false;
+				unset_play_loop ();
+			}
 			_engine.transport_stop ();
 		} else {
-			stop_transport (abort);
+			stop_transport (abort, clear_state);
 		}
 		
 	} else if (transport_stopped() && speed == 1.0) {
@@ -862,8 +920,10 @@ Session::set_transport_speed (float speed, bool abort)
 		   before the last stop, then we have to do extra work.
 		*/
 
+		PostTransportWork todo = PostTransportWork (0);
+
 		if ((_transport_speed && speed * _transport_speed < 0.0f) || (_last_transport_speed * speed < 0.0f) || (_last_transport_speed == 0.0f && speed < 0.0f)) {
-			post_transport_work = PostTransportWork (post_transport_work | PostTransportReverse);
+			todo = PostTransportWork (todo | PostTransportReverse);
 			last_stop_frame = _transport_frame;
 		}
 		
@@ -873,18 +933,20 @@ Session::set_transport_speed (float speed, bool abort)
 		boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
 		for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
 			if ((*i)->realtime_set_speed ((*i)->speed(), true)) {
-				post_transport_work = PostTransportWork (post_transport_work | PostTransportSpeed);
+				todo = PostTransportWork (todo | PostTransportSpeed);
+				break;
 			}
 		}
 		
-		if (post_transport_work & (PostTransportSpeed|PostTransportReverse)) {
+		if (todo) {
+			post_transport_work = PostTransportWork (post_transport_work | todo);
 			schedule_butler_transport_work ();
 		}
 	}
 }
 
 void
-Session::stop_transport (bool abort)
+Session::stop_transport (bool abort, bool clear_state)
 {
 	if (_transport_speed == 0.0f) {
 		return;
@@ -909,18 +971,19 @@ Session::stop_transport (bool abort)
 		merge_event (ev);
 		transport_sub_state |= StopPendingCapture;
 		pending_abort = abort;
+		pending_clear_substate = clear_state;
 		return;
 	} 
 
-
 	if ((transport_sub_state & PendingDeclickOut) == 0) {
 		transport_sub_state |= PendingDeclickOut;
 		/* we'll be called again after the declick */
 		pending_abort = abort;
+		pending_clear_substate = clear_state;
 		return;
 	}
 
-	realtime_stop (abort);
+	realtime_stop (abort, clear_state);
 	schedule_butler_transport_work ();
 }
 
@@ -1111,59 +1174,46 @@ Session::set_diskstream_speed (Diskstream* stream, float speed)
 }
 
 void
-Session::set_audio_range (list<AudioRange>& range)
+Session::unset_play_range ()
 {
-	Event *ev = new Event (Event::SetAudioRange, Event::Add, Event::Immediate, 0, 0.0f);
-	ev->audio_range = range;
-	queue_event (ev);
+	_play_range = false;
+	_clear_event_type (Event::RangeStop);
+	_clear_event_type (Event::RangeLocate);
 }
 
 void
-Session::request_play_range (bool yn)
+Session::set_play_range (list<AudioRange>& range, bool leave_rolling)
 {
-	Event* ev = new Event (Event::SetPlayRange, Event::Add, Event::Immediate, 0, 0.0f, yn);
-	queue_event (ev);
-}
+	Event* ev;
 
-void
-Session::set_play_range (bool yn)
-{
 	/* Called from event-processing context */
 
-	if (_play_range != yn) {
-		_play_range = yn;
-		setup_auto_play ();
-
-		if (!_play_range) {
+	unset_play_range ();
+	
+	if (range.empty()) {
+		/* _play_range set to false in unset_play_range()
+		 */
+		if (!leave_rolling) {
 			/* stop transport */
 			Event* ev = new Event (Event::SetTransportSpeed, Event::Add, Event::Immediate, 0, 0.0f, false);
 			merge_event (ev);
 		}
-	}
-}
-
-void
-Session::setup_auto_play ()
-{
-	/* Called from event-processing context */
-
-	Event* ev;
-	
-	_clear_event_type (Event::RangeStop);
-	_clear_event_type (Event::RangeLocate);
-
-	if (!_play_range) {
 		return;
 	}
 
-	list<AudioRange>::size_type sz = current_audio_range.size();
+	_play_range = true;
+
+	/* cancel loop play */
+	unset_play_loop ();
+
+	list<AudioRange>::size_type sz = range.size();
 	
 	if (sz > 1) {
 		
-		list<AudioRange>::iterator i = current_audio_range.begin(); 
+		list<AudioRange>::iterator i = range.begin(); 
 		list<AudioRange>::iterator next;
 		
-		while (i != current_audio_range.end()) {
+		while (i != range.end()) {
 			
 			next = i;
 			++next;
@@ -1179,7 +1229,7 @@ Session::setup_auto_play ()
 				requested_frame = 0;
 			}
 			
-			if (next == current_audio_range.end()) {
+			if (next == range.end()) {
 				ev = new Event (Event::RangeStop, Event::Add, requested_frame, 0, 0.0f);
 			} else {
 				ev = new Event (Event::RangeLocate, Event::Add, requested_frame, (*next).start, 0.0f);
@@ -1191,16 +1241,32 @@ Session::setup_auto_play ()
 		}
 		
 	} else if (sz == 1) {
-		
-		ev = new Event (Event::RangeStop, Event::Add, current_audio_range.front().end, 0, 0.0f);
+
+		ev = new Event (Event::RangeStop, Event::Add, range.front().end, 0, 0.0f);
 		merge_event (ev);
 		
 	} 
 
+	/* save range so we can do auto-return etc. */
+
+	current_audio_range = range;
+
 	/* now start rolling at the right place */
-	
-	ev = new Event (Event::LocateRoll, Event::Add, Event::Immediate, current_audio_range.front().start, 0.0f, false);
+
+	ev = new Event (Event::LocateRoll, Event::Add, Event::Immediate, range.front().start, 0.0f, false);
 	merge_event (ev);
+	
+	TransportStateChange ();
+}
+
+void
+Session::request_bounded_roll (nframes_t start, nframes_t end)
+{
+	AudioRange ar (start, end, 0);
+	list<AudioRange> lar;
+
+	lar.push_back (ar);
+	request_play_range (&lar, true);
 }
 
 void
@@ -1212,16 +1278,7 @@ Session::request_roll_at_and_return (nframes_t start, nframes_t return_to)
 }
 
 void
-Session::request_bounded_roll (nframes_t start, nframes_t end)
-{
-	request_stop ();
- 	Event *ev = new Event (Event::StopOnce, Event::Replace, end, Event::Immediate, 0.0);
-	queue_event (ev);
-	request_locate (start, true);
-}
-
-void
-Session::engine_halted ()
+Session::engine_halted (const char* /* reason */)
 {
 	bool ignored;
 
@@ -1236,7 +1293,7 @@ Session::engine_halted ()
 	post_transport_work = PostTransportWork (0);
 	stop_butler ();
 
-	realtime_stop (false);
+	realtime_stop (false, true);
 	non_realtime_stop (false, 0, ignored);
 	transport_sub_state = 0;
 
@@ -1260,7 +1317,7 @@ Session::xrun_recovery ()
 		   to handle things in the same way.
 		*/
 
-		engine_halted();
+		engine_halted ("");
 	} 
 }
 
@@ -1347,3 +1404,17 @@ Session::reset_jack_connection (jack_client_t* jack)
 		js->reset_client (jack);
 	}
 }
+
+bool
+Session::maybe_stop (nframes_t limit)
+{
+	if ((_transport_speed > 0.0f && _transport_frame >= limit) || (_transport_speed < 0.0f && _transport_frame == 0)) {
+		if (synced_to_jack () && Config->get_jack_time_master ()) {
+			_engine.transport_stop ();
+		} else if (!synced_to_jack ()) {
+			stop_transport ();
+		}
+		return true;
+	}
+	return false;
+}
