@@ -17,6 +17,7 @@
 
 */
 
+#include <strings.h>
 #include <map>
 #include <vector>
 
@@ -35,6 +36,8 @@ const char * const sndfile_header_formats_strings[SNDFILE_HEADER_FORMATS+1] = {
 	N_("AIFF"),
 	N_("CAF"),
 	N_("W64 (64 bit WAV)"),
+	N_("FLAC"),
+	N_("Ogg/Vorbis"),
 	N_("raw (no header)"),
 	0
 };
@@ -44,6 +47,8 @@ const char* const sndfile_file_endings_strings[SNDFILE_HEADER_FORMATS+1] = {
 	N_(".aiff"),
 	N_(".caf"),
 	N_(".w64"),
+	N_(".flac"),
+	N_(".ogg"),
 	N_(".raw"),
 	0
 };
@@ -53,6 +58,8 @@ int sndfile_header_formats[SNDFILE_HEADER_FORMATS] = {
 	SF_FORMAT_AIFF,
 	SF_FORMAT_CAF,
 	SF_FORMAT_W64,
+	SF_FORMAT_FLAC,
+	SF_FORMAT_OGG,
 	SF_FORMAT_RAW
 };
 
@@ -170,7 +177,16 @@ sndfile_major_format(int format)
 			format_info.format = i;
 			sf_command (0, SFC_GET_FORMAT_MAJOR, 
 					&format_info, sizeof (format_info));
-			m[format_info.format & SF_FORMAT_TYPEMASK] = format_info.name;
+
+			/* normalize a couple of names rather than use what libsndfile gives us */
+
+			if (strncasecmp (format_info.name, "OGG", 3) == 0) {
+				m[format_info.format & SF_FORMAT_TYPEMASK] = "Ogg";
+			} else if (strncasecmp (format_info.name, "WAV", 3) == 0) {
+				m[format_info.format & SF_FORMAT_TYPEMASK] = "WAV";
+			} else {
+				m[format_info.format & SF_FORMAT_TYPEMASK] = format_info.name;
+			}
 		}
 	}
 	
