@@ -179,7 +179,7 @@ SndFileSource::SndFileSource (Session& s, ustring path, SampleFormat sfmt, Heade
 		
 		/* coding history is added by libsndfile */
 
-		if (sf_command (sf, SFC_SET_BROADCAST_INFO, _broadcast_info, sizeof (_broadcast_info)) != SF_TRUE) {
+		if (sf_command (sf, SFC_SET_BROADCAST_INFO, _broadcast_info, sizeof (*_broadcast_info)) != SF_TRUE) {
 			char errbuf[256];
 			sf_error_str (0, errbuf, sizeof (errbuf) - 1);
 			error << string_compose (_("cannot set broadcast info for audio file %1 (%2); dropping broadcast info for this file"), _path, errbuf) << endmsg;
@@ -877,9 +877,15 @@ SndFileSource::get_soundfile_info (const ustring& path, SoundFileInfo& info, str
 	info.samplerate  = sf_info.samplerate;
 	info.channels    = sf_info.channels;
 	info.length      = sf_info.frames;
-	info.format_name = string_compose("%1\n%2",
-					   sndfile_major_format(sf_info.format),
-					   sndfile_minor_format(sf_info.format));
+
+	string major = sndfile_major_format(sf_info.format);
+	string minor = sndfile_minor_format(sf_info.format);
+
+	if (major.length() + minor.length() < 16) { /* arbitrary */
+		info.format_name = string_compose("%1/%2", major, minor);
+	} else {
+		info.format_name = string_compose("%1\n%2", major, minor);
+	}
 
 	memset (&binfo, 0, sizeof (binfo));
 	info.timecode  = get_timecode_info (sf, &binfo, timecode_exists);
