@@ -582,6 +582,14 @@ ARDOUR::auto_style_to_string (AutoStyle as)
 	return "";
 }
 
+bool
+string_is_affirmative (const std::string& str)
+{
+	/* to be used only with XML data - not intended to handle user input */
+
+	return str == "1" || str == "y" || str == "Y" || (!g_strncasecmp(str.c_str(), "yes", str.length()));
+}
+
 extern "C" {
 	void c_stacktrace() { stacktrace (cerr); }
 }
