@@ -530,3 +530,8 @@ VSTPluginInfo::load (Session& session)
 		return PluginPtr ((Plugin*) 0);
 	}
 }
+
+VSTPluginInfo::VSTPluginInfo()
+{
+	type = ARDOUR::VST;
+}
