@@ -3,7 +3,7 @@
 /*
     Rubber Band
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007 Chris Cannam.
+    Copyright 2007-2008 Chris Cannam.
     
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -16,14 +16,12 @@
 
 #include <cstdlib>
 #include <iostream>
+#include <cstdlib>
 
+#include <cstdlib>
 #include <sys/time.h>
 #include <time.h>
 
-//#define DEBUG_THREAD 1
-//#define DEBUG_MUTEX 1
-//#define DEBUG_CONDITION 1
-
 using std::cerr;
 using std::endl;
 using std::string;
@@ -108,8 +106,11 @@ Thread::staticRun(LPVOID arg)
     return 0;
 }
 
-Mutex::Mutex() :
-    m_locked(false)
+Mutex::Mutex()
+#ifndef NO_THREAD_CHECKS
+    :
+    m_lockedBy(-1)
+#endif
 {
     m_mutex = CreateMutex(NULL, FALSE, NULL);
 #ifdef DEBUG_MUTEX
@@ -128,50 +129,71 @@ Mutex::~Mutex()
 void
 Mutex::lock()
 {
-    if (m_locked) {
+#ifndef NO_THREAD_CHECKS
+    DWORD tid = GetCurrentThreadId();
+    if (m_lockedBy == tid) {
         cerr << "ERROR: Deadlock on mutex " << &m_mutex << endl;
     }
+#endif
 #ifdef DEBUG_MUTEX
-    cerr << "MUTEX DEBUG: " << (void *)GetCurrentThreadId() << ": Want to lock mutex " << &m_mutex << endl;
+    cerr << "MUTEX DEBUG: " << (void *)tid << ": Want to lock mutex " << &m_mutex << endl;
 #endif
     WaitForSingleObject(m_mutex, INFINITE);
-    m_locked = true;
+#ifndef NO_THREAD_CHECKS
+    m_lockedBy = tid;
+#endif
 #ifdef DEBUG_MUTEX
-    cerr << "MUTEX DEBUG: " << (void *)GetCurrentThreadId() << ": Locked mutex " << &m_mutex << endl;
+    cerr << "MUTEX DEBUG: " << (void *)tid << ": Locked mutex " << &m_mutex << endl;
 #endif
 }
 
 void
 Mutex::unlock()
 {
+#ifndef NO_THREAD_CHECKS
+    DWORD tid = GetCurrentThreadId();
+    if (m_lockedBy != tid) {
+        cerr << "ERROR: Mutex " << &m_mutex << " not owned by unlocking thread" << endl;
+        return;
+    }
+#endif
 #ifdef DEBUG_MUTEX
-    cerr << "MUTEX DEBUG: " << (void *)GetCurrentThreadId() << ": Unlocking mutex " << &m_mutex << endl;
+    cerr << "MUTEX DEBUG: " << (void *)tid << ": Unlocking mutex " << &m_mutex << endl;
+#endif
+#ifndef NO_THREAD_CHECKS
+    m_lockedBy = -1;
 #endif
-    m_locked = false;
     ReleaseMutex(m_mutex);
 }
 
 bool
 Mutex::trylock()
 {
+#ifndef NO_THREAD_CHECKS
+    DWORD tid = GetCurrentThreadId();
+#endif
     DWORD result = WaitForSingleObject(m_mutex, 0);
     if (result == WAIT_TIMEOUT || result == WAIT_FAILED) {
 #ifdef DEBUG_MUTEX
-        cerr << "MUTEX DEBUG: " << (void *)GetCurrentThreadId() << ": Mutex " << &m_mutex << " unavailable" << endl;
+        cerr << "MUTEX DEBUG: " << (void *)tid << ": Mutex " << &m_mutex << " unavailable" << endl;
 #endif
         return false;
     } else {
-        m_locked = true;
+#ifndef NO_THREAD_CHECKS
+        m_lockedBy = tid;
+#endif
 #ifdef DEBUG_MUTEX
-        cerr << "MUTEX DEBUG: " << (void *)GetCurrentThreadId() << ": Locked mutex " << &m_mutex << " (from trylock)" << endl;
+        cerr << "MUTEX DEBUG: " << (void *)tid << ": Locked mutex " << &m_mutex << " (from trylock)" << endl;
 #endif
         return true;
     }
 }
 
 Condition::Condition(string name) :
-    m_name(name),
     m_locked(false)
+#ifdef DEBUG_CONDITION
+    , m_name(name)
+#endif
 {
     m_mutex = CreateMutex(NULL, FALSE, NULL);
     m_condition = CreateEvent(NULL, FALSE, FALSE, NULL);
@@ -344,8 +366,12 @@ Thread::staticRun(void *arg)
     return 0;
 }
 
-Mutex::Mutex() :
+Mutex::Mutex()
+#ifndef NO_THREAD_CHECKS
+    :
+    m_lockedBy(0),
     m_locked(false)
+#endif
 {
     pthread_mutex_init(&m_mutex, 0);
 #ifdef DEBUG_MUTEX
@@ -364,49 +390,75 @@ Mutex::~Mutex()
 void
 Mutex::lock()
 {
-    if (m_locked) {
+#ifndef NO_THREAD_CHECKS
+    pthread_t tid = pthread_self();
+    if (m_locked && m_lockedBy == tid) {
         cerr << "ERROR: Deadlock on mutex " << &m_mutex << endl;
     }
+#endif
 #ifdef DEBUG_MUTEX
-    cerr << "MUTEX DEBUG: " << (void *)pthread_self() << ": Want to lock mutex " << &m_mutex << endl;
+    cerr << "MUTEX DEBUG: " << (void *)tid << ": Want to lock mutex " << &m_mutex << endl;
 #endif
     pthread_mutex_lock(&m_mutex);
+#ifndef NO_THREAD_CHECKS
+    m_lockedBy = tid;
     m_locked = true;
+#endif
 #ifdef DEBUG_MUTEX
-    cerr << "MUTEX DEBUG: " << (void *)pthread_self() << ": Locked mutex " << &m_mutex << endl;
+    cerr << "MUTEX DEBUG: " << (void *)tid << ": Locked mutex " << &m_mutex << endl;
 #endif
 }
 
 void
 Mutex::unlock()
 {
-#ifdef DEBUG_MUTEX
-    cerr << "MUTEX DEBUG: " << (void *)pthread_self() << ": Unlocking mutex " << &m_mutex << endl;
+#ifndef NO_THREAD_CHECKS
+    pthread_t tid = pthread_self();
+    if (!m_locked) {
+        cerr << "ERROR: Mutex " << &m_mutex << " not locked in unlock" << endl;
+        return;
+    } else if (m_lockedBy != tid) {
+        cerr << "ERROR: Mutex " << &m_mutex << " not owned by unlocking thread" << endl;
+        return;
+    }
 #endif
+#ifdef DEBUG_MUTEX
+    cerr << "MUTEX DEBUG: " << (void *)tid << ": Unlocking mutex " << &m_mutex << endl;
+#endif
+#ifndef NO_THREAD_CHECKS
     m_locked = false;
+#endif
     pthread_mutex_unlock(&m_mutex);
 }
 
 bool
 Mutex::trylock()
 {
+#ifndef NO_THREAD_CHECKS
+    pthread_t tid = pthread_self();
+#endif
     if (pthread_mutex_trylock(&m_mutex)) {
 #ifdef DEBUG_MUTEX
-        cerr << "MUTEX DEBUG: " << (void *)pthread_self() << ": Mutex " << &m_mutex << " unavailable" << endl;
+        cerr << "MUTEX DEBUG: " << (void *)tid << ": Mutex " << &m_mutex << " unavailable" << endl;
 #endif
         return false;
     } else {
+#ifndef NO_THREAD_CHECKS
+        m_lockedBy = tid;
         m_locked = true;
+#endif
 #ifdef DEBUG_MUTEX
-        cerr << "MUTEX DEBUG: " << (void *)pthread_self() << ": Locked mutex " << &m_mutex << " (from trylock)" << endl;
+        cerr << "MUTEX DEBUG: " << (void *)tid << ": Locked mutex " << &m_mutex << " (from trylock)" << endl;
 #endif
         return true;
     }
 }
 
 Condition::Condition(string name) :
-    m_locked(false),
-    m_name(name)
+    m_locked(false)
+#ifdef DEBUG_CONDITION
+    , m_name(name)
+#endif
 {
     pthread_mutex_init(&m_mutex, 0);
     pthread_cond_init(&m_condition, 0);
